/* This program does the clustering part of adaptive mesh refinement(AMR)
   on a structured grid.  It does this by using the Berger and Rigoutsos clustering
   algorithm.  The main implementation part of this algorithm is given
   in the function cluster in "cluster.c". 

   In this function, a temporary variable called "subgrid_temp" is used to generate 
   the data for the variable "subgrid" which is where the data is ultimately
   stored.  

                     Updated 4/15/07

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2007  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "amr.h"

int cluster ( INC_NODE *, SUBGRID *, int *, int *);

extern int nx, ny;

int adaptive( INC_NODE *node, SUBGRID *subgrid, int min_i, int max_i, int min_j, int max_j, int *num_subgrid_temp)
{

	int i, i2, j, j2, j3, k, ijk, node_num, dum, dum1, dum2, span_i, span_j;
	int zero_cross_i, zero_cross_j, grid_size;
	int check, counter;
	SUBGRID *subgrid_temp;
	double fdum;

/* For SUBGRID data */
	subgrid_temp=(SUBGRID *)calloc(10000,sizeof(SUBGRID));
	if(!subgrid_temp)
	{
		printf( " Problems with subgrid_temp\n");
		exit(1);
	}

	span_i = max_i - min_i;
	span_j = max_j - min_j;

	for( i = min_i; i < max_i; ++i)
	{
	    for( j = min_j; j < max_j; ++j)
	    {

		node_num = ny*i + j;

/* Initialize the first block by creating one block around all the nodes */

		if( node[node_num].refine_flag )
		{
		    if( min_i > node[node_num].i) min_i = node[node_num].i;
		    if( min_j > node[node_num].j) min_j = node[node_num].j;

		    if( max_i < node[node_num].i) max_i = node[node_num].i;
		    if( max_j < node[node_num].j) max_j = node[node_num].j;
		}
	    }
	}

	subgrid_temp[0].istart = min_i; subgrid_temp[0].iend = max_i;
	subgrid_temp[0].jstart = min_j; subgrid_temp[0].jend = max_j;
	zero_cross_i = max_i;
	zero_cross_j = max_j;

	dum = 0;
	counter = 0;
	for( i2 = 0; i2 < 300; ++i2)
	{
/* Generate subgrid_temp based on the above data. */

	    /*printf("xxxx %7d %7d %7d %7d %7d %7d\n", dum, subgrid_temp[dum].istart, subgrid_temp[dum].iend,
		subgrid_temp[dum].jstart, subgrid_temp[dum].jend, subgrid_temp[dum].stop_flag);*/
	    check = cluster( node, &subgrid_temp[dum], &zero_cross_i, &zero_cross_j);
	    if(!check) printf( " Problems with cluster\n");

	    if( subgrid_temp[dum].stop_flag )
	    {
/*
   Do not subdivide block, but store it in subgrid.  Then go to last block and
   divide it.

   Note: I may add a step later which loops over subgrid_temp to shrink it like in
   the first loop of cluster.
*/
		subgrid[counter].jstart = subgrid_temp[dum].jstart;
		subgrid[counter].jend = subgrid_temp[dum].jend;
		subgrid[counter].istart = subgrid_temp[dum].istart;
		subgrid[counter].iend = subgrid_temp[dum].iend;

		if( subgrid[counter].jstart == subgrid[counter].jend) subgrid[counter].jstart -= 1;
		if( subgrid[counter].istart == subgrid[counter].iend) subgrid[counter].istart -= 1;
		/*printf("mmmm %7d %7d %7d %7d %7d\n", subgrid[counter].istart, subgrid[counter].iend,
			subgrid[counter].jstart, subgrid[counter].jend, counter);
		printf("node zzzz %7d %7d %7d %7d\n",
			subgrid[counter].istart + span_i*subgrid[counter].jstart, 
			subgrid[counter].iend + span_i*subgrid[counter].jstart, 
			subgrid[counter].iend + span_i*subgrid[counter].jend,
			subgrid[counter].istart + span_i*subgrid[counter].jend);*/
	    	subgrid_temp[dum].stop_flag = 0;
		++counter;
		--dum;
	    }
	    else
	    {
/* Continue subdividing mesh.  In some ways, this is like the current block
   creating 2 child blocks, but I overwrite the parent block with one of the child
   blocks.  The child block which has overwritten the parent is NOT subdivided
   in the next iteration, but the other child may be subdivided.

   Note that most child blocks eventually become parent blocks until the entire
   process is finished. 
*/

		if( subgrid_temp[dum].jend == zero_cross_j)
		{
		    subgrid_temp[dum+1].istart = zero_cross_i;
		    subgrid_temp[dum+1].jstart = subgrid_temp[dum].jstart;
		    subgrid_temp[dum+1].iend = subgrid_temp[dum].iend;
		    subgrid_temp[dum+1].jend = subgrid_temp[dum].jend;

		    subgrid_temp[dum].iend = zero_cross_i - 1;
		    /*printf("crossing on x\n");*/
		}
		else
		{
		    subgrid_temp[dum+1].istart = subgrid_temp[dum].istart;
		    subgrid_temp[dum+1].jstart = zero_cross_j;
		    subgrid_temp[dum+1].iend = subgrid_temp[dum].iend;
		    subgrid_temp[dum+1].jend = subgrid_temp[dum].jend;

		    subgrid_temp[dum].jend = zero_cross_j - 1;
		    /*printf("crossing on y\n");*/
		}
		/*printf("pppp %7d %7d %7d %7d %7d\n", dum, subgrid_temp[dum].istart, subgrid_temp[dum].iend,
			subgrid_temp[dum].jstart, subgrid_temp[dum].jend);
		printf("pppp %7d %7d %7d %7d %7d\n", dum+1, subgrid_temp[dum+1].istart, subgrid_temp[dum+1].iend,
			subgrid_temp[dum+1].jstart, subgrid_temp[dum+1].jend);*/
		++dum;
	    }
	    if(dum < 0) break;
	}

	*num_subgrid_temp = counter;

	free(subgrid_temp);

	return 1;
}
