/* This program does adaptive mesh refinement(AMR) on a structured,
   uniform grid for finite difference programs.

                     Updated 4/10/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2007  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "amr.h"

#define SMALL               1.0e-20
#define CONJ_FLAG           0

int ConnectSurfwriter ();

int writer_SLFFEA_3D( BOUND , MATL *, INC_NODE *, SUBGRID *);

int writer_3D( BOUND , MATL *, char *, INC_NODE *);

int writer_SLFFEA_2D_AMR( BOUND , MATL *, INC_NODE *, SUBGRID *);

int writer_2D( BOUND , MATL *, char *, INC_NODE *);

int GridSet2( double *, double *, SUBGRID , SUBGRID  );

int GridSet1( double *, double *, SUBGRID , SUBGRID  );

int setgrid( SUBGRID * );

int coarseGridSet( double *, double *, SUBGRID , SUBGRID );

int refineGridSet( double *, double *, SUBGRID , SUBGRID );

int setSubgrid( SUBGRID * );

int sor( double *, double *, SUBGRID );

int PoisConjGrad( double *, double *, double *, SUBGRID );

int bcset( BOUND *, int *, INC_NODE * );

#if 0
int qikreader( MATL *, char *, INC_NODE *, FILE *, FLOWLINES , FLOWLINES );
#endif

int adaptive ( INC_NODE *, SUBGRID *, int , int , int , int , int *);

int cfdMemory( double **, int , int **, int , MATL **, int , XYZF **, int , XYZI **, int ,
	XYZI2 **, int , XY_ZX_YZ **, int , INC_NODE **, int, SUBGRID **, int );

int dof, nmat, numel, numnp, sof, nx, ny, nz, num_subgrid;
double dx, dy, dz;
int flag_2D = 0, flag_3D = 1, print_flag = 0;
int mach_read_flag;
double tolerance;
double dt, time_current;	 /* time step for time integration */
int nstep, cstep;
int imax, jmax;
int symmetric_flag;

int main (int argc, char **argv)
{

	int i, i2, j, j2, j3, k, ijk, node_num, node_num2, dum, dum1, dum2, istart, iend, jstart, jend;
	int min_i, max_i, min_j, max_j, zero_cross_i, zero_cross_j, grid_size;
	int ce, ce2, xm1, xm2, xp1, ym1, ym2, yp1, yp2, xp1yp1;
	int check, name_length, counter, MemoryCounter;
	INC_NODE *node, *mem_INC_NODE;
	SUBGRID subgrid0, subgrid0A, *subgrid1, *subgrid2, *subgrid3, *mem_SUBGRID;
	XYZI *mem_XYZI;
	XYZI2 *mem_XYZI2;
	XYZF *mem_XYZF;
	XY_ZX_YZ *mem_XY_ZX_YZ;
	int *mem_int, sofmi, sofmf, sofmXYZF, sofmXYZI, sofmXYZI2, sofmXY_ZX_YZ,
		sofmINC_NODE, sofmSUBGRID, ptr_inc;
	MATL *matl;
	double *mem_double;
	double *prs, *F, *G;
	double *prs0A, *F0A, *G0A, *prs1, *F1, *G1, *prs2, *F2, *G2, *prs3, *F3, *G3;
	double time, time_final;
	double fdum, fdum2, fdum3, fi, fj, fi2;
	char name[30], buf[ BUFSIZ ], *ccheck, one_char;
	FILE *o1, *o2;
	BOUND bc;
	double Qdot, kc, ac, bcc, pi, xc, yc;

	double gam, R;

	sof = sizeof(double);

	memset(name,0,30*sizeof(char));

	printf("What is the name of the file containing the \n");
	printf("finite difference CFD data? \n");
	scanf( "%30s",name);

/*   o1 contains all the structural data */

	o1 = fopen( name,"r" );
	if(o1 == NULL ) {
		printf("Can't find file %30s\n", name);
		exit(1);
	}

	nmat = 1;

	fgets( buf, BUFSIZ, o1 );
	fscanf( o1, "%d %d", &nx, &ny);

/* Check if there is additional coordinate z data and that this is a 3-D file.  */

	flag_3D = 0;
	flag_2D = 1;
	nz = 1;
	while(( one_char = (unsigned char) fgetc(o1)) != '\n')
	{
		if(one_char != ' ' )
		{
		    ungetc( one_char, o1);
		    fscanf( o1,"%d", &nz);
		    flag_3D = 1;
		    flag_2D = 0;
		    break;
		}
	}
	fscanf( o1,"\n");

	fgets( buf, BUFSIZ, o1 );
	fscanf( o1, "%lf %lf %lf\n", &dx, &dy, &dt);

	imax = nx;
	jmax = ny;
	numnp = nx*ny*nz;
	dof=numnp;

	print_flag = 0;
	printf("%d\n", argc);
	if(argc > 2)
	{
		printf("usage %s -p\n", name);
		printf("Too many arguments.\n");
		exit(1);
	}
	if(argc == 2 )
	{
	    if( !strncmp(argv[1], "-p", 2))    
	    {
		print_flag = 1;
	    }
	    else
	    {
		printf("usage %s -p only\n",name);
		exit(1);
	    }
	}


/*   Begin allocation of meomory */

	MemoryCounter = 0;

/* For the doubles */
	sofmf=15*numnp;
	MemoryCounter += sofmf*sizeof(double);
	printf( "\n Memory requrement for doubles is %15d bytes\n",MemoryCounter);

/* For the integers */
	sofmi= 2 + 4*numnp;
	MemoryCounter += sofmi*sizeof(int);
	printf( "\n Memory requrement for integers is %15d bytes\n",MemoryCounter);

/* For the XYZF doubles */
	sofmXYZF = 1;
	MemoryCounter += sofmXYZF*sizeof(XYZF);
	printf( "\n Memory requrement for XYZF doubles is %15d bytes\n",MemoryCounter);

/* For the XYZI integers */
	sofmXYZI = 1;
	MemoryCounter += sofmXYZI*sizeof(XYZI);
	printf( "\n Memory requrement for XYZI integers is %15d bytes\n",MemoryCounter);

/* For the XYZI2 integers */
	sofmXYZI2=4;
	MemoryCounter += sofmXYZI2*sizeof(XYZI2);
	printf( "\n Memory requrement for XYZI2 integers is %15d bytes\n",MemoryCounter);

/* For the XY_ZX_YZ integers */
	sofmXY_ZX_YZ=1;
	MemoryCounter += sofmXY_ZX_YZ*sizeof(XY_ZX_YZ);
	printf( "\n Memory requrement for XY_ZX_YZ integers is %15d bytes\n",MemoryCounter);

/* For the INC_NODE structs */
	sofmINC_NODE=numnp + 1;
	MemoryCounter += sofmINC_NODE*sizeof(INC_NODE);
	printf( "\n Memory requrement for INC_NODE integers is %15d bytes\n",MemoryCounter);

/* For the SUBGRID structs */
	sofmSUBGRID=6*numnp;
	MemoryCounter += sofmSUBGRID*sizeof(SUBGRID);
	printf( "\n Memory requrement for SUBGRID integers is %15d bytes\n",MemoryCounter);

	check = cfdMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat, &mem_XYZF, sofmXYZF,
		&mem_XYZI, sofmXYZI, &mem_XYZI2, sofmXYZI2, &mem_XY_ZX_YZ, sofmXY_ZX_YZ, &mem_INC_NODE,
		sofmINC_NODE, &mem_SUBGRID, sofmSUBGRID );
	if(!check) printf( " Problems with cfdMemory\n");

/* For the doubles */

	                                        ptr_inc=0;
	F =(mem_double+ptr_inc);                ptr_inc += numnp;
	G =(mem_double+ptr_inc);                ptr_inc += numnp;
	prs =(mem_double+ptr_inc);              ptr_inc += numnp;
	F0A =(mem_double+ptr_inc);              ptr_inc += numnp;
	G0A =(mem_double+ptr_inc);              ptr_inc += numnp;
	prs0A =(mem_double+ptr_inc);            ptr_inc += numnp;
	F1 =(mem_double+ptr_inc);               ptr_inc += numnp;
	G1 =(mem_double+ptr_inc);               ptr_inc += numnp;
	prs1 =(mem_double+ptr_inc);             ptr_inc += numnp;
	F2 =(mem_double+ptr_inc);               ptr_inc += numnp;
	G2 =(mem_double+ptr_inc);               ptr_inc += numnp;
	prs2 =(mem_double+ptr_inc);             ptr_inc += numnp;
	F3 =(mem_double+ptr_inc);               ptr_inc += numnp;
	G3 =(mem_double+ptr_inc);               ptr_inc += numnp;
	prs3 =(mem_double+ptr_inc);             ptr_inc += numnp;

/* For the INC_NODE structs*/
	                                      ptr_inc = 0;
	node=(mem_INC_NODE+ptr_inc);          ptr_inc += numnp;

/* For the integers */
	                                      ptr_inc = 0;
	bc.num_none=(mem_int+ptr_inc);        ptr_inc += 1;
	bc.num_prs0=(mem_int+ptr_inc);        ptr_inc += 1;

/* For the XYZI integers */
                                              ptr_inc = 0;
	bc.num_vel0=(mem_XYZI+ptr_inc);       ptr_inc += 1;

/* For the XYZI2 integers */
                                              ptr_inc = 0;
	bc.num_symm=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_far=(mem_XYZI2+ptr_inc);       ptr_inc += 1;
	bc.num_free=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_inflow=(mem_XYZI2+ptr_inc);    ptr_inc += 1;
	bc.num_frestr=(mem_XYZI2+ptr_inc);    ptr_inc += 1;
	bc.num_free2=(mem_XYZI2+ptr_inc);     ptr_inc += 1;
	bc.num_noslp=(mem_XYZI2+ptr_inc);     ptr_inc += 1;

/* For the SUBGRID structs*/
                                              ptr_inc = 0;
	subgrid1=(mem_SUBGRID+ptr_inc);       ptr_inc += 2*numnp;
	subgrid2=(mem_SUBGRID+ptr_inc);       ptr_inc += 2*numnp;
	subgrid3=(mem_SUBGRID+ptr_inc);       ptr_inc += 2*numnp;

	for( i = 0; i < 2*numnp; ++i) {
		subgrid1[i].refine_num = 1;
		subgrid2[i].refine_num = 2;
		subgrid3[i].refine_num = 3;
	}

	k = 0;

/* Generate some data points to test my code. */

	for( i = 0; i < nx; ++i)
	{
	    for( j = 0; j < ny; ++j)
	    {
		node_num = nz*ny*i + nz*j + k;
		node[node_num].refine_flag = 0;
		node[node_num].i = i;
		node[node_num].j = j;
		node[node_num].k = k;
		node[node_num].bc_type = 0;
		/*printf("%7d %7d %7d %7d %7d\n", node[node_num].i, node[node_num].j, node[node_num].k,
			node[node_num].refine_flag, node_num);*/
	    }
	}

#if 1
	dum = 0;
	for( i = 10; i < nx-5; ++i)
	{
		fdum = ((double)i)/5.0;
		j2 = (int)( 12.0 + 7.0*(1.0 + sin(fdum)));
		j3 = (int)( 11.0 + 3.0*(1.0 + sin(fdum)));
		for( j = j3; j < j2; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
#endif

#if 0
	dum = 0;
	for( i = 7; i < 20; ++i)
	{
		for( j = 10; j < 20; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
	for( i = 20; i < 30; ++i)
	{
		for( j = 7; j < 14; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
	for( i = 30; i < 60; ++i)
	{
		for( j = 14; j < 22; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
	for( i = 60; i < 90; ++i)
	{
		for( j = 18; j < 25; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
#endif

#if 0
	i = 10;
	for( j = 6; j < 14; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 11;
	for( j = 6; j < 14; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 12;
	for( j = 6; j < 14; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 13;
	for( j = 6; j < 9; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 14;
	for( j = 6; j < 10; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 15;
	for( j = 6; j < 10; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 16;
	for( j = 6; j < 9; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
	i = 17;
	for( j = 6; j < 9; ++j)
	{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
	}
#endif

#if 0
	for( i = 10; i < 15; ++i)
	{
		for( j = 10; j < 20; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
	for( i = 15; i < 20; ++i)
	{
		for( j = 10; j < 20; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
	for( i = 20; i < 30; ++i)
	{
		for( j = 20; j < 30; ++j)
		{
		    dum = nz*ny*i + nz*j + k;
		    node[dum].refine_flag = 1;
		    node[dum].bc_type = 1;
		}
	}
#endif

	min_i = 0;
	min_j = 0;
	max_i = nx - 1;
	max_j = ny - 1;

/* Below is a test of calculating the adaptive mesh refinement regions based on
   marked nodes.  I then write it out to be viewed by "qdpost".
*/ 

	check = adaptive ( node, subgrid1, min_i, max_i, min_j, max_j, &num_subgrid);
	if(!check) printf( " Problems with adaptive\n");

	if(flag_2D)
	{	
		check = writer_SLFFEA_2D_AMR(bc, matl, node, subgrid1);
		if(!check) printf( " Problems with writer_SLFFEA_2D\n");
	}

#if 0
	check = qikreader( matl, name, node, o1, path, streak );
	if(!check) printf( " Problems with qikreader\n");

	time_final = ((double)nstep)*dt;
#endif

	/*for( i = 0; i < nx; ++i) {
	   for( j = 0; j < ny; ++j) {
	     for( k = 0; k < nz; ++k) {
		node_num = ny*nz*i + nz*j + k;
		printf("%7d %7d %7d %7d %7d\n",
			node[node_num].i, node[node_num].j, node[node_num].k, node_num, node[node_num].bc_type);
	     }
	   }
	}*/

/* Set the boundary condition variable, bc, as well as allocate memory for each individual
   boundary condition
*/

	check = bcset( &bc, (mem_int+2), node);
	if(!check) printf( " Problems with bcset\n");

/*
	for( i = 0; i < bc.num_none[0]; ++i) {
		printf("%d\n", bc.none[i]);
	}
	for( i = 0; i < bc.num_symm[0].xp; ++i) {
		printf("%d\n", bc.symm.xp[i]);
	}
	for( i = 0; i < bc.num_symm[0].ym; ++i) {
		printf("%d\n", bc.symm.ym[i]);
	}
*/

	memset(F,0,numnp*sizeof(double));
	memset(G,0,numnp*sizeof(double));
	memset(prs,0,numnp*sizeof(double));

#if 0
	for( j = 0; j < jmax; ++j)
	{
	    i = imax - 1;
	    node_num = ny*i + j;
	    prs[node_num] = 100.0;
	}
#endif

	for( j = 0; j < jmax; ++j)
	{
	    for( i = 0; i < imax; ++i)
	    {
		node_num = ny*i + j;
		node[node_num].refine_flag = 0;
		node[node_num].bc_type = 0;

	    }
	}

#if 0
	for( i = 0; i < imax; ++i)
	{
	    j = 0;
	    node_num = ny*i + j;
	    F[node_num] = 100.0;
	    j = jmax - 1;
	    node_num = ny*i + j;
	    F[node_num] = 100.0;
	}

	for( j = 0; j < jmax; ++j)
	{
	    i = 0;
	    node_num = ny*i + j;
	    F[node_num] = 100.0;
	    i = imax - 1;
	    node_num = ny*i + j;
	    F[node_num] = 100.0;
	}
#endif

#if 1
	for( j = 1; j < jmax - 1; ++j)
	{
	    for( i = 1; i < imax - 1; ++i)
	    {
		node_num = ny*i + j;
		G[node_num] = 10.0;
	    }
	}
#endif

/* This is the calculation of the Poisson equation based on no refinement. */

	subgrid0.jstart = 0;
	subgrid0.jend = jmax-1;
	subgrid0.istart = 0;
	subgrid0.iend = imax-1;
	subgrid0.dx = dx;
	subgrid0.dy = dy;
	subgrid0.nx = nx;
	subgrid0.ny = ny;

	symmetric_flag = 1;

#if CONJ_FLAG
	check = PoisConjGrad( F, G, prs, subgrid0);
	if(!check) printf( " Problems with PoisConjGrad\n");
#else
	check = sor( G, prs, subgrid0);
	if(!check) printf( " Problems with sor\n");
#endif
	symmetric_flag = 0;

	for( i = 0; i < nx; ++i)
	{
		for( j = 0; j < ny; ++j)
		{
	    		node_num = ny*i + j;
			node[node_num].prs = prs[node_num];
			node[node_num].vx = F[node_num];
			node[node_num].vy = G[node_num];
			node[node_num].i = i;
			node[node_num].j = j;
		}
	}

	check = writer_2D(bc, matl, name, node);
	if(!check) printf( " Problems with writer_2D\n");

/* To make a test, I am taking some of the data from F and G from the previous
   calculation and applying it to a subgrid.
*/

	subgrid1[0].jstart = jmax - (jmax - jmax%2)/2;
	subgrid1[0].jend = jmax-2;
	subgrid1[0].istart = imax - (imax - imax%5)/5;
	subgrid1[0].iend = imax - (imax - imax%9)/9;

	for( j = subgrid1[0].jstart; j < subgrid1[0].jend + 2; ++j)
	{
		for( i = subgrid1[0].istart; i < subgrid1[0].iend + 2; ++i)
		{
	    		node_num = ny*i + j;
			printf("\n%5d %5d %16.8e", i, j, node[node_num].prs);
		}
	}

	for( j = subgrid1[0].jstart; j < subgrid1[0].jend; ++j)
	{
		for( i = subgrid1[0].istart; i < subgrid1[0].iend; ++i)
		{
	    		node_num = ny*i + j;
			node[node_num].refine_flag = 1;
			node[node_num].bc_type = 1;
		}
	}

	if(subgrid1[0].istart < 1)
	{
		printf( " Dimensions of subgrid1[0].istart are bad: %5d\n", subgrid1[0].istart);
		exit(1);
	}
	if(subgrid1[0].iend < 1)
	{
		printf( " Dimensions of subgrid1[0].iend are bad: %5d\n", subgrid1[0].iend);
		exit(1);
	}
	if(subgrid1[0].jstart < 1)
	{
		printf( " Dimensions of subgrid1[0].jstart are bad: %5d\n", subgrid1[0].jstart);
		exit(1);
	}
	if(subgrid1[0].jend < 1)
	{
		printf( " Dimensions of subgrid1[0].jend are bad: %5d\n", subgrid1[0].jend);
		exit(1);
	}

	check = setSubgrid( &subgrid1[0] );
	if(!check) printf( " Problems with setSubgrid\n");

#if 1
	check = refineGridSet( prs, prs1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = refineGridSet( F, F1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = refineGridSet( G, G1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with refineGridSet\n");
#endif

#if 0
	check = interp( prs, prs1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = interp( F, F1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = interp( G, G1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with refineGridSet\n");
#endif

/* I did a test below to see whether setting the internal nodes to zero would change the
   result.  It did not.
*/

	/*for( i = 1; i < subgrid1[0].nx-1; ++i)
	{
		for( j = 1; j < subgrid1[0].ny-1; ++j)
		{
			ce = subgrid1[0].ny*i + j;
			prs1[ce] = 0.0;
		}
	}*/

#if CONJ_FLAG
	check = PoisConjGrad( F1, G1, prs1, subgrid1[0]);
	if(!check) printf( " Problems with PoisConjGrad\n");
#else
	check = sor( G1, prs1, subgrid1[0]);
	if(!check) printf( " Problems with sor\n");
#endif

/* Before I write out the data to "writer_2D", I set the results of the subgrid1 calculation
   with prs1, F1, and G1 back to node, which is what is used by "writer_2D".
*/

	for( i = 0; i < subgrid1[0].nx; ++i)
	{
		for( j = 0; j < subgrid1[0].ny; ++j)
		{
	    		node_num = subgrid1[0].ny*i + j;
			node[node_num].prs = prs1[node_num];
			node[node_num].vx = F1[node_num];
			node[node_num].vy = G1[node_num];
			node[node_num].i = i;
			node[node_num].j = j;
		}
	}

	/*for( j = 0; j < subgrid1[0].ny; ++j)
	{
		for( i = 0; i < subgrid1[0].nx; ++i)
		{
	    		node_num = subgrid1[0].ny*i + j;
			printf("\n%5d %5d %16.8e", i+170, j+32, node[node_num].prs);
		}
	}*/

	nx = subgrid1[0].nx;
	ny = subgrid1[0].ny;

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	ccheck = strncpy(name + name_length, "1", 1);
	if(!ccheck) printf( " Problems with strncpy \n");

	check = writer_2D(bc, matl, name, node);
	if(!check) printf( " Problems with writer_2D\n");

/* To make another test, I am taking the data from F1 and G1 from the previous
   calculation and applying it to a subgrid.
*/

	subgrid2[0].jstart = 2;
	subgrid2[0].jend = subgrid1[0].ny- 2;
	subgrid2[0].istart = 2;
	subgrid2[0].iend = subgrid1[0].nx - 2;

	check = setSubgrid( &subgrid2[0] );
	if(!check) printf( " Problems with setSubgrid\n");

#if 1
	check = refineGridSet( prs1, prs2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = refineGridSet( F1, F2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = refineGridSet( G1, G2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with refineGridSet\n");
#endif

#if 0
	check = interp( prs1, prs2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = interp( F1, F2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with refineGridSet\n");

	check = interp( G1, G2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with refineGridSet\n");
#endif

/* I did a test below to see whether setting the internal nodes to zero would change the
   result.  It did not.
*/
	/*for( i = 1; i < subgrid2[0].nx-1; ++i)
	{
		for( j = 1; j < subgrid2[0].ny-1; ++j)
		{
			ce = subgrid2[0].ny*i + j;
			prs2[ce] = 0.0;
		}
	}*/

#if CONJ_FLAG
	check = PoisConjGrad( F2, G2, prs2, subgrid2[0]);
	if(!check) printf( " Problems with PoisConjGrad\n");
#else
	check = sor( G2, prs2, subgrid2[0]);
	if(!check) printf( " Problems with sor\n");
#endif

/* Before I write out the data to "writer_2D", I set the results of the subgrid2 calculation
   with prs2, F2, and G2 back to node, which is what is used by "writer_2D".
*/

	for( i = 0; i < subgrid2[0].nx; ++i)
	{
		for( j = 0; j < subgrid2[0].ny; ++j)
		{
			node_num = subgrid2[0].ny*i + j;
			node[node_num].prs = prs2[node_num];
			node[node_num].vx = F2[node_num];
			node[node_num].vy = G2[node_num];
			node[node_num].i = i;
			node[node_num].j = j;
		}
	}

	nx = subgrid2[0].nx;
	ny = subgrid2[0].ny;

	ccheck = strncpy(name + name_length, "2", 1);
	if(!ccheck) printf( " Problems with strncpy \n");

	check = writer_2D(bc, matl, name, node);
	if(!check) printf( " Problems with writer_2D\n");

/* For another test, I am creating a grid which has the same discretization as the coarsest grid.
   I will set its boundary values to that of the coarse grid.

   In theory, after doing the calculation over the region represented by subgridOA and then
   sending the results back to the main grid, the relsults should be the same as if I simply
   did the calculation over the main grid.
*/

	subgrid0A.jstart = 0;
	subgrid0A.jend = subgrid0.ny - 1;
	subgrid0A.istart = subgrid0.nx - 20;
	subgrid0A.iend = subgrid0.nx - 4;

	check = setgrid( &subgrid0A );
	if(!check) printf( " Problems with setgrid\n");
	
	check = GridSet1( prs, prs0A, subgrid0, subgrid0A );
	if(!check) printf( " Problems with coarseGridSet\n");

	check = GridSet1( F, F0A, subgrid0, subgrid0A );
	if(!check) printf( " Problems with coarseGridSet\n");

	check = GridSet1( G, G0A, subgrid0, subgrid0A );
	if(!check) printf( " Problems with coarseGridSet\n");

/* I did a test below to see whether setting the internal nodes to zero would change the
   result.  It did not.
*/
	/*for( i = 1; i < subgrid0A.nx-1; ++i)
	{
		for( j = 1; j < subgrid0A.ny-1; ++j)
		{
			ce = subgrid0A.ny*i + j;
			prs0A[ce] = 0.0;
		}
	}*/

#if CONJ_FLAG
	check = PoisConjGrad( F0A, G0A, prs0A, subgrid0A);
	if(!check) printf( " Problems with PoisConjGrad\n");
#else
	check = sor( G0A, prs0A, subgrid0A);
	if(!check) printf( " Problems with sor\n");
#endif

/* Before I write out the data to "writer_2D", I set the results of the subgridOA calculation
   with prsOA, FOA, and GOA back to node, which is what is used by "writer_2D".
*/

	for( i = 0; i < subgrid0A.nx; ++i)
	{
		for( j = 0; j < subgrid0A.ny; ++j)
		{
			node_num = subgrid0A.ny*i + j;
			node[node_num].prs = prs0A[node_num];
			node[node_num].vx = F0A[node_num];
			node[node_num].vy = G0A[node_num];
			node[node_num].i = i;
			node[node_num].j = j;
		}
	}

	nx = subgrid0A.nx;
	ny = subgrid0A.ny;

	ccheck = strncpy(name + name_length, "3", 1);
	if(!ccheck) printf( " Problems with strncpy \n");

	check = writer_2D(bc, matl, name, node);
	if(!check) printf( " Problems with writer_2D\n");


/* Now, I will pass the data from the refined grids up to the coarse grids. */

/* From the 2nd refinement to the first */

	nx = subgrid1[0].nx;
	ny = subgrid1[0].ny;

	check = coarseGridSet( prs1, prs2, subgrid1[0], subgrid2[0] );
	if(!check) printf( " Problems with coarseGridSet\n");

/* From the 1st refinement to the main grid */

	nx = imax;
	ny = jmax;

	check = coarseGridSet( prs, prs1, subgrid0, subgrid1[0] );
	if(!check) printf( " Problems with coarseGridSet\n");

/* From the test refinement to the main grid */

	nx = imax;
	ny = jmax;

/* Note to San: Edit out the lines below to see the refinements from subgrid1 and subgrid2
   sent back to prs rather than that of prs0A which is from the coarse grid test.
*/

	/*check = GridSet2( prs, prs0A, subgrid0, subgrid0A );
	if(!check) printf( " Problems with coarseGridSet\n");*/

/* Before I write out the data to "writer_2D", I set the results of the the F, G, and prs reverse
   refinement back to node, which is what is used by "writer_2D".  Ultimately, the output files
   to compare are:

     diff (file).ofd (file)cor.ofd
*/

	for( i = 0; i < nx; ++i)
	{
		for( j = 0; j < ny; ++j)
		{
			node_num = ny*i + j;
			node[node_num].prs = prs[node_num];
			node[node_num].vx = F[node_num];
			node[node_num].vy = G[node_num];
			node[node_num].i = i;
			node[node_num].j = j;
		}
	}

	ccheck = strncpy(name + name_length, "cor", 3);
	if(!ccheck) printf( " Problems with strncpy \n");

	check = writer_2D(bc, matl, name, node);
	if(!check) printf( " Problems with writer_2D\n");

	if(flag_3D)
	{	
#if 0
		check = writer_3D(bc, matl, name, node);
		if(!check) printf( " Problems with writer_3D\n");

		check = ConnectSurfwriter();
		if(!check) printf( " Problems with ConnectSurfwriter\n");

		check = writer_SLFFEA_3D(bc, matl, node, subgrid1);
		if(!check) printf( " Problems with writer_SLFFEA_3D\n");
#endif
	}

#if 0
	Qdot = 40.0;
	kc = 4.0;
	ac = nx*dx/2.0;
	bcc = ny*dy/2.0;
	pi = acos(-1.0);

	printf("\n hhhhh \n");
/*
	for( j = subgrid1[0].jstart; j < subgrid1[0].jend + 2; ++j)
	{
		for( i = subgrid1[0].istart; i < subgrid1[0].iend + 2; ++i)
		{
*/
	for( j = 0; j < ny; ++j)
	{
		for( i = 0; i < nx; ++i)
		{
		    fi = (double)i;
		    fj = (double)j;
		    xc = dx*i - ac;
		    yc = dy*j - bcc;
		    fdum3 = -1.0;
		    fdum2 = 0.0;
		    for( i2 = 0; i2 < 20; ++i2)
		    {
			fi2 = (double)i2;
			fdum = (2.0*i2 + 1.0)*(2.0*i2 + 1.0)*(2.0*i2 + 1.0)*cosh((2.0*i2 + 1.0)*pi*bcc/2.0/ac);
			fdum3 *= -1.0;
			fdum2 += cos((2.0*i2 + 1.0)*pi*xc/2.0/ac)*cosh((2.0*i2 + 1.0)*pi*yc/2.0/ac)/fdum;
			/*printf("\n\n  ggg %16.8e %16.8e %16.8e\n\n", fdum, fdum2, fdum3);*/
		    }
		    fdum = Qdot/kc*(ac*ac - xc*xc)/2.0 - 16.0*ac*ac*Qdot/(kc*pi*pi*pi)*fdum2;
		    /*printf("\n%5d %5d %16.8e %16.8e %16.8e", i, j, xc, yc, fdum);*/
		    node_num = ny*i + j;
		    node[node_num].prs = fdum;
		    node[node_num].vx = F[node_num];
		    node[node_num].vy = G[node_num];
		    node[node_num].i = i;
		    node[node_num].j = j;
		}
	}
	printf("\n\n");
#endif

#if 0

	nx = jmax;
	ny = imax;

	Qdot = 40.0;
	kc = 4.0;
	ac = nx*dx/2.0;
	bcc = ny*dy/2.0;
	pi = acos(-1.0);

/*
	printf("\n hhhhh \n");
	for( i = subgrid1[0].jstart; i < subgrid1[0].jend + 2; ++i)
	{
		for( j = subgrid1[0].istart; j < subgrid1[0].iend + 2; ++j)
		{
*/
	for( j = 0; j < ny; ++j)
	{
		for( i = 0; i < nx; ++i)
		{
		    fi = (double)i;
		    fj = (double)j;
		    xc = dx*fi - ac;
		    yc = dy*fj - bcc;
		    fdum3 = -1.0;
		    fdum2 = 0.0;
		    for( i2 = 0; i2 < 20; ++i2)
		    {
			fi2 = (double)i2;
			fdum = (2.0*fi2 + 1.0)*(2.0*fi2 + 1.0)*(2.0*fi2 + 1.0)*cosh((2.0*fi2 + 1.0)*pi*bcc/2.0/ac);
			fdum3 *= -1.0;
			fdum2 += cos((2.0*fi2 + 1.0)*pi*xc/2.0/ac)*cosh((2.0*fi2 + 1.0)*pi*yc/2.0/ac)/fdum;
			/*printf("\n\n  ggg %16.8e %16.8e %16.8e\n\n", fdum, fdum2, fdum3);*/
		    }
		    fdum = Qdot/kc*(ac*ac - xc*xc)/2.0 - 16.0*ac*ac*Qdot/(kc*pi*pi*pi)*fdum2;
		    /*printf("\n%5d %5d %16.8e %16.8e %16.8e", i, j, xc, yc, fdum);*/
		    node_num = i + nx*j;
		    node[node_num].prs = fdum;
		    node[node_num].vx = F[node_num];
		    node[node_num].vy = G[node_num];
		}
	}
	printf("\n\n");

	nx = imax;
	ny = jmax;
#endif

	ccheck = strncpy(name + name_length, "ex", 2);
	if(!ccheck) printf( " Problems with strncpy \n");

	check = writer_2D(bc, matl, name, node);
	if(!check) printf( " Problems with writer_2D\n");

	Qdot = 40.0;
	kc = 4.0;
	ac = 1.0/2.0;
	bcc = 1.5/2.0;
	pi = acos(-1.0);
	xc = 0.0;
	yc = 0.0;

	fdum3 = -1.0;
	fdum2 = 0.0;
	for( i2 = 0; i2 < 100; ++i2)
	{
		fdum = (2.0*i2 + 1.0)*(2.0*i2 + 1.0)*(2.0*i2 + 1.0)*cosh((2.0*i2 + 1.0)*pi*bcc/2.0/ac);
		fdum3 *= -1.0;
		fdum2 += cos((2.0*i2 + 1.0)*pi*xc/2.0/ac)*cosh((2.0*i2 + 1.0)*pi*yc/2.0/ac)/fdum;
		/*printf("\n\n%16.8e %16.8e %16.8e\n\n", fdum, fdum2, fdum3);*/
	}
	fdum = Qdot/kc*(ac*ac - xc*xc)/2.0 - 16.0*ac*ac*Qdot/(kc*pi*pi*pi)*fdum2;

	printf("\n\n%16.8e\n\n", fdum);

	free(matl);
	free(mem_double);
	free(mem_int);
	free(node);
	free(mem_SUBGRID);

	return 1;
}
