/* This program calculates the next values of F and G which are velocities before
   being pressure corrected.

   The algorithm for the code is based on the MAC(Marker and Cell) SCHEME.
   The conjugate gradient method is used to calculate the pressure corrector.
   The main references are:

   More is given on this method on page 337-338 of:

Fletcher, Clive A. J., Computational Techniques for Fluid Dynamics: Vol II, second
        edition, John Wiley & Sons Inc., New York, 1991.

                     Updated 9/29/07

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../compressible/flwconst.h"
#include "../compressible/flwstruct.h"
#include "../AMR/amr.h"

extern double Re;

int MarkerAndCell( double *F, double *G, double *uQ, double *vQ, SUBGRID subgrid)
{
	int i, j, k, dum, dum2, node_num;
	int nx, ny, istart, iend, jstart, jend;
	int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        double fdum;
	double usq1, usq2, vsq1, vsq2, uv1, uv2;
	double dx, dy, dt;

        dx = subgrid.dx;
        dy = subgrid.dy;
        dt = subgrid.dt;
        nx = subgrid.nx;
        ny = subgrid.ny;

        for( j = 1; j < ny-1; ++j )
        {
            for( i = 1; i < nx-1; ++i )
            {
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xp1 = ny*(i+1) + j;
                ym1 = ny*i + (j-1);
                yp1 = ny*i + (j+1);
                xm1ym1 = ny*(i-1) + j-1;
                xm1yp1 = ny*(i-1) + j+1;
                xp1ym1 = ny*(i+1) + j-1;


                usq1 = (uQ[xm1] + uQ[ce])/2.0;
                usq1 = usq1*usq1;
                usq2 = (uQ[ce] + uQ[xp1])/2.0;
                usq2 = usq2*usq2;
                uv1 = (uQ[ym1] + uQ[ce])*(vQ[xp1ym1] + vQ[ym1])/4.0;
                uv2 = (uQ[ce] + uQ[yp1])*(vQ[xp1] + vQ[ce])/4.0;

                F[ce] = uQ[ce] + dt*(uQ[xp1] - 2*uQ[ce] + uQ[xm1])/dx/dx/Re +
                     dt*(uQ[ym1] - 2*uQ[ce] + uQ[yp1])/dy/dy/Re -
                     dt*(usq2 - usq1)/dx - dt*(uv2 - uv1)/dy;

                vsq1 = (vQ[ym1] + vQ[ce])/2.0;
                vsq1 = vsq1*vsq1;
                vsq2 = (vQ[ce] + vQ[yp1])/2.0;
                vsq2 = vsq2*vsq2;
                uv1 = (uQ[xm1] + uQ[xm1yp1])*(vQ[ce] + vQ[xm1])/4.0;
                uv2 = (uQ[ce] + uQ[yp1])*(vQ[xp1] + vQ[ce])/4.0;

                G[ce] = vQ[ce] + dt*(vQ[xp1] - 2*vQ[ce] + vQ[xm1])/dx/dx/Re +
                     dt*(vQ[yp1] - 2*vQ[ce] + vQ[ym1])/dy/dy/Re -
                     dt*(vsq2 - vsq1)/dy - dt*(uv2 - uv1)/dx;

                /*F[ce] = uQ[ce];
                G[ce] = vQ[ce];*/

                /*printf(  "chchchch %4d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", i, j, ce, F[ce], uQ[ce], G[ce], vQ[ce]);*/
            }
        }

        return 1;
}


