/*  This program solves a linear system of equations using
    Cyclic Reduction rule as well as SOR.  It tests the
    code cycredux.c.

            Updated 5/9/06
 */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int n;

int ludcmp(double *, int , int *, double *, double *);
int lubksb(double *, int, int *, double *);

int Cyclic_Reduction1(double *, double *, double *, double *, int );

int Cyclic_Reduction(double *, double *, double *, double *, int , int );

int sor( double *, double *, double *);

double dx, dy, dz;
double dt, err, Re;
int imax, jmax;

/*int  reader(  double *A, double *B, FILE *oA, FILE *oB )*/
int  reader( double *B, FILE *oB )
{
	int i, j, dum;
	int ij;

/*
	for( i = 0; i < n; ++i )
	{
	   printf( "%i ", i);
	   for( j = 0; j < n; ++j )
	   {
		fscanf( oA, "%lf ",(A+n*i+j));
		printf( "%14.10lf ",*(A+n*i+j));
	   }
	   fscanf( oA,"\n");
	   printf( "\n");
	}
	printf( "\n");
	printf( "\n");
*/
	for( i = 0; i < n; ++i )
	{
		/*printf( "%i ", i);*/
		fscanf( oB, "%d %lf \n", &dum, (B+i));
		/*printf(  "%20.12e\n", *(B+i));*/
	}
	return 1;
}

int main()
{
	double *A, *D0, *F0, *B, *x, fdum;
	double *u2, *v2;
	int numnp, q, qsq, tester, dum;
	int check, *indx;
	int i,j;
	FILE *oA,*oB;
	double d, *vv;

	oA = fopen( "lua.in","r");
	oB = fopen( "lub.in","r");
	if(oA == NULL || oA == NULL ) {
		printf("error on open\n");
		exit(1);
	}

/*
	dx = 0.266;
	dy = 0.266;
	dt = 0.050;
*/
	dx = 1.0;
	dy = 1.0;
	dt = 1.0;

	Re = 1000.0; /* Reynolds number */

        imax = 68;
        jmax = 65;
	q = imax - 2;
	numnp = imax*jmax;

	/*fscanf( oA, "%i \n",&n);*/
	fscanf( oB, "%i \n",&n);
	printf( "%i \n",n);
	A=(double *)malloc(n*n*sizeof(double));
	if(!A) printf( "error allocating memory for A\n");
	B=(double *)malloc(numnp*sizeof(double));
	if(!B) printf( "error allocating memory for B\n");
	u2=(double *)malloc(numnp*sizeof(double));
	v2=(double *)malloc(numnp*sizeof(double));
	x=(double *)malloc(numnp*sizeof(double));
	if(!x) printf( "error allocating memory for x\n");
	vv=(double *)malloc(n*sizeof(double));
	if(!vv) printf( "error allocating memory for vv\n");
	indx=(int *)malloc(n*sizeof(int));
	if(!indx) printf( "error allocating memory for index\n");

	/*check=reader(A, B, oA, oB);*/
	check=reader( B, oB);
	if(!check) printf( "Problems with reader\n");

	printf("What test do you want to run?\n");
	printf("1) General Cyclic Reduction\n");
	printf("2) SOR\n");
	printf("3) Full Matrix\n");
	scanf( "%d",&tester);

	qsq = q*q;
	D0 = (double *)calloc(imax*imax,sizeof(double));
	F0 = (double *)calloc(imax*imax,sizeof(double));

	if(tester == 2)
	{
		printf( "\n\n");
		for( i = 1; i < imax-1; ++i )
		{
		   for( j = 1; j < jmax-1; ++j )
		   {
			*(u2 + jmax*i + j) = -*(B + (i-1) + (imax-2)*(j-1));
			*(v2 + jmax*i + j) = 0.0;
			/*printf( "%4d %4d %16.8e %16.8e\n", i, j, *(u2 + jmax*i + j), *(B + (jmax-2)*(i-1) + (j-1)));*/
		   }
		}

		err = 1.0e-15;

		printf( "\n\n");

		check = sor( u2, v2, x);
		if(!check) printf( "Problems with SOR\n");

		for( j = 50; j < 60; ++j )
		{
		    printf(  " %d ", j);
		    for( i = 50; i < 60; ++i )
		    {
			printf(  " %4d %9.5f", i, *(x + jmax*i + j));
		    }
		    printf(  "\n");
		}
	}

	if(tester == 1)
	{
/*
		printf( "\n\n");
		for( i = 0; i < q; ++i )
		{
		       for( j = 0; j < q; ++j )
		       {
			   *(D0 + q*i + j) = *(A + n*i + j);
			   *(F0 + q*i + j) = *(A + n*i + j + q);
			   printf( "%16.6e %16.6e\n", *(D0 + q*i + j), *(F0 + q*i + j));
		       }
		}
*/

		fdum = 1.0;
		*(D0 + 1) = -1.0/fdum;
		*(D0) = 4.0/fdum;
		*(F0) = -1.0/fdum;
		for( i = 1; i < imax-1; ++i )
		{
			*(D0 + (imax-2)*i + i + 1) = -1.0/fdum;
			*(D0 + (imax-2)*i + i) = 4.0/fdum;
			*(D0 + (imax-2)*i + i - 1) = -1.0/fdum;
			*(F0 + (imax-2)*i + i) = -1.0/fdum;
		}
		i = imax - 1;
		*(D0 + (imax-2)*i + i) = 4.0/fdum;
		*(D0 + (imax-2)*i + i - 1) = -1.0/fdum;
		*(F0 + (imax-2)*i + i) = -1.0/fdum;

		for( i = 0; i < n; ++i )
		{
			*(B+i) /= fdum;
		}

		dum = (jmax-2)*(imax-2);
		check = Cyclic_Reduction(D0, F0, B, x, dum, imax-2);
		if(!check) printf( "Problems with Cyclic_Reduction\n");

		for( j = 50; j < 60; ++j )
		{
		    printf(  " %d ", j);
		    for( i = 50; i < 60; ++i )
		    {
			printf(  " %4d %9.5f", i, *(x + (i-1) + (imax-2)*(j-1)));
		    }
		    printf(  "\n");
		}
	}

	if(tester == 3)
	{
		dum = (jmax-2)*(imax-2);
		*(A + 1) = -1.0;
		*(A) = 4.0;
		for( i = 1; i < dum-1; ++i )
		{
			*(A + dum*i + i + 1) = -1.0;
			*(A + dum*i + i) = 4.0;
			*(A + dum*i + i - 1) = -1.0;
		}
		i = dum - 1;
		*(A + dum*i + i) = 4.0;
		*(A + dum*i + i - 1) = -1.0;
		for( i = 0; i < dum-(imax-2); ++i )
		{
			*(A + dum*i + imax-2 + i) = -1.0;
			*(A + dum*(i + imax-2) + i) = -1.0;
		}

		check=ludcmp( A, n, indx, &d, vv);
		check=lubksb( A, n, indx, B);

		for( j = 50; j < 60; ++j )
		{
		    printf(  " %d ", j);
		    for( i = 50; i < 60; ++i )
		    {
			printf(  " %4d %9.5f", i, *(B + (i-1) + (imax-2)*(j-1)));
		    }
		    printf(  "\n");
		}
	}

	printf( "This is the Solution \n");
	/*for( j = 0; j < jmax; ++j )
	{
		for( i = 0; i < imax; ++i )
		{
			printf( "%5d %5d   %14.10e\n", i, j, *(x + jmax*i + j));
		}
	}*/


	free(A);
	free(B);
	free(x);

	return 1;
}
