/*
    This program contains the control mouse routine for the finite
    difference mesh.
  
                        Last Update 4/2/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/****** FEA globals ******/

extern int dof, nmat, numnp, numel;
extern double time_current, time_current_input, time_current_post;
extern int nstep, cstep, cstep_input, cstep_post;
extern int flag_2D, flag_3D, gr_optimize_flag;
extern int compressible_flag, incompressible_flag;
extern double *coord;
extern double *U;
extern int *connecter;
extern BOUND bc;
extern XYZF *velocity_vec, *velocity_vec0;
extern GR_NODE *node;

/****** For drawing the Mesh Window ******/
extern double coord_rescale;

/* Global variables for the mesh color and nodal data */

extern int *el_matl_color;
extern MATL *matl_crtl;

/****** EXTERNAL VARIABLES ********/

extern NORM *norm, *norm0;

extern int ControlDiv_y[rowdim + 2], ControlDiv_x[rowdim + 2];
extern int boxMove_x, boxMove_y, boxTextMove_x, textMove_x, textMove_y[rowdim+2];
extern int textDiv_xa, textDiv_xb;
extern double matl_choicef, node_choicef, ele_choicef, bound_choicef;

extern int ControlWindow, MeshWindow;
extern double step_sizex, step_sizey, step_sizez;
extern int control_height, control_width, mesh_height, mesh_width;
extern double xAngle, yAngle, zAngle;
extern double amplify_factor, amplify_step, amplify_step0;
extern double left_right, up_down, in_out;
extern double left_right0, up_down0, in_out0;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
extern int ortho_redraw_flag;

extern int input_flag, post_flag, color_choice, matl_choice, node_choice, ele_choice, bound_choice;
extern int input_color_flag;
extern int Solid_flag, Perspective_flag, Render_flag, Boundary_flag,
	VelocityVec_flag, Material_flag, Node_flag, Element_flag,
	Axes_flag, Outline_flag, Pathline_flag, Streakline_flag;
extern int Before_flag, After_flag, Both_flag, Time_flag; 
extern int nodal_quantities_flag;

extern GLfloat yellow[4], orange[4], orangeRed[4], red[4], green[4],
	violetRed[4], magenta[4], purple[4], blue[4],
	white[4], grey[4], black[4];

extern char RotateData[3][25];
extern char MoveData[3][25];
extern char StepData[25], TimeData[25];
extern char BoxData[2*boxnumber+2][25];
extern char BoxText[25];

extern int Color_flag[rowdim];
extern GR_NODE node_div0[boxnumber+1], node_div[boxnumber+1];

int fdnormal_vectors (int *, double *, NORM * );

void ScreenShot( int , int );

void cfd_ControlMouse(int button, int state, int x, int y)
{
	int i, check, dum1, dum2, dum3;
	double fpointx, fpointy, fpointz;
	GR_NODE node_div_box[boxnumber+1];

	memset(BoxText,0,25*sizeof(char));
	for( i = 0; i < 2*boxnumber+2; ++i)
	{
		memset(BoxData[i],0,25*sizeof(char));
	}

	if (button == GLUT_LEFT_BUTTON)
	{
		if ( x < textDiv_xa )
		{

/* These are for the View Option Keys */

			if ( y >= ControlDiv_y[3] && y < ControlDiv_y[4] )
			{
/* Solid Turned On */
				Solid_flag = 1;
			}
			if ( y >= ControlDiv_y[4] && y < ControlDiv_y[5] )
			{
/* Node ID Turned On increment up */
				VelocityVec_flag = 0;
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 1;
				node_choicef += .5;
				if ( node_choicef > 1.0 )
				{
					node_choicef = 0.0;
				}
				node_choice += (int)node_choicef;
				if ( node_choice > numnp - 1 )
				{
					node_choice = 0;
				}
				color_choice = 31;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[5] && y < ControlDiv_y[6] )
			{
/* Element ID Turned On increment up */
				VelocityVec_flag = 0;
				Boundary_flag = 0;
				Element_flag = 1;
				Material_flag = 0;
				Node_flag = 0;
				ele_choicef += .5;
				if ( ele_choicef > 1.0 )
				{
					ele_choicef = 0.0;
				}
				ele_choice += (int)ele_choicef;
				if ( ele_choice > numel - 1 )
				{
					ele_choice = 0;
				}
				color_choice = 32;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[6] && y < ControlDiv_y[7] )
			{
/* Material ID Turned On increment up */
				VelocityVec_flag = 0;
				Boundary_flag = 0;
				Element_flag = 0;
				Node_flag = 0;
				Material_flag = 1;
				matl_choicef += .5;
				if ( matl_choicef > 1.0 )
				{
					matl_choicef = 0.0;
				}
				matl_choice += (int)matl_choicef;
				if ( matl_choice > nmat - 1 )
				{
					matl_choice = 0;
				}
				color_choice = 30;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[7] && y < ControlDiv_y[8] )
			{
/* Boundary ID Turned On increment up */
				VelocityVec_flag = 0;
				Boundary_flag = 1;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				bound_choicef += .5;
				if ( bound_choicef > 1.0 )
				{
					bound_choicef = 0.0;
				}
				bound_choice += (int)bound_choicef;
				if ( bound_choice > numbc - 1 )
				{
					bound_choice = 0;
				}
				color_choice = 33;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[8] && y < ControlDiv_y[9] )
			{
/* Instantaneous Velocity Vectors Turned On */
				VelocityVec_flag = 1;
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				/*if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}*/
				/*Before_flag = 0;*/
			}
			if ( y >= ControlDiv_y[9] && y < ControlDiv_y[10] )
			{
/* Axes Turned On */
				Axes_flag = 1;
			}
/* These are for the Rotation Keys */
			if ( y >= ControlDiv_y[12] && y < ControlDiv_y[13] )
			{
/* Rotate -x */
				xAngle -= 5.0;
			}
			if ( y >= ControlDiv_y[13] && y < ControlDiv_y[14] )
			{
/* Rotate -y */
				yAngle -= 5.0;
			}
			if ( y >= ControlDiv_y[14] && y < ControlDiv_y[15] )
			{
/* Rotate -z */
				zAngle -= 5.0;
			}
			if ( y >= ControlDiv_y[15] && y < ControlDiv_y[16] )
			{
/* Reset Angles */
				xAngle = 0.0; yAngle = 0.0; zAngle = 0.0;
			}
/* These are for the Move Keys */
			if ( y >= ControlDiv_y[17] && y < ControlDiv_y[18] )
			{
/* Move -x */
				left_right -= step_sizex;
			}
			if ( y >= ControlDiv_y[18] && y < ControlDiv_y[19] )
			{
/* Move -y */
				up_down -= step_sizey;
			}
			if ( y >= ControlDiv_y[19] && y < ControlDiv_y[20] )
			{
/* Move -z */
				in_out -= step_sizez;
			}
			if ( y >= ControlDiv_y[20] && y < ControlDiv_y[21] )
			{
/* Reset Position */
				left_right = left_right0;
				up_down = up_down0;
				in_out = in_out0;

				ortho_right = ortho_right0;
				ortho_left = ortho_left0;
				ortho_top = ortho_top0;
				ortho_bottom = ortho_bottom0;
			}
/* These are for the Deformation Keys */
			if ( y >= ControlDiv_y[24] && y < ControlDiv_y[25] )
			{
/* Before Turned On */
				After_flag = 0;
				/*amplify_factor = 1.0;
				Time_flag = 0;*/
				Before_flag = 1;
				Both_flag = 0;
				if(Pathline_flag == 2) Pathline_flag = 1;
				if(Streakline_flag == 2) Streakline_flag = 1;
			}
			if ( y >= ControlDiv_y[25] && y < ControlDiv_y[26] )
			{
/* After Turned On */
				After_flag = 1;
				Before_flag = 0;
				Both_flag = 0;
				if(Pathline_flag == 1) Pathline_flag = 2;
				if(Streakline_flag == 1) Streakline_flag = 2;
			}
#if 0
			if ( y >= ControlDiv_y[26] && y < ControlDiv_y[27] )
			{
/* Both Before and After Turned On */
				After_flag = 1;
				/*VelocityVec_flag = 0;*/
				Both_flag = 1;
				Before_flag = 1;
				/* Boundary_flag = 0;*/
				/*Material_flag = 0;
				Solid_flag = 0;*/
			}
			if ( y >= ControlDiv_y[27] && y < ControlDiv_y[28] )
			{
/* Amplification increased */
			    if ( post_flag )
			    {
				After_flag = 1;
				amplify_step = amplify_step0;
				if( amplify_factor < 1.0 - SMALL2 )
					amplify_step = .1;
				amplify_factor += amplify_step;
				Time_flag = 1;
				/*VelocityVec_flag = 0;
				Boundary_flag = 0;*/
			    }
			}
#endif

/* These are for the Engineering Analysis Option Keys */
			if ( y >= ControlDiv_y[30] )
			{
				/*ScreenShot( control_width, control_height );*/
/* Nodal Quantities Turned On */
				/*if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}*/
				Both_flag = 0;
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 1;
/* Nodal Quantities */

/* Density */
				nodal_quantities_flag = 1;
				color_choice = 1;
			}
			if ( y >= ControlDiv_y[31] )
			{
/* Temperature */
				Boundary_flag = 0;
				color_choice = 2;
			}
			if ( y >= ControlDiv_y[32] )
			{
/* Pressure */
				Boundary_flag = 0;
				color_choice = 3;
			}
			if ( y >= ControlDiv_y[33] )
			{
/* Velocity X */
				Boundary_flag = 0;
				color_choice = 4;
			}
			if ( y >= ControlDiv_y[34] )
			{
/* Velocity Y */
				Boundary_flag = 0;
				color_choice = 5;
			}
			if ( y >= ControlDiv_y[35] )
			{
/* Velocity Z */
				Boundary_flag = 0;
				color_choice = 6;
			}
			if ( y >= ControlDiv_y[36] )
			{
/* Enthalpy */
				Boundary_flag = 0;
				color_choice = 7;
			}
			if ( y >= ControlDiv_y[37] )
			{
/* Energy */
				Boundary_flag = 0;
				color_choice = 8;
			}
			if ( y >= ControlDiv_y[38] )
			{
/* Mach Number or Velocity Magnitude*/
				Boundary_flag = 0;
				color_choice = 9;
			}
		}
		if ( x >= textDiv_xa && x < textDiv_xb )
		{
			if ( y >= ControlDiv_y[3] && y <= ControlDiv_y[4] )
			{
/* Solid Turned Off */
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 0;
				Outline_flag = 1;
			}
			if ( y >= ControlDiv_y[4] && y < ControlDiv_y[5] )
			{
/* Node ID Turned On increment down */
				VelocityVec_flag = 0;
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 1;
				node_choicef += .5;
				if ( node_choicef > 1.0 )
				{
					node_choicef = 0.0;
				}
				node_choice -= (int)node_choicef;
				if ( node_choice < 0 )
				{
					node_choice = numnp-1;
				}
				color_choice = 31;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[5] && y < ControlDiv_y[6] )
			{
/* Element ID Turned On increment down */
				/*ScreenShot( 350, 700);*/

				VelocityVec_flag = 0;
				Boundary_flag = 0;
				Element_flag = 1;
				Material_flag = 0;
				Node_flag = 0;
				ele_choicef += .5;
				if ( ele_choicef > 1.0 )
				{
					ele_choicef = 0.0;
				}
				ele_choice -= (int)ele_choicef;
				if ( ele_choice < 0 )
				{
					ele_choice = numel-1;
				}
				color_choice = 32;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[6] && y < ControlDiv_y[7] )
			{
/* Material ID Turned On increment down */
				VelocityVec_flag = 0;
				Boundary_flag = 0;
				Element_flag = 0;
				Node_flag = 0;
				Material_flag = 1;
				matl_choicef += .5;
				if ( matl_choicef > 1.0 )
				{
					matl_choicef = 0.0;
				}
				matl_choice -= (int)matl_choicef;
				if ( matl_choice < 0 )
				{
					matl_choice = nmat-1;
				}
				color_choice = 30;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[7] && y <= ControlDiv_y[8] )
			{
/* Boundary ID Turned On increment down */
				VelocityVec_flag = 0;
				Boundary_flag = 1;
				Element_flag = 0;
				Node_flag = 0;
				Material_flag = 0;
				bound_choicef += .5;
				if ( bound_choicef > 1.0 )
				{
					bound_choicef = 0.0;
				}
				bound_choice -= (int)bound_choicef;
				if ( bound_choice < 0 )
				{
					bound_choice = numbc-1;
				}
				color_choice = 33;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[8] && y < ControlDiv_y[9] )
			{
/* Instantaneous Velocity Vectors Turned Off */
				VelocityVec_flag = 0;
				Boundary_flag = 0;
			}
			if ( y >= ControlDiv_y[9] && y < ControlDiv_y[10] )
			{
/* Axes Turned Off */
				Axes_flag = 0;
			}
/* These are for the Rotation Keys */
			if ( y >= ControlDiv_y[12] && y < ControlDiv_y[13] )
			{
/* Rotate +x */
				xAngle += 5.0;
			}
			if ( y >= ControlDiv_y[13] && y < ControlDiv_y[14] )
			{
/* Rotate +y */
				yAngle += 5.0;
			}
			if ( y >= ControlDiv_y[14] && y < ControlDiv_y[15] )
			{
/* Rotate +z */
				zAngle += 5.0;
			}
/* These are for the Move Keys */
			if ( y >= ControlDiv_y[17] && y < ControlDiv_y[18] )
			{
/* Move +x */
				left_right += step_sizex;
			}
			if ( y >= ControlDiv_y[18] && y < ControlDiv_y[19] )
			{
/* Move +y */
				up_down += step_sizey;
			}
			if ( y >= ControlDiv_y[19] && y < ControlDiv_y[20] )
			{
/* Move +z */
				in_out += step_sizez;
			}

#if 0
/* These are for the Deformation Keys */
			if ( y >= ControlDiv_y[27] && y < ControlDiv_y[28] )
			{
/* Amplification decreased */
			    if( post_flag )
			    {
				After_flag = 1;
				amplify_step = amplify_step0;
				if( amplify_factor < 1.0 + amplify_step0 - SMALL2 )
					amplify_step = .1;
				amplify_factor -= amplify_step;
				/*Time_flag = 1;*/
				if ( amplify_factor < 0.0 )
				    amplify_factor = 0.0;
				/*printf("amplify factor %f %f\n", amplify_factor,amplify_step);*/
			    }
			}
#endif


/* The lines below are redundant and may not be needed.  I keep them here in case
   there are more quantities to look at
*/

/* These are for the Engineering Analysis Option Keys */
			if ( y >= ControlDiv_y[30] )
			{
				/*ScreenShot( control_width, control_height );*/
/* Nodal Quantities Turned On */
				/*if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}*/
				Both_flag = 0;
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 1;
/* Nodal Quantities */

/* Density */
				nodal_quantities_flag = 1;
				color_choice = 1;
			}
			if ( y >= ControlDiv_y[31] )
			{
/* Temperature */
				Boundary_flag = 0;
				color_choice = 2;
			}
			if ( y >= ControlDiv_y[32] )
			{
/* Pressure */
				Boundary_flag = 0;
				color_choice = 3;
			}
			if ( y >= ControlDiv_y[33] )
			{
/* Velocity X */
				Boundary_flag = 0;
				color_choice = 4;
			}
			if ( y >= ControlDiv_y[34] )
			{
/* Velocity Y */
				Boundary_flag = 0;
				color_choice = 5;
			}
			if ( y >= ControlDiv_y[35] )
			{
/* Velocity Z */
				Boundary_flag = 0;
				color_choice = 6;
			}
			if ( y >= ControlDiv_y[36] )
			{
/* Enthalpy */
				Boundary_flag = 0;
				color_choice = 7;
			}
			if ( y >= ControlDiv_y[37] )
			{
/* Energy */
				Boundary_flag = 0;
				color_choice = 8;
			}
			if ( y >= ControlDiv_y[38] )
			{
/* Mach Number or Velocity Magnitude*/
				Boundary_flag = 0;
				color_choice = 9;
			}

		}
	}

	sprintf( RotateData[0], "%8.2f ", xAngle);
	sprintf( RotateData[1], "%8.2f ", yAngle);
	sprintf( RotateData[2], "%8.2f ", zAngle);

	sprintf( MoveData[0], "%8.2f ", left_right);
	sprintf( MoveData[1], "%8.2f ", up_down);
	sprintf( MoveData[2], "%8.2f ", in_out);

	if( input_flag && Before_flag)
	{
		sprintf( StepData, "%10d", cstep_input);
		sprintf( TimeData, "%10.3e ", time_current_input);
	}
	if( post_flag && After_flag)
	{
		sprintf( StepData, "%10d", cstep_post);
		sprintf( TimeData, "%10.3e ", time_current_post);
	}

	for(i = 29; i < rowdim ; ++i)
	{
		Color_flag[i] = 0;
	}

	node_div_box[0] = node_div[0];
	node_div_box[1] = node_div[1];
	node_div_box[2] = node_div[2];
	node_div_box[3] = node_div[3];
	node_div_box[4] = node_div[4];
	node_div_box[5] = node_div[5];
	node_div_box[6] = node_div[6];
	node_div_box[7] = node_div[7];
	node_div_box[8] = node_div[8];

	if( Before_flag )
	{
	    node_div_box[0] = node_div0[0];
	    node_div_box[1] = node_div0[1];
	    node_div_box[2] = node_div0[2];
	    node_div_box[3] = node_div0[3];
	    node_div_box[4] = node_div0[4];
	    node_div_box[5] = node_div0[5];
	    node_div_box[6] = node_div0[6];
	    node_div_box[7] = node_div0[7];
	    node_div_box[8] = node_div0[8];
	}

	switch (color_choice) {
		case 1:
			if(compressible_flag)
			{
			    strncpy(BoxText, "density", 7);
			    Color_flag[29] = 1;
			    nodal_quantities_flag = 1;
			    sprintf( BoxData[0], "%10.3e ", node_div_box[8].dns);
			    sprintf( BoxData[2], "%10.3e ", node_div_box[7].dns);
			    sprintf( BoxData[4], "%10.3e ", node_div_box[6].dns);
			    sprintf( BoxData[6], "%10.3e ", node_div_box[5].dns);
			    sprintf( BoxData[8], "%10.3e ", node_div_box[4].dns);
			    sprintf( BoxData[10], "%10.3e ", node_div_box[3].dns);
			    sprintf( BoxData[12], "%10.3e ", node_div_box[2].dns);
			    sprintf( BoxData[14], "%10.3e ", node_div_box[1].dns);
			    sprintf( BoxData[16], "%10.3e ", node_div_box[0].dns);
			}
		break;
		case 2:
			if(compressible_flag)
			{
			    strncpy(BoxText, "temperature", 11);
			    Color_flag[30] = 1;
			    nodal_quantities_flag = 1;
			    sprintf( BoxData[0], "%10.3e ", node_div_box[8].temp);
			    sprintf( BoxData[2], "%10.3e ", node_div_box[7].temp);
			    sprintf( BoxData[4], "%10.3e ", node_div_box[6].temp);
			    sprintf( BoxData[6], "%10.3e ", node_div_box[5].temp);
			    sprintf( BoxData[8], "%10.3e ", node_div_box[4].temp);
			    sprintf( BoxData[10], "%10.3e ", node_div_box[3].temp);
			    sprintf( BoxData[12], "%10.3e ", node_div_box[2].temp);
			    sprintf( BoxData[14], "%10.3e ", node_div_box[1].temp);
			    sprintf( BoxData[16], "%10.3e ", node_div_box[0].temp);
			}
		break;
		case 3:
			strncpy(BoxText, "pressure", 8);
			Color_flag[31] = 1;
			nodal_quantities_flag = 1;
			sprintf( BoxData[0], "%10.3e ", node_div_box[8].prs);
			sprintf( BoxData[2], "%10.3e ", node_div_box[7].prs);
			sprintf( BoxData[4], "%10.3e ", node_div_box[6].prs);
			sprintf( BoxData[6], "%10.3e ", node_div_box[5].prs);
			sprintf( BoxData[8], "%10.3e ", node_div_box[4].prs);
			sprintf( BoxData[10], "%10.3e ", node_div_box[3].prs);
			sprintf( BoxData[12], "%10.3e ", node_div_box[2].prs);
			sprintf( BoxData[14], "%10.3e ", node_div_box[1].prs);
			sprintf( BoxData[16], "%10.3e ", node_div_box[0].prs);
		break;
		case 4:
			strncpy(BoxText, "velocity x", 10);
			Color_flag[32] = 1;
			nodal_quantities_flag = 1;
			sprintf( BoxData[0], "%10.3e ", node_div_box[8].vx);
			sprintf( BoxData[2], "%10.3e ", node_div_box[7].vx);
			sprintf( BoxData[4], "%10.3e ", node_div_box[6].vx);
			sprintf( BoxData[6], "%10.3e ", node_div_box[5].vx);
			sprintf( BoxData[8], "%10.3e ", node_div_box[4].vx);
			sprintf( BoxData[10], "%10.3e ", node_div_box[3].vx);
			sprintf( BoxData[12], "%10.3e ", node_div_box[2].vx);
			sprintf( BoxData[14], "%10.3e ", node_div_box[1].vx);
			sprintf( BoxData[16], "%10.3e ", node_div_box[0].vx);
		break;
		case 5:
			strncpy(BoxText, "velocity y", 10);
			Color_flag[33] = 1;
			nodal_quantities_flag = 1;
			sprintf( BoxData[0], "%10.3e ", node_div_box[8].vy);
			sprintf( BoxData[2], "%10.3e ", node_div_box[7].vy);
			sprintf( BoxData[4], "%10.3e ", node_div_box[6].vy);
			sprintf( BoxData[6], "%10.3e ", node_div_box[5].vy);
			sprintf( BoxData[8], "%10.3e ", node_div_box[4].vy);
			sprintf( BoxData[10], "%10.3e ", node_div_box[3].vy);
			sprintf( BoxData[12], "%10.3e ", node_div_box[2].vy);
			sprintf( BoxData[14], "%10.3e ", node_div_box[1].vy);
			sprintf( BoxData[16], "%10.3e ", node_div_box[0].vy);
		break;
		case 6:
			strncpy(BoxText, "velocity z", 10);
			Color_flag[34] = 1;
			nodal_quantities_flag = 1;
			sprintf( BoxData[0], "%10.3e ", node_div_box[8].vz);
			sprintf( BoxData[2], "%10.3e ", node_div_box[7].vz);
			sprintf( BoxData[4], "%10.3e ", node_div_box[6].vz);
			sprintf( BoxData[6], "%10.3e ", node_div_box[5].vz);
			sprintf( BoxData[8], "%10.3e ", node_div_box[4].vz);
			sprintf( BoxData[10], "%10.3e ", node_div_box[3].vz);
			sprintf( BoxData[12], "%10.3e ", node_div_box[2].vz);
			sprintf( BoxData[14], "%10.3e ", node_div_box[1].vz);
			sprintf( BoxData[16], "%10.3e ", node_div_box[0].vz);
		break;
		case 7:
			if(compressible_flag)
			{
			    strncpy(BoxText, "enthalpy", 8);
			    Color_flag[35] = 1;
			    nodal_quantities_flag = 1;
			    sprintf( BoxData[0], "%10.3e ", node_div_box[8].enthal);
			    sprintf( BoxData[2], "%10.3e ", node_div_box[7].enthal);
			    sprintf( BoxData[4], "%10.3e ", node_div_box[6].enthal);
			    sprintf( BoxData[6], "%10.3e ", node_div_box[5].enthal);
			    sprintf( BoxData[8], "%10.3e ", node_div_box[4].enthal);
			    sprintf( BoxData[10], "%10.3e ", node_div_box[3].enthal);
			    sprintf( BoxData[12], "%10.3e ", node_div_box[2].enthal);
			    sprintf( BoxData[14], "%10.3e ", node_div_box[1].enthal);
			    sprintf( BoxData[16], "%10.3e ", node_div_box[0].enthal);
			}
		break;
		case 8:
			if(compressible_flag)
			{
			    strncpy(BoxText, "energy", 6);
			    Color_flag[36] = 1;
			    nodal_quantities_flag = 1;
			    sprintf( BoxData[0], "%10.3e ", node_div_box[8].ene);
			    sprintf( BoxData[2], "%10.3e ", node_div_box[7].ene);
			    sprintf( BoxData[4], "%10.3e ", node_div_box[6].ene);
			    sprintf( BoxData[6], "%10.3e ", node_div_box[5].ene);
			    sprintf( BoxData[8], "%10.3e ", node_div_box[4].ene);
			    sprintf( BoxData[10], "%10.3e ", node_div_box[3].ene);
			    sprintf( BoxData[12], "%10.3e ", node_div_box[2].ene);
			    sprintf( BoxData[14], "%10.3e ", node_div_box[1].ene);
			    sprintf( BoxData[16], "%10.3e ", node_div_box[0].ene);
			}
		break;
		case 9:
			if(compressible_flag)
			{
			    strncpy(BoxText, "mach number", 11);
			    Color_flag[37] = 1;
			    nodal_quantities_flag = 1;
			    sprintf( BoxData[0], "%10.3e ", node_div_box[8].mach);
			    sprintf( BoxData[2], "%10.3e ", node_div_box[7].mach);
			    sprintf( BoxData[4], "%10.3e ", node_div_box[6].mach);
			    sprintf( BoxData[6], "%10.3e ", node_div_box[5].mach);
			    sprintf( BoxData[8], "%10.3e ", node_div_box[4].mach);
			    sprintf( BoxData[10], "%10.3e ", node_div_box[3].mach);
			    sprintf( BoxData[12], "%10.3e ", node_div_box[2].mach);
			    sprintf( BoxData[14], "%10.3e ", node_div_box[1].mach);
			    sprintf( BoxData[16], "%10.3e ", node_div_box[0].mach);
			}
			if(incompressible_flag)
			{
			    strncpy(BoxText, "velocity mag", 12);
			    Color_flag[37] = 1;
			    nodal_quantities_flag = 1;
			    sprintf( BoxData[0], "%10.3e ", node_div_box[8].vmag);
			    sprintf( BoxData[2], "%10.3e ", node_div_box[7].vmag);
			    sprintf( BoxData[4], "%10.3e ", node_div_box[6].vmag);
			    sprintf( BoxData[6], "%10.3e ", node_div_box[5].vmag);
			    sprintf( BoxData[8], "%10.3e ", node_div_box[4].vmag);
			    sprintf( BoxData[10], "%10.3e ", node_div_box[3].vmag);
			    sprintf( BoxData[12], "%10.3e ", node_div_box[2].vmag);
			    sprintf( BoxData[14], "%10.3e ", node_div_box[1].vmag);
			    sprintf( BoxData[16], "%10.3e ", node_div_box[0].vmag);
			}
		break;
		case 30:
			strncpy(BoxText, "Material", 8);
			Color_flag[5] = 1;
			input_color_flag = 0;
			nodal_quantities_flag = 0;
			Boundary_flag = 0;
			Element_flag = 0;
			Node_flag = 0;
			Material_flag = 1;
			if(compressible_flag)
			{
			    sprintf( BoxData[0], "%4d ", matl_choice);
			    strncpy( BoxData[1], "Ideal Gas", 9);
			    strncpy( BoxData[2], " Constant", 9);
			    sprintf( BoxData[3], "%10.3e ", matl_crtl[matl_choice].R);
			    strncpy( BoxData[4], "Spec. Heat", 10);
			    strncpy( BoxData[5], "  Ratio", 7);
			    sprintf( BoxData[6], "%10.3e ", matl_crtl[matl_choice].gam);
			}
			if(incompressible_flag)
			{
			    sprintf( BoxData[0], "%4d ", matl_choice);
			    strncpy( BoxData[1], "Reynods", 7);
			    strncpy( BoxData[2], "Number", 6);
			    sprintf( BoxData[3], "%10.3e ", matl_crtl[matl_choice].Re);
			}
			sprintf( BoxData[10], " " );
			sprintf( BoxData[12], " " );
			sprintf( BoxData[14], " " );
			sprintf( BoxData[16], " " );
		break;
		case 31:
			strncpy(BoxText, "Node", 4);
			Color_flag[3] = 1;
			input_color_flag = 0;
			nodal_quantities_flag = 0;
			Boundary_flag = 0;
			Element_flag = 0;
			Material_flag = 0;
			Node_flag = 1;
			fpointx = *(coord + nsd*node_choice)*coord_rescale;
			fpointy = *(coord + nsd*node_choice + 1)*coord_rescale;
			fpointz = *(coord + nsd*node_choice + 2)*coord_rescale;
			sprintf( BoxData[0], "%4d", node_choice);
			strncpy( BoxData[2], "coord x", 7);
			sprintf( BoxData[4], "%10.3e ", fpointx);
			strncpy( BoxData[6], "coord y", 7);
			sprintf( BoxData[8], "%10.3e ", fpointy);
			strncpy( BoxData[10], "coord z", 7);
			sprintf( BoxData[12], "%10.3e ", fpointz);
			strncpy( BoxData[14], "  I   J   K", 11);
			dum1 = node[node_choice].i;
			dum2 = node[node_choice].j;
			dum3 = node[node_choice].k;
			sprintf( BoxData[15], "%3d %3d %3d", dum1, dum2, dum3);
			sprintf( BoxData[16], " " );
		break;
		case 32:
			strncpy(BoxText, "Element", 7);
			Color_flag[4] = 1;
			input_color_flag = 0;
			nodal_quantities_flag = 0;
			Boundary_flag = 0;
			Material_flag = 0;
			Node_flag = 0;
			Element_flag = 1;
			sprintf( BoxData[0], "%4d ", ele_choice);
			strncpy( BoxData[2], "Material", 8);
			sprintf( BoxData[4], "%4d ", *(el_matl_color+ele_choice));
			strncpy( BoxData[6], "Connect", 7);
			if(flag_3D && !gr_optimize_flag)
			{
				dum1 = *(connecter + npel*ele_choice);
				dum2 = *(connecter + npel*ele_choice+1);
				sprintf( BoxData[8], "%4d,%4d ",dum1, dum2);
				dum1 = *(connecter + npel*ele_choice+2);
				dum2 = *(connecter + npel*ele_choice+3);
				sprintf( BoxData[10], "%4d,%4d ",dum1, dum2);
				dum1 = *(connecter + npel*ele_choice+4);
				dum2 = *(connecter + npel*ele_choice+5);
				sprintf( BoxData[12], "%4d,%4d ",dum1, dum2);
				dum1 = *(connecter + npel*ele_choice+6);
				dum2 = *(connecter + npel*ele_choice+7);
				sprintf( BoxData[14], "%4d,%4d ",dum1, dum2);
			}
			else
			{
				dum1 = *(connecter + npel2*ele_choice);
				dum2 = *(connecter + npel2*ele_choice+1);
				sprintf( BoxData[8], "%4d,%4d ",dum1, dum2);
				dum1 = *(connecter + npel2*ele_choice+2);
				dum2 = *(connecter + npel2*ele_choice+3);
				sprintf( BoxData[10], "%4d,%4d ",dum1, dum2);
				sprintf( BoxData[12], " ");
				sprintf( BoxData[14], " ");
			}
			sprintf( BoxData[16], " ");
		break;
		case 33:
			strncpy(BoxText, "Boundary", 8);
			Color_flag[6] = 1;
			input_color_flag = 0;
			nodal_quantities_flag = 0;
			Boundary_flag = 1;
			Element_flag = 0;
			Node_flag = 0;
			sprintf( BoxData[0], " ");
			sprintf( BoxData[1], " ");

			dum1 = bound_choice;
			switch (dum1) {
			   case 1:
				strncpy( BoxData[2], "symmetric", 9);
			   break;
			   case 2:
				/*strncpy( BoxData[2], "symmetric I", 11);*/
				strncpy( BoxData[2], "   none", 7);
			   break;
			   case 3:
				strncpy( BoxData[2], "far field", 9);
			   break;
			   case 4:
				strncpy( BoxData[2], "   free", 7);
			   break;
			   case 5:
				strncpy( BoxData[2], " inflow", 7);
			   break;
			   case 6:
				strncpy( BoxData[2], "freestream", 10);
			   break;
			   case 7:
				strncpy( BoxData[2], "  free2", 7);
			   break;
			   case 8:
				strncpy( BoxData[2], "no slip", 7);
			   break;
			   case 9:
				strncpy( BoxData[2], "0 pressure", 10);
			   break;
			   default:
				strncpy( BoxData[2], "   none", 7);
			   break;
			}

			sprintf( BoxData[3], " ");
			sprintf( BoxData[4], " ");
			sprintf( BoxData[5], " ");
			sprintf( BoxData[6], " ");
			sprintf( BoxData[10], " " );
			sprintf( BoxData[12], " " );
			sprintf( BoxData[14], " " );
			sprintf( BoxData[16], " " );
		break;
	}

	input_color_flag = 0;
	if( VelocityVec_flag )
	{
		if( color_choice > 29) input_color_flag = 1;
		Boundary_flag = 0;
		Element_flag = 0;
		Material_flag = 0;
		Node_flag = 0;
	}

	Color_flag[2] = Solid_flag;
/*
	Color_flag[3] = Perspective_flag;
	Color_flag[4] = Render_flag;
*/
	Color_flag[3] = Node_flag;
	Color_flag[4] = Element_flag;
	Color_flag[5] = Material_flag;
	Color_flag[6] = Boundary_flag;
	Color_flag[7] = VelocityVec_flag;
	Color_flag[8] = Axes_flag;

	Color_flag[23] = Before_flag;
	Color_flag[24] = After_flag;
	Color_flag[25] = Both_flag;
	Color_flag[26] = Time_flag;

	if(!post_flag) After_flag = 0;
	if(!input_flag) Before_flag = 0;

	glutPostWindowRedisplay(ControlWindow);
	glutPostWindowRedisplay(MeshWindow);
}

