/*
    This program contains the mesh display routine for the finite
    difference mesh.
  
                  Last Update 6/29/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

void cfd_qdmeshdraw(void);
void cfd_brmeshdraw(void);
void cfd_qdrender(void);
void cfd_brrender(void);
void velocity_vectors(BOUND , double *, XYZF *);

extern int flag_2D, flag_3D;
extern double *coord;
extern BOUND bc;
extern BODY *body;
extern FLOWLINES path0, streak0, stream0;


extern int anim_flag;
extern int nstep, step_counter, anim_counter, save_anim_counter;

extern double left_right, up_down, in_out, left_right0,
	up_down0, in_out0, xAngle, yAngle, zAngle;
extern int bound_choice, color_choice, gr_optimize_flag;
extern ANIM_DAT anim[1];
extern int app_type_choice;
extern GLuint AxesList, BodyList, BoundList[10], VelocityList0, VelocityList,   /* Display lists */
	PathlineList0, PathlineList, StreaklineList0, StreaklineList,
	StreamlineList0, StreamlineList;
extern XYZF *velocity_vec;
extern int Render_flag, Boundary_flag, AppliedDisp_flag, VelocityVec_flag,
	Pathline_flag, Streakline_flag, Streamline_flag, Body_flag, Axes_flag,
	Before_flag, After_flag; 
extern int CrossSection_flag;

void bodies_in_flow3( BODY * );

void MeshAnimate(void);

void velocity_vectors_anim( double *coord, XYZF * );

void flowline_points_anim( FLOWLINES );

void AxesNumbers2(void);

void AxesNumbers(void);

void AxesLabel(void);

void CrossSetionPlaneDraw(void);

void cfd_MeshDisplay(void)
{

	glClear (GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);

	glLoadIdentity ();  /*  clear the matrix    */

	glTranslatef (left_right, up_down, in_out);
	glRotatef (xAngle, 1, 0, 0);
	glRotatef (yAngle, 0, 1, 0);
	glRotatef (zAngle, 0, 0, 1);

	glPointSize(8);

	if(Axes_flag)
		glCallList(AxesList);
	if(Boundary_flag)
	{
		switch (bound_choice) {
			case 1:
			    glCallList(BoundList[1]);
			break;
			case 3:
			    glCallList(BoundList[3]);
			break;
			case 4:
			    glCallList(BoundList[4]);
			break;
			case 5:
			    glCallList(BoundList[5]);
			break;
			case 6:
			    glCallList(BoundList[6]);
			break;
			case 7:
			    glCallList(BoundList[7]);
			break;
			case 8:
			    glCallList(BoundList[8]);
			break;
			case 9:
			    glCallList(BoundList[9]);
			break;
			default:
			    glCallList(BoundList[0]);
			break;
		}
	}

	if(VelocityVec_flag)
	{
		if(Before_flag ) glCallList(VelocityList0);
		if(After_flag ) glCallList(VelocityList);
	}
	if(CrossSection_flag)
	{
		CrossSetionPlaneDraw();
	}
	glLineWidth (2.0);
	if(Render_flag)
		if(flag_3D && !gr_optimize_flag)
			cfd_brrender();
		else
			cfd_qdrender();
	else
	{
		if(flag_3D && !gr_optimize_flag)
			cfd_brmeshdraw();
		else
			cfd_qdmeshdraw();
	}

/* The display lists below aren't called when the animation flag is on because
   I don't generate them.
*/
	if(Pathline_flag == 1)
		glCallList(PathlineList0);
	if(Pathline_flag == 2)
		glCallList(PathlineList);
	if(Streakline_flag == 1)
		glCallList(StreaklineList0);
	if(Streakline_flag == 2)
		glCallList(StreaklineList);
	if(Streamline_flag == 1)
		glCallList(StreamlineList0);
	if(Streamline_flag == 2)
		glCallList(StreamlineList);
	if(Body_flag)
		glCallList(BodyList);
	if(Axes_flag)
	{
		AxesNumbers();
		/*AxesNumbers2();*/
	}
	if( app_type_choice > 0 )
	{
	    Before_flag = 0;
	    switch (anim[0].color_choice) {
		case 0:
		    After_flag = 0; 
		break;
		case 1:
		    color_choice = 1;
		    After_flag = 1; 
		break;
		case 2:
		    color_choice = 2;
		    After_flag = 1; 
		break;
		case 3:
		    color_choice = 3;
		    After_flag = 1; 
		break;
		case 4:
		    color_choice = 4;
		    After_flag = 1; 
		break;
		case 5:
		    color_choice = 5;
		    After_flag = 1; 
		break;
		case 6:
		    color_choice = 6;
		    After_flag = 1; 
		break;
		case 7:
		    color_choice = 7;
		    After_flag = 1; 
		break;
		case 8:
		    color_choice = 8;
		    After_flag = 1; 
		break;
		case 9:
		    color_choice = 9;
		    After_flag = 1; 
		break;
		case 10:
		    color_choice = 10;
		    After_flag = 1; 
		break;
	    }

	    switch (anim[0].flowtype_choice) {
		case 1:
		    flowline_points_anim( path0 );
		break;
		case 2:
		    flowline_points_anim( streak0 );
		break;
		case 3:
		    flowline_points_anim( stream0 );
		break;
	    }

	    if( anim[0].velocity_choice) velocity_vectors_anim( coord, velocity_vec );


	}
	if( app_type_choice == 3 ) bodies_in_flow3( body );

	if( app_type_choice < 3 ) glutSwapBuffers(); /* Note that all graphics functions need to
                                                        be called prior to this line. */

	if( app_type_choice > 0 ) MeshAnimate();

	/*printf( "\n step counter: %4d anim counter: %4d\n", step_counter, anim_counter);
	if(step_counter > 9 && anim_counter == 9) anim_save();*/

}

