/*
    This program plots the mesh with the various forms of viewing
    including density, temperature, pressure, velocity x-y-z, enthalpy,
    energy, and mach number(or velocity magnitude).  It works with
    the finite difference mesh.

                     Last Update 3/29/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/* FEA globals */

extern double *coord;
extern NORM *norm, *norm0;
extern int *connecter;
extern int nmat, numnp, numel;
extern GLfloat MeshColor[boxnumber+5][4];
extern GLfloat wire_color[4], black[4], green[4], yellow[4];
extern GLfloat RenderColor[4];
extern I_GR_NODE *node_color0, *node_color;
extern int *el_matl_color;
extern int color_choice, input_flag, post_flag, flag_2D, flag_3D;
extern int compressible_flag, incompressible_flag;
extern int input_color_flag;
extern int Solid_flag, Perspective_flag, Render_flag, AppliedDisp_flag,
	VelocityVec_flag, Material_flag, Node_flag, Element_flag, Axes_flag,
	Outline_flag, Transparent_flag, CrossSection_flag;
extern int Before_flag, After_flag, Both_flag;
extern int nodal_quantities_flag;
extern int matl_choice, node_choice, ele_choice;
extern double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out;

void cfd_brrender(void)
{
/* This draws the mesh when it is made up of 6 sided brick elements consisting of
   8 nodes.

                     Last Update 2/21/06
*/

	int i, i2, j, k, dof_el[npel*ndof], sdof_el[npel*nsd],
		ii, check, counter, node_num;
	int l,m,n;
	int c0,c1,c2,c3,c4,c5,c6,c7;
	int matl_number, node_number, connect_el[npel];
	int gr_flag = 0, Before_gr_flag = 0,
		element_draw_flag = 1;
	double coord_el[npel*3],
		fpointx, fpointy, fpointz;
	GLfloat d1[3], d2[3], norm_temp[3];
	I_GR_NODE node_color_el[npel];

	gr_flag = Before_flag + After_flag;

	*(wire_color + 2) = 0.0;
	if(!Solid_flag) *(wire_color + 2) = 1.0;

	MeshColor[0][3] = 1.0;
	MeshColor[1][3] = 1.0;
	MeshColor[2][3] = 1.0;
	MeshColor[3][3] = 1.0;
	MeshColor[4][3] = 1.0;
	MeshColor[5][3] = 1.0;
	MeshColor[6][3] = 1.0;
	MeshColor[7][3] = 1.0;

	if(Transparent_flag)
	{
		MeshColor[0][3] = 0.175;
		MeshColor[1][3] = 0.175;
		MeshColor[2][3] = 0.175;
		MeshColor[3][3] = 0.175;
		MeshColor[4][3] = 0.175;
		MeshColor[5][3] = 0.175;
		MeshColor[6][3] = 0.175;
		MeshColor[7][3] = 0.175;
	}
	if(color_choice == 30 || color_choice == 32) MeshColor[7][3] = 1.0;

	for( k = 0; k < numel; ++k )
	{
		element_draw_flag = 1;

		for( j = 0; j < npel; ++j )
		{

/* Calculate element degrees of freedom */

		    *(connect_el + j) = *(connecter+npel*k+j);
		    *(sdof_el+nsd*j) = nsd*(*(connect_el + j));
		    *(sdof_el+nsd*j+1) = nsd*(*(connect_el + j))+1;
		    *(sdof_el+nsd*j+2) = nsd*(*(connect_el + j))+2;

		    *(dof_el+ndof*j) = ndof*(*(connect_el + j));
		    *(dof_el+ndof*j+1) = ndof*(*(connect_el + j))+1;
		    *(dof_el+ndof*j+2) = ndof*(*(connect_el + j))+2;


/* Calculate local deformed coordinates */

		    *(coord_el+3*j)=*(coord+*(sdof_el+nsd*j));
		    *(coord_el+3*j+1)=*(coord+*(sdof_el+nsd*j+1));
		    *(coord_el+3*j+2)=*(coord+*(sdof_el+nsd*j+2));
		    if( *(coord_el+3*j) > cross_sec_left_right)
			element_draw_flag = 0;
		    if( *(coord_el+3*j + 1) > cross_sec_up_down)
			element_draw_flag = 0;
		    if( *(coord_el+3*j + 2) > cross_sec_in_out)
			element_draw_flag = 0;

			/*printf( "%9.5f %9.5f %9.5f \n",*(coord_el+3*j),
				*(coord_el+3*j+1),*(coord_el+3*j+2));*/
		}
		if(!CrossSection_flag)
		{
		    element_draw_flag = 1;
		}

/* Calculate element material number */

		matl_number = *(el_matl_color + k);

		node_color_el[0] = node_color[*(connect_el+0)];
		node_color_el[1] = node_color[*(connect_el+1)];
		node_color_el[2] = node_color[*(connect_el+2)];
		node_color_el[3] = node_color[*(connect_el+3)];
		node_color_el[4] = node_color[*(connect_el+4)];
		node_color_el[5] = node_color[*(connect_el+5)];
		node_color_el[6] = node_color[*(connect_el+6)];
		node_color_el[7] = node_color[*(connect_el+7)];

		if( Before_flag )
		{
		    node_color_el[0] = node_color0[*(connect_el+0)];
		    node_color_el[1] = node_color0[*(connect_el+1)];
		    node_color_el[2] = node_color0[*(connect_el+2)];
		    node_color_el[3] = node_color0[*(connect_el+3)];
		    node_color_el[4] = node_color0[*(connect_el+4)];
		    node_color_el[5] = node_color0[*(connect_el+5)];
		    node_color_el[6] = node_color0[*(connect_el+6)];
		    node_color_el[7] = node_color0[*(connect_el+7)];
		}

		switch (color_choice) {
		    case 1:
			c0 = node_color_el[0].dns;
			c1 = node_color_el[1].dns;
			c2 = node_color_el[2].dns;
			c3 = node_color_el[3].dns;
			c4 = node_color_el[4].dns;
			c5 = node_color_el[5].dns;
			c6 = node_color_el[6].dns;
			c7 = node_color_el[7].dns;
		    break;
		    case 2:
			c0 = node_color_el[0].temp;
			c1 = node_color_el[1].temp;
			c2 = node_color_el[2].temp;
			c3 = node_color_el[3].temp;
			c4 = node_color_el[4].temp;
			c5 = node_color_el[5].temp;
			c6 = node_color_el[6].temp;
			c7 = node_color_el[7].temp;
		    break;
		    case 3:
			c0 = node_color_el[0].prs;
			c1 = node_color_el[1].prs;
			c2 = node_color_el[2].prs;
			c3 = node_color_el[3].prs;
			c4 = node_color_el[4].prs;
			c5 = node_color_el[5].prs;
			c6 = node_color_el[6].prs;
			c7 = node_color_el[7].prs;
		    break;
		    case 4:
			c0 = node_color_el[0].vx;
			c1 = node_color_el[1].vx;
			c2 = node_color_el[2].vx;
			c3 = node_color_el[3].vx;
			c4 = node_color_el[4].vx;
			c5 = node_color_el[5].vx;
			c6 = node_color_el[6].vx;
			c7 = node_color_el[7].vx;
		    break;
		    case 5:
			c0 = node_color_el[0].vy;
			c1 = node_color_el[1].vy;
			c2 = node_color_el[2].vy;
			c3 = node_color_el[3].vy;
			c4 = node_color_el[4].vy;
			c5 = node_color_el[5].vy;
			c6 = node_color_el[6].vy;
			c7 = node_color_el[7].vy;
		    break;
		    case 6:
			c0 = node_color_el[0].vz;
			c1 = node_color_el[1].vz;
			c2 = node_color_el[2].vz;
			c3 = node_color_el[3].vz;
			c4 = node_color_el[4].vz;
			c5 = node_color_el[5].vz;
			c6 = node_color_el[6].vz;
			c7 = node_color_el[7].vz;
		    break;
		    case 7:
			c0 = node_color_el[0].enthal;
			c1 = node_color_el[1].enthal;
			c2 = node_color_el[2].enthal;
			c3 = node_color_el[3].enthal;
			c4 = node_color_el[4].enthal;
			c5 = node_color_el[5].enthal;
			c6 = node_color_el[6].enthal;
			c7 = node_color_el[7].enthal;
		    break;
		    case 8:
			c0 = node_color_el[0].ene;
			c1 = node_color_el[1].ene;
			c2 = node_color_el[2].ene;
			c3 = node_color_el[3].ene;
			c4 = node_color_el[4].ene;
			c5 = node_color_el[5].ene;
			c6 = node_color_el[6].ene;
			c7 = node_color_el[7].ene;
		    break;
		    case 9:
			if(compressible_flag)
			{
			    c0 = node_color_el[0].mach;
			    c1 = node_color_el[1].mach;
			    c2 = node_color_el[2].mach;
			    c3 = node_color_el[3].mach;
			    c4 = node_color_el[4].mach;
			    c5 = node_color_el[5].mach;
			    c6 = node_color_el[6].mach;
			    c7 = node_color_el[7].mach;
			}
			if(incompressible_flag)
			{
			    c0 = node_color_el[0].vmag;
			    c1 = node_color_el[1].vmag;
			    c2 = node_color_el[2].vmag;
			    c3 = node_color_el[3].vmag;
			    c4 = node_color_el[4].vmag;
			    c5 = node_color_el[5].vmag;
			    c6 = node_color_el[6].vmag;
			    c7 = node_color_el[7].vmag;
			}
		    break;

		    case 30:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
			c4 = 0;
			c5 = 0;
			c6 = 0;
			c7 = 0;
			if( matl_choice == matl_number )
			{
				c0 = 7;
				c1 = 7;
				c2 = 7;
				c3 = 7;
				c4 = 7;
				c5 = 7;
				c6 = 7;
				c7 = 7;
			}
		    break;
		    case 31:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
			c4 = 0;
			c5 = 0;
			c6 = 0;
			c7 = 0;
		    break;
		    case 32:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
			c4 = 0;
			c5 = 0;
			c6 = 0;
			c7 = 0;
			if( ele_choice == k )
			{
				c0 = 7;
				c1 = 7;
				c2 = 7;
				c3 = 7;
				c4 = 7;
				c5 = 7;
				c6 = 7;
				c7 = 7;
			}
		    break;
		    case 33:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
			c4 = 0;
			c5 = 0;
			c6 = 0;
			c7 = 0;
		    break;
		}

		if( input_color_flag )
		{
		     c0 = 0;
		     c1 = 0;
		     c2 = 0;
		     c3 = 0;
		     c4 = 0;
		     c5 = 0;
		     c6 = 0;
		     c7 = 0;
		}

		if( gr_flag && element_draw_flag )
		{
		   if( Solid_flag )
		   {

/* Triangle face 0 */

			*(norm_temp) = norm[k].face[0].x;
			*(norm_temp+1) = norm[k].face[0].y;
			*(norm_temp+2) = norm[k].face[0].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c1]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c1]);
				glVertex3dv((coord_el+3));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
			glEnd();

/* Triangle face 1 */

			*(norm_temp) = norm[k].face[1].x;
			*(norm_temp+1) = norm[k].face[1].y;
			*(norm_temp+2) = norm[k].face[1].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c3]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c3]);
				glVertex3dv((coord_el+9));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
			glEnd();

/* Triangle face 2 */

			*(norm_temp) = norm[k].face[2].x;
			*(norm_temp+1) = norm[k].face[2].y;
			*(norm_temp+2) = norm[k].face[2].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c1]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c1]);
				glVertex3dv((coord_el+3));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c5]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c5]);
				glVertex3dv((coord_el+15));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
			glEnd();

/* Triangle face 3 */

			*(norm_temp) = norm[k].face[3].x;
			*(norm_temp+1) = norm[k].face[3].y;
			*(norm_temp+2) = norm[k].face[3].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c4]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c4]);
				glVertex3dv((coord_el+12));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c5]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c5]);
				glVertex3dv((coord_el+15));
			glEnd();

/* Triangle face 4 */

			*(norm_temp) = norm[k].face[4].x;
			*(norm_temp+1) = norm[k].face[4].y;
			*(norm_temp+2) = norm[k].face[4].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c1]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c1]);
				glVertex3dv((coord_el+3));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c5]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c5]);
				glVertex3dv((coord_el+15));
			glEnd();

/* Triangle face 5 */

			*(norm_temp) = norm[k].face[5].x;
			*(norm_temp+1) = norm[k].face[5].y;
			*(norm_temp+2) = norm[k].face[5].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c6]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c6]);
				glVertex3dv((coord_el+18));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c5]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c5]);
				glVertex3dv((coord_el+15));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
			glEnd();

/* Triangle face 6 */

			*(norm_temp) = norm[k].face[6].x;
			*(norm_temp+1) = norm[k].face[6].y;
			*(norm_temp+2) = norm[k].face[6].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c7]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c7]);
				glVertex3dv((coord_el+21));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c6]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c6]);
				glVertex3dv((coord_el+18));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c3]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c3]);
				glVertex3dv((coord_el+9));
			glEnd();

/* Triangle face 7 */

			*(norm_temp) = norm[k].face[7].x;
			*(norm_temp+1) = norm[k].face[7].y;
			*(norm_temp+2) = norm[k].face[7].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c3]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c3]);
				glVertex3dv((coord_el+9));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c6]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c6]);
				glVertex3dv((coord_el+18));
			glEnd();

/* Triangle face 8 */

			*(norm_temp) = norm[k].face[8].x;
			*(norm_temp+1) = norm[k].face[8].y;
			*(norm_temp+2) = norm[k].face[8].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c7]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c7]);
				glVertex3dv((coord_el+21));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c3]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c3]);
				glVertex3dv((coord_el+9));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c4]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c4]);
				glVertex3dv((coord_el+12));
			glEnd();

/* Triangle face 9 */

			*(norm_temp) = norm[k].face[9].x;
			*(norm_temp+1) = norm[k].face[9].y;
			*(norm_temp+2) = norm[k].face[9].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c4]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c4]);
				glVertex3dv((coord_el+12));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c3]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c3]);
				glVertex3dv((coord_el+9));
			glEnd();

/* Triangle face 10 */

			*(norm_temp) = norm[k].face[10].x;
			*(norm_temp+1) = norm[k].face[10].y;
			*(norm_temp+2) = norm[k].face[10].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c5]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c5]);
				glVertex3dv((coord_el+15));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c6]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c6]);
				glVertex3dv((coord_el+18));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c4]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c4]);
				glVertex3dv((coord_el+12));
			glEnd();

/* Triangle face 11 */

			*(norm_temp) = norm[k].face[11].x;
			*(norm_temp+1) = norm[k].face[11].y;
			*(norm_temp+2) = norm[k].face[11].z;
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c7]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c7]);
				glVertex3dv((coord_el+21));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c4]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c4]);
				glVertex3dv((coord_el+12));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c6]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c6]);
				glVertex3dv((coord_el+18));
			glEnd();
		   }
   
/* Draw the wire frame around the mesh */
   
		   if( Outline_flag )
		   {
			glMaterialfv(GL_FRONT, GL_DIFFUSE, wire_color);
			glMaterialfv(GL_FRONT, GL_AMBIENT, wire_color);
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el+9));
				glVertex3dv((coord_el+6));
				glVertex3dv((coord_el+3));
				glVertex3dv((coord_el));
			glEnd();
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el));
				glVertex3dv((coord_el+3));
				glVertex3dv((coord_el+15));
				glVertex3dv((coord_el+12));
			glEnd();
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el+3));
				glVertex3dv((coord_el+6));
				glVertex3dv((coord_el+18));
				glVertex3dv((coord_el+15));
			glEnd();
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el+21));
				glVertex3dv((coord_el+18));
				glVertex3dv((coord_el+6));
				glVertex3dv((coord_el+9));
			glEnd();
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el+9));
				glVertex3dv((coord_el));
				glVertex3dv((coord_el+12));
				glVertex3dv((coord_el+21));
			glEnd();
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el+12));
				glVertex3dv((coord_el+15));
				glVertex3dv((coord_el+18));
				glVertex3dv((coord_el+21));
			glEnd();
		   }
		}

	}
/* This draws the Node ID node */
	if (color_choice == 31)
	{
	    glPointSize(8);
	    node_number=node_choice;
	    fpointx = *(coord+nsd*node_number);
	    fpointy = *(coord+nsd*node_number+1);
	    fpointz = *(coord+nsd*node_number+2);
	    glBegin(GL_POINTS);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, yellow);
		glMaterialfv(GL_FRONT, GL_AMBIENT, yellow);
		glVertex3f(fpointx, fpointy, fpointz);
	    glEnd();
	}
	/*return 1;*/
}

void cfd_qdrender(void)
{
/* This draws the mesh when it is made up of single sided quad elements consisting
   of 4 nodes.  In the case of a 2-D mesh, all the points and elements lie on
   one plane.  For a 3-D mesh, the elements are in 3-D space.

   The reason I am using this function to draw, in addition to 2-D meshes, the 3-D
   finite difference meshes is because I wanted to optimize the drawing routines.  The
   function above draws all 6 sides of the mesh despite the fact that usually only 1
   side can be seen.   Because these meshes are so big, and finite difference meshes
   are highly structured, this is quite feasible.

                     Last Update 2/21/06
*/

	int i, i2, j, k, dof_el[npel2*ndof], sdof_el[npel2*nsd],
		ii, check, counter, node_num;
	int l,m,n;
	int c0,c1,c2,c3,c4,c5,c6,c7;
	int matl_number, node_number, connect_el[npel2];
	int gr_flag = 0, Before_gr_flag = 0,
		element_draw_flag = 1;
	double coord_el[npel*3],
		fpointx, fpointy, fpointz;
	GLfloat d1[3], d2[3], norm_temp[] = {0.0, 0.0, 1.0};
	I_GR_NODE node_color_el[npel2];

	gr_flag = Before_flag + After_flag;

	*(wire_color + 2) = 0.0;
	if(!Solid_flag) *(wire_color + 2) = 1.0;

	MeshColor[0][3] = 1.0;
	MeshColor[1][3] = 1.0;
	MeshColor[2][3] = 1.0;
	MeshColor[3][3] = 1.0;
	MeshColor[4][3] = 1.0;
	MeshColor[5][3] = 1.0;
	MeshColor[6][3] = 1.0;
	MeshColor[7][3] = 1.0;

	if(Transparent_flag)
	{
		MeshColor[0][3] = 0.175;
		MeshColor[1][3] = 0.175;
		MeshColor[2][3] = 0.175;
		MeshColor[3][3] = 0.175;
		MeshColor[4][3] = 0.175;
		MeshColor[5][3] = 0.175;
		MeshColor[6][3] = 0.175;
		MeshColor[7][3] = 0.175;
	}
	if(color_choice == 30 || color_choice == 32) MeshColor[7][3] = 1.0;

	for( k = 0; k < numel; ++k )
	{
		element_draw_flag = 1;

		for( j = 0; j < npel2; ++j )
		{

/* Calculate element degrees of freedom */

		    *(connect_el + j) = *(connecter+npel2*k+j);
		    *(sdof_el+nsd*j) = nsd*(*(connect_el + j));
		    *(sdof_el+nsd*j+1) = nsd*(*(connect_el + j))+1;
		    *(sdof_el+nsd*j+2) = nsd*(*(connect_el + j))+2;

		    *(dof_el+ndof*j) = ndof*(*(connect_el + j));
		    *(dof_el+ndof*j+1) = ndof*(*(connect_el + j))+1;
		    *(dof_el+ndof*j+2) = ndof*(*(connect_el + j))+2;


/* Calculate local deformed coordinates */

		    *(coord_el+3*j)=*(coord+*(sdof_el+nsd*j));
		    *(coord_el+3*j+1)=*(coord+*(sdof_el+nsd*j+1));
		    *(coord_el+3*j+2)=*(coord+*(sdof_el+nsd*j+2));
		    if( *(coord_el+3*j) > cross_sec_left_right)
			element_draw_flag = 0;
		    if( *(coord_el+3*j + 1) > cross_sec_up_down)
			element_draw_flag = 0;
		    if( *(coord_el+3*j + 2) > cross_sec_in_out)
			element_draw_flag = 0;

			/*printf( "%9.5f %9.5f %9.5f \n",*(coord_el+3*j),
				*(coord_el+3*j+1),*(coord_el+3*j+2));*/
		}
		if(!CrossSection_flag)
		{
		    element_draw_flag = 1;
		}

/* Calculate element material number */

		matl_number = *(el_matl_color + k);

		node_color_el[0] = node_color[*(connect_el+0)];
		node_color_el[1] = node_color[*(connect_el+1)];
		node_color_el[2] = node_color[*(connect_el+2)];
		node_color_el[3] = node_color[*(connect_el+3)];

		if( Before_flag )
		{
		    node_color_el[0] = node_color0[*(connect_el+0)];
		    node_color_el[1] = node_color0[*(connect_el+1)];
		    node_color_el[2] = node_color0[*(connect_el+2)];
		    node_color_el[3] = node_color0[*(connect_el+3)];
		}

		switch (color_choice) {
		    case 1:
			c0 = node_color_el[0].dns;
			c1 = node_color_el[1].dns;
			c2 = node_color_el[2].dns;
			c3 = node_color_el[3].dns;
		    break;
		    case 2:
			c0 = node_color_el[0].temp;
			c1 = node_color_el[1].temp;
			c2 = node_color_el[2].temp;
			c3 = node_color_el[3].temp;
		    break;
		    case 3:
			c0 = node_color_el[0].prs;
			c1 = node_color_el[1].prs;
			c2 = node_color_el[2].prs;
			c3 = node_color_el[3].prs;
		    break;
		    case 4:
			c0 = node_color_el[0].vx;
			c1 = node_color_el[1].vx;
			c2 = node_color_el[2].vx;
			c3 = node_color_el[3].vx;
		    break;
		    case 5:
			c0 = node_color_el[0].vy;
			c1 = node_color_el[1].vy;
			c2 = node_color_el[2].vy;
			c3 = node_color_el[3].vy;
		    break;
		    case 6:
			c0 = node_color_el[0].vz;
			c1 = node_color_el[1].vz;
			c2 = node_color_el[2].vz;
			c3 = node_color_el[3].vz;
		    break;
		    case 7:
			c0 = node_color_el[0].enthal;
			c1 = node_color_el[1].enthal;
			c2 = node_color_el[2].enthal;
			c3 = node_color_el[3].enthal;
		    break;
		    case 8:
			c0 = node_color_el[0].ene;
			c1 = node_color_el[1].ene;
			c2 = node_color_el[2].ene;
			c3 = node_color_el[3].ene;
		    break;
		    case 9:
			if(compressible_flag)
			{
			    c0 = node_color_el[0].mach;
			    c1 = node_color_el[1].mach;
			    c2 = node_color_el[2].mach;
			    c3 = node_color_el[3].mach;
			}
			if(incompressible_flag)
			{
			    c0 = node_color_el[0].vmag;
			    c1 = node_color_el[1].vmag;
			    c2 = node_color_el[2].vmag;
			    c3 = node_color_el[3].vmag;
			}
		    break;

		    case 30:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
			if( matl_choice == matl_number )
			{
				c0 = 7;
				c1 = 7;
				c2 = 7;
				c3 = 7;
			}
		    break;
		    case 31:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
		    break;
		    case 32:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
			if( ele_choice == k )
			{
				c0 = 7;
				c1 = 7;
				c2 = 7;
				c3 = 7;
			}
		    break;
		    case 33:
			c0 = 0;
			c1 = 0;
			c2 = 0;
			c3 = 0;
		    break;
		}

		if( input_color_flag )
		{
		     c0 = 0;
		     c1 = 0;
		     c2 = 0;
		     c3 = 0;
		}

		if( gr_flag && element_draw_flag )
		{
		   if( Solid_flag )
		   {

/* Triangle face 0 */

			if(flag_3D)
			{
				*(norm_temp) = norm[k].face[0].x;
				*(norm_temp+1) = norm[k].face[0].y;
				*(norm_temp+2) = norm[k].face[0].z;
			}
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c1]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c1]);
				glVertex3dv((coord_el+3));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
			glEnd();

/* Triangle face 1 */

			if(flag_3D)
			{
				*(norm_temp) = norm[k].face[1].x;
				*(norm_temp+1) = norm[k].face[1].y;
				*(norm_temp+2) = norm[k].face[1].z;
			}
			glBegin(GL_TRIANGLES);
				glNormal3fv(norm_temp);
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c3]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c3]);
				glVertex3dv((coord_el+9));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c2]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c2]);
				glVertex3dv((coord_el+6));
				glMaterialfv(GL_FRONT, GL_DIFFUSE, MeshColor[c0]);
				glMaterialfv(GL_FRONT, GL_AMBIENT, MeshColor[c0]);
				glVertex3dv((coord_el));
			glEnd();

		   }
   
/* Draw the wire frame around the mesh */
   
		   if( Outline_flag )
		   {
			glMaterialfv(GL_FRONT, GL_DIFFUSE, wire_color);
			glMaterialfv(GL_FRONT, GL_AMBIENT, wire_color);
			glBegin(GL_LINE_LOOP);
				glVertex3dv((coord_el+9));
				glVertex3dv((coord_el+6));
				glVertex3dv((coord_el+3));
				glVertex3dv((coord_el));
			glEnd();
		   }
		}

	}
/* This draws the Node ID node */
	if (color_choice == 31)
	{
	    glPointSize(8);
	    node_number=node_choice;
	    fpointx = *(coord+nsd*node_number);
	    fpointy = *(coord+nsd*node_number+1);
	    fpointz = *(coord+nsd*node_number+2);
	    glBegin(GL_POINTS);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, yellow);
		glMaterialfv(GL_FRONT, GL_AMBIENT, yellow);
		glVertex3f(fpointx, fpointy, fpointz);
	    glEnd();
	}
	/*return 1;*/
}

