/*
    This program reads in the new input file and prepares it
    for graphical display.
  
                  Last Update 6/9/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"
#include "../AMR/amr.h"

/* glut header files */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/********************* These are all the subroutines **************************/

/******** Data management and calculations ********/

void flowline_points(int , FLOWLINES );

void velocity_vectors(int , BOUND , double *, XYZF *);

void bound_vectors(GLuint *, BOUND , double *);

void agvMakeAxesList(GLuint);

int cfd_set( BOUND , XYZF *, GR_NODE *, I_GR_NODE *, GR_NODE *);

int bcset( BOUND *, int *, GR_NODE * );

int fdparameter( GR_NODE *);

int coord_parameter( double * );

int compare_body ( const void *,  const void *);

int create_body( BODY *);

int bcset_noslip( BOUND *, BODY *, int *, GR_NODE * );

int bcset( BOUND *, int *, GR_NODE * );

int fdnormal_vectors (int *, double *, NORM * );

int ConnectSurfMesher2( int *, int *, GR_NODE *);

int ConnectSurfMesher( int *, int *, GR_NODE *);

int fdMesher( int *, double *, GR_NODE *);

int qikreader( MATL *, char *, GR_NODE *, FILE *, FLOWLINES, FLOWLINES, FLOWLINES );

int flwreader( MATL *, char *, GR_NODE *, FILE *, FLOWLINES, FLOWLINES, FLOWLINES );

int fdReGetMemory_gr( I_GR_NODE **, int, NORM **, int );

int cfdReGetMemory( double **, int , int **, int , MATL **, int, XYZF **, int, XYZI **, int,
	XYZI2 **, int , XY_ZX_YZ **, int, GR_NODE **, int, SUBGRID **, int  );

int Flowlines_File( FLOWLINES * );

int filecheck( char *, char *, FILE **, FILE **, FILE **, char *, int );

/******************************* GLOBAL VARIABLES **************************/

/****** CFD globals ******/
extern int dof, sdof, nmat, nmode, numel, numnp, nx, ny, nz, num_bodies;
extern int flag_2D, flag_3D, mach_read_flag, vmag_read_flag, print_flag, gr_optimize_flag;
extern int app_type_choice;
extern double dx, dy, dz;
extern double dt, err;
extern double time_current, time_current_input, time_current_post;
extern int nstep, cstep, cstep_input, cstep_post;
extern int compressible_flag, incompressible_flag;

extern XYZI2 *mem_XYZI2;
extern XYZF *mem_XYZF;
extern XYZI *mem_XYZI;
extern XY_ZX_YZ *mem_XY_ZX_YZ;
extern int *mem_int;
extern double *mem_double;
extern NORM *mem_NORM;
extern double *coord;
extern double *U;
extern int *connecter;
extern BOUND bc;
extern BODY *body;
extern MATL *matl;
extern int *el_matl;
extern double *velocity;
extern GR_NODE *node0, *node;
extern SUBGRID *subgrid1, *mem_SUBGRID;
extern FLOWLINES path0, path, streak0, streak, stream0, stream;
extern ANIM_DAT anim[1];


/* Global variables for the mesh color and nodal data */

I_GR_NODE *node_color0, *node_color;
extern int *el_matl_color;
extern NORM *norm, *norm0;
extern MATL *matl_crtl;

/****** Cross Section Plane Translation Variables ******/
double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
	cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;

/* Global variables for drawing the axes */
extern GLuint AxesList, BodyList, BoundList[10], VelocityList0, VelocityList,   /* Display lists */
	PathlineList0, PathlineList, StreaklineList0, StreaklineList,
	StreamlineList0, StreamlineList;

/* Global variables for drawing the velocity vectors */
extern XYZF *velocity_vec, *velocity_vec0;

/****** For drawing the Mesh Window ******/
extern double coord_rescale;

extern char NameData[25];
extern char ControlText[][90];

extern GR_NODE node_div[boxnumber+1], node_div0[boxnumber+1];
extern int input_flag, post_flag, matl_choice, node_choice, ele_choice, mode_choice;
extern int Before_flag, After_flag;
extern double amplify_factor;

int fdGetNewMesh(void)
{
	int i, j, node_num, check, file_loc, name_length;
	double dns, temp, vx, vy, vz, prs, ene, enthal, mach, R, gam;
	char *ccheck;
	int dum, dum1, dum2, dum3, numel0, bod_num, pt;
	double fpointx, fpointy, fpointz, fdum, fdum2, fdum3;
	int  sofmi, sofmf, sofmGR_NODE, sofmI_GR_NODE, sofmXYZF, sofmXYZI, sofmXYZI2, sofmXY_ZX_YZ,
		sofmNORM, sofmSUBGRID, ptr_inc;
	char name[30], name2[30], ofd_exten[4], buf[ BUFSIZ ], one_char, char_dum[20], object_name[30];
	int ofd_exten_length = 4;
	XYZF *dum_XYZF;
	FILE *o1, *o2, *o3;

	print_flag = 0;
	anim[0].flag = 0;
	anim[0].save_flag = 0;
	app_type_choice = 0;

/* Delete the old display lists */

	glDeleteLists(AxesList,1);
	glDeleteLists(BoundList[0],1);
	glDeleteLists(BoundList[1],1);
	glDeleteLists(BoundList[2],1);
	glDeleteLists(BoundList[3],1);
	glDeleteLists(BoundList[4],1);
	glDeleteLists(BoundList[5],1);
	glDeleteLists(BoundList[6],1);
	glDeleteLists(BoundList[7],1);
	glDeleteLists(BoundList[8],1);
	glDeleteLists(BoundList[9],1);

	if(num_bodies)
	{
		glDeleteLists(BodyList,1);
	}

	if(input_flag)
	{
		glDeleteLists(VelocityList0,1);
		glDeleteLists(PathlineList0,1);
		glDeleteLists(StreaklineList0,1);
		glDeleteLists(StreamlineList0,1);
	}
	if(post_flag)
	{
		glDeleteLists(VelocityList,1);
		glDeleteLists(PathlineList,1);
		glDeleteLists(StreaklineList,1);
		glDeleteLists(StreamlineList,1);
	}

/* Initialize filenames */

	memset(name,0,30*sizeof(char));
	memset(name2,0,30*sizeof(char));
	memset(ofd_exten,0,ofd_exten_length*sizeof(char));

/* Initialize old variables */

	input_flag = 1;
	post_flag = 1;
	After_flag = 1;
	Before_flag = 0;
	mode_choice = 0;
	amplify_factor = 1.0;
	matl_choice = 0;
	node_choice = 0;
	ele_choice = 0;

	ccheck = strncpy(ofd_exten,".ofd",ofd_exten_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	printf("What is the name of the input file containing the \n");
	printf("finite difference CFD data? \n");
	scanf( "%30s",name2);

/*   o1 contains all the structural data for input
     o3 contains all the structural data for postprocessing
     o2 is used to determine the existance of input and post files
*/
	
	o2 = fopen( name2,"r" );
	if(o2 == NULL ) {
		printf("Can't find file %30s\n", name2);
		exit(1);
	}
	/*printf( "%3d %30s\n ",name2_length,name2);*/
	
	fgets( buf, BUFSIZ, o2 );
	/*fscanf( o2, "%d %d %d\n ",&numel,&numnp,&nmat);*/
	fscanf( o2, "%d %d %d", &numnp, &nx, &ny);

/* Check if there is additional coordinate z data and that this is a 3-D file.  */

	flag_3D = 0;
	flag_2D = 1;
	nz = 1;
	while(( one_char = (unsigned char) fgetc(o2)) != '\n')
	{
		if(one_char != ' ' )
		{
		    ungetc( one_char, o2);
		    fscanf( o2,"%d", &nz);
		    flag_3D = 1;
		    flag_2D = 0;
		    break;
		}
	}
	numnp = nx*ny*nz;

	numel = (nx-1)*(ny-1);
	if(flag_3D) numel = (nx-1)*(ny-1)*(nz-1);
	numel0 = numel; /* numel0 is needed because numel is changed in fdMesher. */

	nmat = 1;
	fscanf( o2,"\n");

	if(flag_3D) printf( "total nodes:%d   nodes in x:%d nodes in y:%d nodes in z:%d\n", numnp, nx, ny, nz);
	else        printf( "total nodes:%d   nodes in x:%d nodes in y:%d\n", numnp, nx, ny);
	fgets( buf, BUFSIZ, o2 );
	printf( "\n");

	fscanf( o2, "%lf %d %d %lf", &dt, &nstep, &cstep, &time_current);
	printf( "Time step:     %16.8e\n", dt);
	printf( "number of steps:  %4d\n", nstep);
	printf( "current step:     %4d\n", cstep);
	printf( "current time:  %16.8e\n", time_current);
	printf( "\n");

	compressible_flag = 1;
	incompressible_flag = 0;
	while(( one_char = (unsigned char) fgetc(o2)) != '\n')
	{
		if(one_char != ' ' )
		{
		    ungetc( one_char, o2);
		    fscanf( o2,"%lf\n", &err);
		    printf( "SOR tolerence: %16.8e\n", err);
		    incompressible_flag = 1;
		    compressible_flag = 0;
		    mach_read_flag = 0;
		    vmag_read_flag = 1;
		    break;
		}
	}
	fgets( buf, BUFSIZ, o2 );
	printf( "\n");
	/*printf( "8888888 %4d %4d\n", compressible_flag, incompressible_flag);*/

	dof=numnp*ndof;
	sdof=numnp*nsd;
	nmode = 0;

/* Begin exmaining and checking for the existence of data files */

	check = filecheck( name, name2, &o1, &o2, &o3, ofd_exten, ofd_exten_length );
	if(!check) printf( " Problems with filecheck \n");

	if( input_flag )
	{
		fgets( buf, BUFSIZ, o1 );
		if(flag_2D) {
			fscanf( o1, "%d %d %d\n ",&dum,&dum1,&dum2);
			printf( "%d %d %d\n",dum,dum1,dum2);
		}
		else {
			fscanf( o1, "%d %d %d %d\n ",&dum,&dum1,&dum2,&dum3);
			printf( "%d %d %d %d\n",dum,dum1,dum2,dum3);
		}
		fgets( buf, BUFSIZ, o1 );
		if(compressible_flag) {
		    fscanf( o1, "%lf %d %d %lf\n ",&fdum,&dum1,&dum2,&fdum2);
		    /*printf( "%lf %d %d %lf\n",fdum,dum1,dum2,fdum2);*/
		}
		if(incompressible_flag) {
		    fscanf( o1, "%lf %d %d %lf %d %lf\n ",&fdum,&dum1,&dum2,&fdum2,&dum3,&fdum3);
		    /*printf( "%lf %d %d %lf %d %lf\n",fdum,dum1,dum2,fdum2,dum3,fdum3);*/
		}
		fgets( buf, BUFSIZ, o1 );

		/*printf( "name %30s\n ",name);*/
	}

	/*post_flag = 0;*/
	if( post_flag )
	{
		fgets( buf, BUFSIZ, o3 );
		if(flag_2D) {
			fscanf( o3, "%d %d %d\n ",&dum,&dum1,&dum2);
			printf( "%d %d %d\n",dum,dum1,dum2);
		}
		else {
			fscanf( o3, "%d %d %d %d\n ",&dum,&dum1,&dum2,&dum3);
			printf( "%d %d %d %d\n",dum,dum1,dum2,dum3);
		}
		fgets( buf, BUFSIZ, o3 );
		if(compressible_flag) {
		    fscanf( o3, "%lf %d %d %lf\n ",&fdum,&dum1,&dum2,&fdum2);
		    /*printf( "%lf %d %d %lf\n",fdum,dum1,dum2,fdum2);*/
		}
		if(incompressible_flag) {
		    fscanf( o3, "%lf %d %d %lf %d %lf\n ",&fdum,&dum1,&dum2,&fdum2,&dum3,&fdum3);
		    /*printf( "%lf %d %d %lf %d %lf\n",fdum,dum1,dum2,fdum2,dum3,fdum3);*/
		}
		fgets( buf, BUFSIZ, o3 );
		/*printf( "out %30s\n ",out);*/
	}

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;


                      /* PATHLINES */

/* The code below is for pathlines coming from the post mesh. */

/* Set the name of the post pathline file */

	memset(path.name_in,0,41*sizeof(char));

	ccheck = strncpy(path.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(path.name_in+name_length, ".path.ofd", 11);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the pathline post file as well as its properties */

	check = Flowlines_File( &path );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(path.new_flag) path.nstep_in = 1;

/* The code below is for pathlines coming from the input mesh. */

/* Set the name of the input pathline file */

	memset(path0.name_in,0,41*sizeof(char));

	ccheck = strncpy(path0.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(path0.name_in+name_length, ".path", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the pathline input file as well as its properties */

	check = Flowlines_File( &path0 );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(path0.new_flag) path0.nstep_in = 1;


                  /* STREAKLINES */

/* The code below is for streaklines coming from the post mesh. */

/* Set the name of the post streakline file */

	memset(streak.name_in,0,41*sizeof(char));

	ccheck = strncpy(streak.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(streak.name_in+name_length, ".streak.ofd", 11);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streakline post file as well as its properties */

	check = Flowlines_File( &streak );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(streak.new_flag) streak.nstep_in = 1;

/* The code below is for streaklines coming from the input mesh. */

/* Set the name of the input streakline file */

	memset(streak0.name_in,0,41*sizeof(char));

	ccheck = strncpy(streak0.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(streak0.name_in+name_length, ".streak", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streakline input file as well as its properties */

	check = Flowlines_File( &streak0 );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(streak0.new_flag) streak0.nstep_in = 1;


                  /* STREAMLINES */

/* The code below is for streamlines coming from the post mesh. */

/* Set the name of the post streamline file */

	memset(stream.name_in,0,41*sizeof(char));

	ccheck = strncpy(stream.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(stream.name_in+name_length, ".stream.ofd", 11);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streamline post file as well as its properties */

	check = Flowlines_File( &stream );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(stream.new_flag) stream.nstep_in = 1;

/* The code below is for streamlines coming from the input mesh. */

/* Set the name of the input streamline file */

	memset(stream0.name_in,0,41*sizeof(char));

	ccheck = strncpy(stream0.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(stream0.name_in+name_length, ".stream", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streamline input file as well as its properties */

	check = Flowlines_File( &stream0 );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(stream0.new_flag) stream0.nstep_in = 1;


	/*printf( "ththth %s %s\n ", streak.name_in, streak0.name_in);*/

/*   Begin allocation of meomory */

/* For the doubles */
	sofmf = sdof + 2*dof;

/* For the integers */
	if(flag_2D) sofmi= numel0*npel2 + numnp + 2 + 4*numnp;
		else sofmi= (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2 + 4*numnp;

/* Note is that the last value, 2*numnp, is based on the fact that some nodes
   may have more than 1 boundary condition, specifically the no-slip corner
   boundary which results in a node being no-slip in more than 1 direction.  So
   I can have, for instance:

      bc[0].noslp.xp[3] = 5;
      bc[0].noslp.yp[3] = 5;

   This potential for having more than one boundary condition is why I have allocated
   2*numnp. 
*/

/* For the XYZF doubles */

	dum = 0;
	if( path0.read_flag ) dum += path0.num_pt0 + path0.num_pt0*path0.nstep_in;
	if( path.read_flag ) dum += path.num_pt0 + path.num_pt0*path.nstep_in;
	if( streak0.read_flag ) dum += streak0.num_pt0 + streak0.num_pt0*streak0.nstep_in;
	if( streak.read_flag ) dum += streak.num_pt0 + streak.num_pt0*streak.nstep_in;
	if( stream0.read_flag ) dum += stream0.num_pt0 + stream0.num_pt0*stream0.nstep_in;
	if( stream.read_flag ) dum += stream.num_pt0 + stream.num_pt0*stream.nstep_in;
	sofmXYZF=2*numnp + dum;

/* For the XYZI integers */
	sofmXYZI = 1;

/* For the XYZI2 integers */
	sofmXYZI2=7;

/* For the XY_ZX_YZ integers */
	sofmXY_ZX_YZ=1;

/* For the GR_NODE */
	sofmGR_NODE=numnp;
	if( input_flag && post_flag ) sofmGR_NODE=2*numnp;

/* For the I_GR_NODE */
	sofmI_GR_NODE=numnp;
	if( input_flag && post_flag ) sofmI_GR_NODE=2*numnp;

/* For the NORMS */
	sofmNORM=numel0;
	if(gr_optimize_flag) sofmNORM = 2*nx*ny + 2*nx*nz + 2*ny*nz;

/* For the SUBGRID structs*/
	sofmSUBGRID=1;

	check = cfdReGetMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat, &mem_XYZF, sofmXYZF,
		&mem_XYZI, sofmXYZI, &mem_XYZI2, sofmXYZI2, &mem_XY_ZX_YZ, sofmXY_ZX_YZ, &node0,
		sofmGR_NODE, &mem_SUBGRID, sofmSUBGRID );
	if(!check) printf( " Problems with cfdReGetMemory \n");

	check = fdReGetMemory_gr( &node_color0, sofmI_GR_NODE, &mem_NORM, sofmNORM );
	if(!check) printf( " Problems with fdReGetMemory_gr \n");

/* For the doubles */
	                                ptr_inc=0;
	coord=(mem_double+ptr_inc);     ptr_inc += sdof;
	velocity=(mem_double+ptr_inc);  ptr_inc += dof;
	U=(mem_double+ptr_inc);         ptr_inc += dof;

/* For the materials */

	matl_crtl = matl;

/* For the integers */
	                                         ptr_inc = 0;
	connecter=(mem_int+ptr_inc);
	                            if(flag_2D) ptr_inc += numel0*npel2;
	                                   else ptr_inc += (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2;
	el_matl=(mem_int+ptr_inc);              ptr_inc += numnp;
	bc.num_none=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_prs0=(mem_int+ptr_inc);          ptr_inc += 1;

/* Note that bcset below has to be corrected for mem_int so that it has the correct
   offset for mem_int, i.e.:

        if(flag_2D) dum = numel0*npel2 + numnp + 2;
        else dum = (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2;

        check = bcset( &bc, (mem_int + dum), node);
        if(!check) printf( " Problems with bcset\n");
*/

	memset(el_matl,0,numnp*sizeof(int));
	el_matl_color = el_matl;

/* For the XYZI integers */
	                                      ptr_inc = 0;
	bc.num_vel0=(mem_XYZI+ptr_inc);       ptr_inc += 1;

/* For the XYZI2 integers */
	                                      ptr_inc = 0;
	bc.num_symm=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_far=(mem_XYZI2+ptr_inc);       ptr_inc += 1;
	bc.num_free=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_inflow=(mem_XYZI2+ptr_inc);    ptr_inc += 1;
	bc.num_frestr=(mem_XYZI2+ptr_inc);    ptr_inc += 1;
	bc.num_free2=(mem_XYZI2+ptr_inc);     ptr_inc += 1;
	bc.num_noslp=(mem_XYZI2+ptr_inc);     ptr_inc += 1;

/* For the XY_ZX_YZ integers */
	                                          ptr_inc = 0;
	bc.num_noslp_prs=(mem_XY_ZX_YZ+ptr_inc);  ptr_inc += 1;

/* For the GR_NODE doubles */
	                                      ptr_inc = numnp;
	if( input_flag && post_flag ) node =(node0+ptr_inc);

/* For the I_GR_NODE integers */
                                              ptr_inc = numnp;
	if( input_flag && post_flag ) node_color =(node_color0+ptr_inc);

/* For the NORM doubles */
	                                      ptr_inc = 0;
	norm =(mem_NORM+ptr_inc);
#if 0
	if( input_flag && post_flag )         ptr_inc += numel0;
	norm0 =(mem_NORM+ptr_inc);            ptr_inc += numel0;
#endif

/* For the XYZF doubles */
	                                          ptr_inc = 0;
	velocity_vec =(mem_XYZF+ptr_inc);         ptr_inc += numnp;
	velocity_vec0 =(mem_XYZF+ptr_inc);        ptr_inc += numnp;
	if(path0.read_flag)
	{
		path0.pt0=(mem_XYZF+ptr_inc);   ptr_inc += path0.num_pt0;
		path0.pt=(mem_XYZF+ptr_inc);    ptr_inc += path0.num_pt0*path0.nstep_in;
	}
	if(path.read_flag)
	{
		path.pt0=(mem_XYZF+ptr_inc);    ptr_inc += path.num_pt0;
		path.pt=(mem_XYZF+ptr_inc);     ptr_inc += path.num_pt0*path.nstep_in;
	}
	if(streak0.read_flag)
	{
		streak0.pt0=(mem_XYZF+ptr_inc);   ptr_inc += streak0.num_pt0;
		streak0.pt=(mem_XYZF+ptr_inc);    ptr_inc += streak0.num_pt0*streak0.nstep_in;
	}
	if(streak.read_flag)
	{
		streak.pt0=(mem_XYZF+ptr_inc);    ptr_inc += streak.num_pt0;
		streak.pt=(mem_XYZF+ptr_inc);     ptr_inc += streak.num_pt0*streak.nstep_in;
	}
	if(stream0.read_flag)
	{
		stream0.pt0=(mem_XYZF+ptr_inc);   ptr_inc += stream0.num_pt0;
		stream0.pt=(mem_XYZF+ptr_inc);    ptr_inc += stream0.num_pt0*stream0.nstep_in;
	}
	if(stream.read_flag)
	{
		stream.pt0=(mem_XYZF+ptr_inc);    ptr_inc += stream.num_pt0;
		stream.pt=(mem_XYZF+ptr_inc);     ptr_inc += stream.num_pt0*stream.nstep_in;
	}

/* For the SUBGRID structs*/
                                              ptr_inc = 0;
        subgrid1=(mem_SUBGRID+ptr_inc);       ptr_inc += 1;

	if( !post_flag )
	{
	    node = node0;
	    node_color = node_color0;
	    After_flag = 0;
	    Before_flag = 1;
	}

	if( !input_flag )
	{
	    node = node0;
	    node_color = node_color0;
	    After_flag = 1;
	    Before_flag = 0;
	}

	dum = strlen(name);
	memset(char_dum,0,20*sizeof(char));
	if( dum < 9 )
	{
		ccheck = strncpy(char_dum, name, 8);
		sprintf( NameData, "%s", char_dum);
	}
	else
	{
		ccheck = strncpy(char_dum, name, 5);
		sprintf( NameData, "%s...", char_dum);
	}

	if( compressible_flag )
	{
		ccheck = strncpy(ControlText[58], " density    ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[60], " temperature", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[70], " enthalpy   ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[72], " energy     ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[74], " mach number", 12);
		if(!ccheck) printf( " Problems with strncpy \n");
	}
	if( incompressible_flag )
	{
		ccheck = strncpy(ControlText[58], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[60], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[70], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[72], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[74], " velocity mag", 13);
		if(!ccheck) printf( " Problems with strncpy \n");
	}

	if( post_flag )
	{
		if( compressible_flag )
		{
			check = flwreader( matl, name, node, o3, path, streak, stream);
			if(!check) printf( " Problems with flwreader\n");
		}
		if( incompressible_flag )
		{
			check = qikreader( matl, name, node, o3, path, streak, stream);
			if(!check) printf( " Problems with qikreader\n");
		}

/* The 33 lines below are for a case that I don't think will ever come up.  The only case
   is when a file like (filename).path.ofd would still have the format of a starting file.
   I probably put it here because I did a direct copy from the "if( input_flag )" case
   where it will often come up.  But on the off chance that this case will happen for post
   "*.ofd" files, I am leaving it in.
*/
		if(path.read_flag)
		{
			if(path.new_flag)
			{
			    for( j = 0; j < path.num_pt0; ++j )
			    {
				path.pt[j].x = path.pt0[j].x;
				path.pt[j].y = path.pt0[j].y;
			    }
			}
		}
		if(streak.read_flag)
		{
			if(streak.new_flag)
			{
			    for( j = 0; j < streak.num_pt0; ++j )
			    {
				streak.pt[j].x = streak.pt0[j].x;
				streak.pt[j].y = streak.pt0[j].y;
			    }
			}
		}
		if(stream.read_flag)
		{
			if(stream.new_flag)
			{
			    for( j = 0; j < stream.num_pt0; ++j )
			    {
				stream.pt[j].x = stream.pt0[j].x;
				stream.pt[j].y = stream.pt0[j].y;
			    }
			}
		}

		time_current_post = time_current;
		cstep_post = cstep;

		if(!mach_read_flag)
		{
		    R = matl[0].R;
		    gam = matl[0].gam;
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			temp   = node[node_num].temp;
			vx     = node[node_num].vx;
			vy     = node[node_num].vy;
			vz     = node[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz)/(R*gam*temp + SMALL);
			node[node_num].mach = sqrt(fdum);
		   }
		}

		if(!vmag_read_flag && incompressible_flag)
		{
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			vx     = node[node_num].vx;
			vy     = node[node_num].vy;
			vz     = node[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz);
			node[node_num].vmag = sqrt(fdum);
		   }
		}
	}
	if( input_flag )
	{
		if( compressible_flag )
		{
			check = flwreader( matl, name, node0, o1, path0, streak0, stream0);
			if(!check) printf( " Problems with flwreader\n");
		}
		if( incompressible_flag )
		{
			check = qikreader( matl, name, node0, o1, path0, streak0, stream0);
			if(!check) printf( " Problems with qikreader\n");
		}

		if(path0.read_flag)
		{
			if(path0.new_flag)
			{
			    for( j = 0; j < path0.num_pt0; ++j )
			    {
				path0.pt[j].x = path0.pt0[j].x;
				path0.pt[j].y = path0.pt0[j].y;
			    }
			}
		}
		if(streak0.read_flag)
		{
			if(streak0.new_flag)
			{
			    for( j = 0; j < streak0.num_pt0; ++j )
			    {
				streak0.pt[j].x = streak0.pt0[j].x;
				streak0.pt[j].y = streak0.pt0[j].y;
			    }
			}
		}
		if(stream0.read_flag)
		{
			if(stream0.new_flag)
			{
			    for( j = 0; j < stream0.num_pt0; ++j )
			    {
				stream0.pt[j].x = stream0.pt0[j].x;
				stream0.pt[j].y = stream0.pt0[j].y;
			    }
			}
		}

		time_current_input = time_current;
		cstep_input = cstep;

		if(!mach_read_flag)
		{
		    R = matl[0].R;
		    gam = matl[0].gam;
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			temp   = node0[node_num].temp;
			vx     = node0[node_num].vx;
			vy     = node0[node_num].vy;
			vz     = node0[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz)/(R*gam*temp + SMALL);
			node0[node_num].mach = sqrt(fdum);
		   }
		}

		if(!vmag_read_flag && incompressible_flag)
		{
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			vx     = node0[node_num].vx;
			vy     = node0[node_num].vy;
			vz     = node0[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz);
			node0[node_num].vmag = sqrt(fdum);
		   }
		}
	}

/* Create the connectivity and nodal coordinates for a standard finite difference uniform
   mesh.  This mesh will not change and applies to all data over all time. */

	check = fdMesher( connecter, coord, node);
	if(!check) printf( " Problems with fdMesher \n");

/* Create the normal vectors for the above mesh. */

	if(flag_3D)
	{
	    if(!gr_optimize_flag)
	    {
		check = ConnectSurfMesher( connecter, el_matl, node);
		if(!check) printf( " Problems with ConnectSurfMesher \n");
	    }
	    else
	    {
		coord_rescale = 10000.0;
		cross_sec_left_right = 10.0; 
		cross_sec_up_down = 10.0; 
		cross_sec_in_out = 10.0; 
		check = ConnectSurfMesher2( connecter, el_matl, node);
		if(!check) printf( " Problems with ConnectSurfMesher \n");
	    }

	    check = fdnormal_vectors(connecter, coord, norm );
	    if(!check) printf( " Problems with normal_vectors \n");
	}

	if(flag_2D) dum = numel0*npel2 + numnp + 2;
	else dum = (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2;

	check = bcset( &bc, (mem_int + dum), node);
	if(!check) printf( " Problems with bcset\n");

/* Allocate memory for the bodies in the flow.  */

	/*printf(  "pppp  %7d\n", num_bodies);*/

	if(num_bodies)
	{
	    body = (BODY *)realloc(body,num_bodies*sizeof(BODY));
	    if(!body )
	    {
		printf( "failed to allocate memory for body structures\n ");
		exit(1);
	    }
	    memset(body,0,num_bodies*sizeof(BODY));

	    dum = 2*bc.num_noslp[0].xm;   /* twice the amount necessary, just in case. */
	    for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	    {
		body[bod_num].nd = (J_I1_I2_INDX *)realloc(body[bod_num].nd, dum*sizeof(J_I1_I2_INDX));
		body[bod_num].coord = (double *)realloc(body[bod_num].coord, 2*dum*nsd*sizeof(double));
	    }
	}

	if(flag_2D) dum = numel0*npel2 + numnp + 2 + 2*numnp;
	else dum = (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2 + 2*numnp;

	check = bcset_noslip( &bc, body, (mem_int+dum), node);
	if(!check) printf( " Problems with bcset_noslip\n");

/* This is only used for generating points which draw the body. */

	if(num_bodies)
	{
	    check = create_body( body);
	    if(!check) printf( " Problems with create_body\n");
	}

/* Search for extreme values */
 
/* In mesh viewer, search for extreme values of nodal points, displacements
   and stresss and strains to obtain viewing parameters and set color
   assignments.  Also initialize variables */

	check = coord_parameter( coord);
	if(!check) printf( " Problems with coord_parameter \n");

	if(path.read_flag)
	{
	    for( i = 0; i < path.nstep_in; ++i)
	    {
		for( j = 0; j < path.num_pt0; ++j)
		{
		    dum = path.num_pt0*i + j;
		    path.pt[dum].x /= coord_rescale;
		    path.pt[dum].y /= coord_rescale;
		    path.pt[dum].z /= coord_rescale;
		    /*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, path.pt[dum].x, path.pt[dum].y, path.pt[dum].z, coord_rescale);*/
		}
	    }
	}
	if(path0.read_flag)
	{
	    for( i = 0; i < path0.nstep_in; ++i)
	    {
		for( j = 0; j < path0.num_pt0; ++j)
		{
		    dum = path0.num_pt0*i + j;
		    path0.pt[dum].x /= coord_rescale;
		    path0.pt[dum].y /= coord_rescale;
		    path0.pt[dum].z /= coord_rescale;
		    /*printf(  "vvvv  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, path0.pt[dum].x, path0.pt[dum].y, path0.pt[dum].z, coord_rescale);*/
		}
	    }
	}


	if(streak.read_flag)
	{
	    for( i = 0; i < streak.nstep_in; ++i)
	    {
		for( j = 0; j < streak.num_pt0; ++j)
		{
		    dum = streak.num_pt0*i + j;
		    streak.pt[dum].x /= coord_rescale;
		    streak.pt[dum].y /= coord_rescale;
		    streak.pt[dum].z /= coord_rescale;
		    /*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, streak.pt[dum].x, streak.pt[dum].y, streak.pt[dum].z, coord_rescale);*/
		}
	    }
	}
	if(streak0.read_flag)
	{
	    for( i = 0; i < streak0.nstep_in; ++i)
	    {
		for( j = 0; j < streak0.num_pt0; ++j)
		{
		    dum = streak0.num_pt0*i + j;
		    streak0.pt[dum].x /= coord_rescale;
		    streak0.pt[dum].y /= coord_rescale;
		    streak0.pt[dum].z /= coord_rescale;
		    /*printf(  "vvvv  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, streak0.pt[dum].x, streak0.pt[dum].y, streak0.pt[dum].z, coord_rescale);*/
		}
	    }
	}

	if(stream.read_flag)
	{
	    for( i = 0; i < stream.nstep_in; ++i)
	    {
		for( j = 0; j < stream.num_pt0; ++j)
		{
		    dum = stream.num_pt0*i + j;
		    stream.pt[dum].x /= coord_rescale;
		    stream.pt[dum].y /= coord_rescale;
		    stream.pt[dum].z /= coord_rescale;
		    /*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, stream.pt[dum].x, stream.pt[dum].y, stream.pt[dum].z, coord_rescale);*/
		}
	    }
	}
	if(stream0.read_flag)
	{
	    for( i = 0; i < stream0.nstep_in; ++i)
	    {
		for( j = 0; j < stream0.num_pt0; ++j)
		{
		    dum = stream0.num_pt0*i + j;
		    stream0.pt[dum].x /= coord_rescale;
		    stream0.pt[dum].y /= coord_rescale;
		    stream0.pt[dum].z /= coord_rescale;
		    /*printf(  "vvvv  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, stream0.pt[dum].x, stream0.pt[dum].y, stream0.pt[dum].z, coord_rescale);*/
		}
	    }
	}

	for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	{
	    for( pt = 0; pt < 2*body[bod_num].num_points; ++pt )
	    {
		body[bod_num].coord[2*pt] /= coord_rescale;
		body[bod_num].coord[2*pt+1] /= coord_rescale;
	    }
	}


	if( input_flag )
	{
		check = fdparameter( node0);
		if(!check) printf( " Problems with fdparameter \n");

		check = cfd_set( bc, velocity_vec0, node0, node_color0, node_div0);
		if(!check) printf( " Problems with cfd_set \n");
	}

	if( post_flag )
	{
		check = fdparameter( node);
		if(!check) printf( " Problems with fdparameter \n");

		check = cfd_set( bc, velocity_vec, node, node_color, node_div);
		if(!check) printf( " Problems with cfd_set \n");
	}

	AxesList = glGenLists(1);
	agvMakeAxesList(AxesList);

/* create display list for displacement */

	BoundList[0] = glGenLists(1);
	BoundList[1] = glGenLists(1);
	BoundList[2] = glGenLists(1);
	BoundList[3] = glGenLists(1);
	BoundList[4] = glGenLists(1);
	BoundList[5] = glGenLists(1);
	BoundList[6] = glGenLists(1);
	BoundList[7] = glGenLists(1);
	BoundList[8] = glGenLists(1);
	BoundList[9] = glGenLists(1);
	bound_vectors(BoundList, bc, coord);

	if(num_bodies)
	{
	    BodyList = glGenLists(1);
	    bodies_in_flow(BodyList, body);
	}

	if( post_flag )
	{
/* create display list for velocity grapics vectors on post mesh*/

	    VelocityList = glGenLists(1);
	    velocity_vectors(VelocityList, bc, coord, velocity_vec);
	    if(path.read_flag)
	    {
		PathlineList = glGenLists(1);
		flowline_points(PathlineList, path);
	    }
	    if(streak.read_flag)
	    {
		StreaklineList = glGenLists(1);
		flowline_points(StreaklineList, streak);
	    }
	    if(stream.read_flag)
	    {
		StreamlineList = glGenLists(1);
		flowline_points(StreamlineList, stream);
	    }
	}

	if( input_flag )
	{

/* create display list for velocity grapics vectors on input mesh*/

	    VelocityList0 = glGenLists(1);
	    velocity_vectors(VelocityList0, bc, coord, velocity_vec0);
	    if(path0.read_flag)
	    {
		PathlineList0 = glGenLists(1);
		flowline_points(PathlineList0, path0);
	    }
	    if(streak0.read_flag)
	    {
		StreaklineList0 = glGenLists(1);
		flowline_points(StreaklineList0, streak0);
	    }
	    if(stream0.read_flag)
	    {
		StreamlineList0 = glGenLists(1);
		flowline_points(StreamlineList0, stream0);
	    }
	}

	return 1;
}
