/*
    This program calculates and writes the parameters for
    the CFD GUI.
  
    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>

#define init_far0      -2.0

extern int nmat, numnp, numel, dof, nx, ny, nz;
extern double time_current, time_current_input, time_current_post;
extern int nstep, cstep, cstep_input, cstep_post;
extern double step_sizex, step_sizey, step_sizez;
extern double left, right, top, bottom, near, far, fscale, coord_rescale, line_rescale;
extern double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
        cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;
extern int control_height, control_width, mesh_height, mesh_width;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
extern double AxisLength_x, AxisLength_y, AxisLength_z,
	AxisLength_max;
extern double AxisPoint_step;
extern double amplify_step0;

extern double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far;
extern int nxyz_max, nx_inc, ny_inc, nz_inc,
	nxyz_max0, nx_inc0, ny_inc0, nz_inc0;
extern GR_NODE del_node, max_node, min_node;
extern double absolute_max_coord;
extern GLfloat line_width, point_size,
	line_width0, point_size0;


int fdparameter( GR_NODE *node)
{
/* Find the extreme values for all nodal quantities.  Also, based on the dimensions
   of the mesh, find the parameters which define mesh window as well as the placement
   of the mesh inside the window.  Finally, write it all out to the file "fdview.dat".

   			Last Update 3/21/08

*/

	int i, j, check, node_num;
	int dum, dum1;
 	IJK_GR_NODE node_max, node_min;
	FILE *fddata;

/*   fddata contains all the parameters and extreme values 
*/
	fddata = fopen( "fdview.dat","w" );

/* Initialize parameters */

	step_sizex = .1; step_sizey = .1; step_sizez = .1;
	init_right = - BIG; init_left = BIG;
	init_top = - BIG; init_bottom = BIG;
	init_near = - BIG; init_far = init_far0; true_far = BIG;
	 
 	node_min.dns.i    = 0; node_max.dns.i    = 0;
 	node_min.dns.j    = 0; node_max.dns.j    = 0;
 	node_min.dns.k    = 0; node_max.dns.k    = 0;
 	node_min.prs.i    = 0; node_max.prs.i    = 0;
 	node_min.prs.j    = 0; node_max.prs.j    = 0;
 	node_min.prs.k    = 0; node_max.prs.k    = 0;
 	node_min.temp.i   = 0; node_max.temp.i   = 0;
 	node_min.temp.j   = 0; node_max.temp.j   = 0;
 	node_min.temp.k   = 0; node_max.temp.k   = 0;
 	node_min.ene.i    = 0; node_max.ene.i    = 0;
 	node_min.ene.j    = 0; node_max.ene.j    = 0;
 	node_min.ene.k    = 0; node_max.ene.k    = 0;
 	node_min.enthal.i = 0; node_max.enthal.i = 0;
 	node_min.enthal.j = 0; node_max.enthal.j = 0;
 	node_min.enthal.k = 0; node_max.enthal.k = 0;
 	node_min.vx.i     = 0; node_max.vx.i     = 0;
 	node_min.vx.j     = 0; node_max.vx.j     = 0;
 	node_min.vx.k     = 0; node_max.vx.k     = 0;
 	node_min.vy.i     = 0; node_max.vy.i     = 0;
 	node_min.vy.j     = 0; node_max.vy.j     = 0;
 	node_min.vy.k     = 0; node_max.vy.k     = 0;
 	node_min.vz.i     = 0; node_max.vz.i     = 0;
 	node_min.vz.j     = 0; node_max.vz.j     = 0;
 	node_min.vz.k     = 0; node_max.vz.k     = 0;
 	node_min.mach.i   = 0; node_max.mach.i   = 0;
 	node_min.mach.j   = 0; node_max.mach.j   = 0;
 	node_min.mach.k   = 0; node_max.mach.k   = 0;
 	node_min.vmag.i   = 0; node_max.vmag.i   = 0;
 	node_min.vmag.j   = 0; node_max.vmag.j   = 0;
 	node_min.vmag.k   = 0; node_max.vmag.k   = 0;

/* Initialize largest and smallest quantities of density, temperature, pressure,
   velocity x-y-z, enthalpy, energy, and mach number(or velocity magnitude) */

	max_node.dns    = - BIG; min_node.dns    = BIG;
	max_node.prs    = - BIG; min_node.prs    = BIG;
	max_node.temp   = - BIG; min_node.temp   = BIG;
	max_node.ene    = - BIG; min_node.ene    = BIG;
	max_node.enthal = - BIG; min_node.enthal = BIG;
	max_node.vx     = - BIG; min_node.vx     = BIG;
	max_node.vy     = - BIG; min_node.vy     = BIG;
	max_node.vz     = - BIG; min_node.vz     = BIG;
	max_node.mach   = - BIG; min_node.mach   = BIG;
	max_node.vmag   = - BIG; min_node.vmag   = BIG;

/* To improve the display of the mesh, I don't want to draw the velocity vectors at
   every nodal point.  So I will skip points based on the code below.
*/

	nx_inc = 1;
	ny_inc = 1;
	nz_inc = 1;
	if( nx > 20 )
	{
		nx_inc = (int)nx/20;
		dum = nx%20;
		if(dum > 5) nx_inc += 1;
	}
	if( ny > 20 )
	{
		ny_inc = (int)ny/20;
		dum = ny%20;
		if(dum > 5) ny_inc += 1;
	}
	if( nz > 20 )
	{
		nz_inc = (int)nz/20;
		dum = nz%20;
		if(dum > 5) nz_inc += 1;
	}

/* Similar to the above, the display of the velocity vectors and boundary conditions
   are adjusted to the thickness of the lines which are used to draw them.  The CFD
   meshes tend to have a great number of nodal points which cause the display of
   lines which are too thick to not appear as individual lines but one solid block.
   So thickness will depend on the dimension with the largest number of nodal points.

   In addition, the lengths of the lines which draw the velocity vectors and
   boundary condition lines are shortened based on the number of nodal points.
*/ 
	nxyz_max = nx;
	if( nxyz_max < ny) nxyz_max = ny;
	if( nxyz_max < nz) nxyz_max = nz;

	line_width = 4.0;
	line_rescale = 4.0;
	if( nxyz_max > 20 )
	{
		line_width = 60.0/((double)nxyz_max);
		line_rescale = 0.05*((double)nxyz_max)/AxisLength_max;
	}
	point_size = 3.0*line_width;
	/*printf("  %5d %5d %5d %5d %16.8e %16.8e\n",
		nx_inc, ny_inc, nz_inc, nxyz_max, line_rescale, AxisLength_max);*/


/* Search for extreme values of density, temperature, pressure, velocity x-y-z, enthalpy,
  energy, and mach number(or velocity magnitude).  But ignore nodes which are discarded. */

	for( node_num = 0; node_num < numnp; ++node_num )
	{
		if(node[node_num].bc_type < 100)
		{
		    if( max_node.dns < node[node_num].dns )
		    {
			max_node.dns = node[node_num].dns;
			node_max.dns.i = node[node_num].i;
			node_max.dns.j = node[node_num].j;
			node_max.dns.k = node[node_num].k;
		    }
		    if( min_node.dns > node[node_num].dns )
		    {
			min_node.dns = node[node_num].dns;
			node_min.dns.i = node[node_num].i;
			node_min.dns.j = node[node_num].j;
			node_min.dns.k = node[node_num].k;
		    }
		    if( max_node.prs < node[node_num].prs )
		    {
			max_node.prs = node[node_num].prs;
			node_max.prs.i = node[node_num].i;
			node_max.prs.j = node[node_num].j;
			node_max.prs.k = node[node_num].k;
		    }
		    if( min_node.prs > node[node_num].prs )
		    {
			min_node.prs = node[node_num].prs;
			node_min.prs.i = node[node_num].i;
			node_min.prs.j = node[node_num].j;
			node_min.prs.k = node[node_num].k;
		    }
		    if( max_node.temp < node[node_num].temp )
		    {
			max_node.temp = node[node_num].temp;
			node_max.temp.i = node[node_num].i;
			node_max.temp.j = node[node_num].j;
			node_max.temp.k = node[node_num].k;
		    }
		    if( min_node.temp > node[node_num].temp )
		    {
			min_node.temp = node[node_num].temp;
			node_min.temp.i = node[node_num].i;
			node_min.temp.j = node[node_num].j;
			node_min.temp.k = node[node_num].k;
		    }
		    if( max_node.enthal < node[node_num].enthal )
		    {
			max_node.enthal = node[node_num].enthal;
			node_max.enthal.i = node[node_num].i;
			node_max.enthal.j = node[node_num].j;
			node_max.enthal.k = node[node_num].k;
		    }
		    if( min_node.enthal > node[node_num].enthal )
		    {
			min_node.enthal = node[node_num].enthal;
			node_min.enthal.i = node[node_num].i;
			node_min.enthal.j = node[node_num].j;
			node_min.enthal.k = node[node_num].k;
		    }
		    if( max_node.ene < node[node_num].ene )
		    {
			max_node.ene = node[node_num].ene;
			node_max.ene.i = node[node_num].i;
			node_max.ene.j = node[node_num].j;
			node_max.ene.k = node[node_num].k;
		    }
		    if( min_node.ene > node[node_num].ene )
		    {
			min_node.ene = node[node_num].ene;
			node_min.ene.i = node[node_num].i;
			node_min.ene.j = node[node_num].j;
			node_min.ene.k = node[node_num].k;
		    }
		    if( max_node.vx < node[node_num].vx )
		    {
			max_node.vx = node[node_num].vx;
			node_max.vx.i = node[node_num].i;
			node_max.vx.j = node[node_num].j;
			node_max.vx.k = node[node_num].k;
		    }
		    if( min_node.vx > node[node_num].vx )
		    {
			min_node.vx = node[node_num].vx;
			node_min.vx.i = node[node_num].i;
			node_min.vx.j = node[node_num].j;
			node_min.vx.k = node[node_num].k;
		    }
		    if( max_node.vy < node[node_num].vy )
		    {
			max_node.vy = node[node_num].vy;
			node_max.vy.i = node[node_num].i;
			node_max.vy.j = node[node_num].j;
			node_max.vy.k = node[node_num].k;
		    }
		    if( min_node.vy > node[node_num].vy )
		    {
			min_node.vy = node[node_num].vy;
			node_min.vy.i = node[node_num].i;
			node_min.vy.j = node[node_num].j;
			node_min.vy.k = node[node_num].k;
		    }
		    if( max_node.vz < node[node_num].vz )
		    {
			max_node.vz = node[node_num].vz;
			node_max.vz.i = node[node_num].i;
			node_max.vz.j = node[node_num].j;
			node_max.vz.k = node[node_num].k;
		    }
		    if( min_node.vz > node[node_num].vz )
		    {
			min_node.vz = node[node_num].vz;
			node_min.vz.i = node[node_num].i;
			node_min.vz.j = node[node_num].j;
			node_min.vz.k = node[node_num].k;
		    }
		    if( max_node.mach < node[node_num].mach )
		    {
			max_node.mach = node[node_num].mach;
			node_max.mach.i = node[node_num].i;
			node_max.mach.j = node[node_num].j;
			node_max.mach.k = node[node_num].k;
		    }
		    if( min_node.mach > node[node_num].mach )
		    {
			min_node.mach = node[node_num].mach;
			node_min.mach.i = node[node_num].i;
			node_min.mach.j = node[node_num].j;
			node_min.mach.k = node[node_num].k;
		    }
		    if( max_node.vmag < node[node_num].vmag )
		    {
			max_node.vmag = node[node_num].vmag;
			node_max.vmag.i = node[node_num].i;
			node_max.vmag.j = node[node_num].j;
			node_max.vmag.k = node[node_num].k;
		    }
		    if( min_node.vmag > node[node_num].vmag )
		    {
			min_node.vmag = node[node_num].vmag;
			node_min.vmag.i = node[node_num].i;
			node_min.vmag.j = node[node_num].j;
			node_min.vmag.k = node[node_num].k;
		    }
		}
	}


/* Print the above data in the file "fdview.dat" */

	fprintf( fddata, "                          index min       node       index max      node\n");
	fprintf( fddata, "                         i     j     k    min      i     j     k    max       min            max\n");
	fprintf( fddata, "\n");
	dum = node_min.dns.i*ny*nz + node_min.dns.j*nz + node_min.dns.k;
	dum1 = node_max.dns.i*ny*nz + node_max.dns.j*nz + node_max.dns.k;
	fprintf( fddata,"density              %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.dns.i, node_min.dns.j, node_min.dns.k, dum,
		node_max.dns.i, node_max.dns.j, node_max.dns.k, dum1,
		min_node.dns, max_node.dns);
	dum = node_min.temp.i*ny*nz + node_min.temp.j*nz + node_min.temp.k;
	dum1 = node_max.temp.i*ny*nz + node_max.temp.j*nz + node_max.temp.k;
	fprintf( fddata,"temperature          %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.temp.i, node_min.temp.j, node_min.temp.k, dum,
		node_max.temp.i, node_max.temp.j, node_max.temp.k, dum1,
		min_node.temp, max_node.temp);
	dum = node_min.prs.i*ny*nz + node_min.prs.j*nz + node_min.prs.k;
	dum1 = node_max.prs.i*ny*nz + node_max.prs.j*nz + node_max.prs.k;
	fprintf( fddata,"pressure             %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.prs.i, node_min.prs.j, node_min.prs.k, dum,
		node_max.prs.i, node_max.prs.j, node_max.prs.k, dum1,
		min_node.prs, max_node.prs);
	dum = node_min.vx.i*ny*nz + node_min.vx.j*nz + node_min.vx.k;
	dum1 = node_max.vx.i*ny*nz + node_max.vx.j*nz + node_max.vx.k;
	fprintf( fddata,"velocity x           %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.vx.i, node_min.vx.j, node_min.vx.k, dum,
		node_max.vx.i, node_max.vx.j, node_max.vx.k, dum1,
		min_node.vx, max_node.vx);
	dum = node_min.vy.i*ny*nz + node_min.vy.j*nz + node_min.vy.k;
	dum1 = node_max.vy.i*ny*nz + node_max.vy.j*nz + node_max.vy.k;
	fprintf( fddata,"velocity y           %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.vy.i, node_min.vy.j, node_min.vy.k, dum,
		node_max.vy.i, node_max.vy.j, node_max.vy.k, dum1,
		min_node.vy, max_node.vy);
	dum = node_min.vz.i*ny*nz + node_min.vz.j*nz + node_min.vz.k;
	dum1 = node_max.vz.i*ny*nz + node_max.vz.j*nz + node_max.vz.k;
	fprintf( fddata,"velocity z           %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.vz.i, node_min.vz.j, node_min.vz.k, dum,
		node_max.vz.i, node_max.vz.j, node_max.vz.k, dum1,
		min_node.vz, max_node.vz);
	dum = node_min.enthal.i*ny*nz + node_min.enthal.j*nz + node_min.enthal.k;
	dum1 = node_max.enthal.i*ny*nz + node_max.enthal.j*nz + node_max.enthal.k;
	fprintf( fddata,"enthalpy             %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.enthal.i, node_min.enthal.j, node_min.enthal.k, dum,
		node_max.enthal.i, node_max.enthal.j, node_max.enthal.k, dum1,
		min_node.enthal, max_node.enthal);
	dum = node_min.ene.i*ny*nz + node_min.ene.j*nz + node_min.ene.k;
	dum1 = node_max.ene.i*ny*nz + node_max.ene.j*nz + node_max.ene.k;
	fprintf( fddata,"energy               %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.ene.i, node_min.ene.j, node_min.ene.k, dum,
		node_max.ene.i, node_max.ene.j, node_max.ene.k, dum1,
		min_node.ene, max_node.ene);
	dum = node_min.mach.i*ny*nz + node_min.mach.j*nz + node_min.mach.k;
	dum1 = node_max.mach.i*ny*nz + node_max.mach.j*nz + node_max.mach.k;
	fprintf( fddata,"mach number          %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.mach.i, node_min.mach.j, node_min.mach.k, dum,
		node_max.mach.i, node_max.mach.j, node_max.mach.k, dum1,
		min_node.mach, max_node.mach);
	dum = node_min.vmag.i*ny*nz + node_min.vmag.j*nz + node_min.vmag.k;
	dum1 = node_max.vmag.i*ny*nz + node_max.vmag.j*nz + node_max.vmag.k;
	fprintf( fddata,"velocity magnitude   %5d %5d %5d   %5d %5d %5d %5d   %5d %14.6e %14.6e\n",
		node_min.vmag.i, node_min.vmag.j, node_min.vmag.k, dum,
		node_max.vmag.i, node_max.vmag.j, node_max.vmag.k, dum1,
		min_node.vmag, max_node.vmag);
	fprintf( fddata,"\n");
	fprintf( fddata,"Orthographic viewport parameters(right, left, top, bottom, near, far)\n ");
	fprintf( fddata,"%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n", ortho_right, ortho_left,
		ortho_top, ortho_bottom, near, 1000.0);
	fprintf( fddata,"Perspective viewport parameters( mesh width and height)\n ");
	fprintf( fddata,"%6d %6d\n", mesh_width, mesh_height);
	fprintf( fddata,"Step sizes in x, y, z\n ");
	fprintf( fddata,"%14.6e %14.6e %14.6e\n",step_sizex, step_sizey, step_sizez);
	fprintf( fddata,"Amplification size\n ");
	fprintf( fddata,"%14.6e\n",amplify_step0);
	fprintf( fddata,"Current Step, Current Time\n ");
	fprintf( fddata,"%7d %14.6e\n",cstep, time_current);
	fprintf( fddata,"nx_inc, ny_inc, nz_inc\n ");
	fprintf( fddata,"%5d %5d %5d\n", nx_inc, ny_inc, nz_inc);
	nx_inc0 = nx_inc; ny_inc0 = ny_inc; nz_inc0 = nz_inc;
	fprintf( fddata,"line width, point size\n ");
	fprintf( fddata,"%14.6e %14.6e\n", line_width, point_size);
	line_width0 = line_width;
	point_size0 = point_size;

	fclose( fddata );

  	return 1;    /* ANSI C requires main to return int. */
}



int fdparameter_anim( GR_NODE *node)
{
/* Find the extreme values for all nodal quantities.  This is all that is needed when
   doing the parameters of an animation.

   			Last Update 5/30/08
*/

	int i, j, check, node_num;
	int dum, dum1;
 	IJK_GR_NODE node_max, node_min;

/* Initialize largest and smallest quantities of density, temperature, pressure,
   velocity x-y-z, enthalpy, energy, and mach number(or velocity magnitude) */

	max_node.dns    = - BIG; min_node.dns    = BIG;
	max_node.prs    = - BIG; min_node.prs    = BIG;
	max_node.temp   = - BIG; min_node.temp   = BIG;
	max_node.ene    = - BIG; min_node.ene    = BIG;
	max_node.enthal = - BIG; min_node.enthal = BIG;
	max_node.vx     = - BIG; min_node.vx     = BIG;
	max_node.vy     = - BIG; min_node.vy     = BIG;
	max_node.vz     = - BIG; min_node.vz     = BIG;
	max_node.mach   = - BIG; min_node.mach   = BIG;
	max_node.vmag   = - BIG; min_node.vmag   = BIG;

/* Search for extreme values of density, temperature, pressure, velocity x-y-z, enthalpy,
  energy, and mach number(or velocity magnitude).  But ignore nodes which are discarded. */

	for( node_num = 0; node_num < numnp; ++node_num )
	{
		if(node[node_num].bc_type < 100)
		{
		    if( max_node.dns < node[node_num].dns )
		    {
			max_node.dns = node[node_num].dns;
			node_max.dns.i = node[node_num].i;
			node_max.dns.j = node[node_num].j;
			node_max.dns.k = node[node_num].k;
		    }
		    if( min_node.dns > node[node_num].dns )
		    {
			min_node.dns = node[node_num].dns;
			node_min.dns.i = node[node_num].i;
			node_min.dns.j = node[node_num].j;
			node_min.dns.k = node[node_num].k;
		    }
		    if( max_node.prs < node[node_num].prs )
		    {
			max_node.prs = node[node_num].prs;
			node_max.prs.i = node[node_num].i;
			node_max.prs.j = node[node_num].j;
			node_max.prs.k = node[node_num].k;
		    }
		    if( min_node.prs > node[node_num].prs )
		    {
			min_node.prs = node[node_num].prs;
			node_min.prs.i = node[node_num].i;
			node_min.prs.j = node[node_num].j;
			node_min.prs.k = node[node_num].k;
		    }
		    if( max_node.temp < node[node_num].temp )
		    {
			max_node.temp = node[node_num].temp;
			node_max.temp.i = node[node_num].i;
			node_max.temp.j = node[node_num].j;
			node_max.temp.k = node[node_num].k;
		    }
		    if( min_node.temp > node[node_num].temp )
		    {
			min_node.temp = node[node_num].temp;
			node_min.temp.i = node[node_num].i;
			node_min.temp.j = node[node_num].j;
			node_min.temp.k = node[node_num].k;
		    }
		    if( max_node.enthal < node[node_num].enthal )
		    {
			max_node.enthal = node[node_num].enthal;
			node_max.enthal.i = node[node_num].i;
			node_max.enthal.j = node[node_num].j;
			node_max.enthal.k = node[node_num].k;
		    }
		    if( min_node.enthal > node[node_num].enthal )
		    {
			min_node.enthal = node[node_num].enthal;
			node_min.enthal.i = node[node_num].i;
			node_min.enthal.j = node[node_num].j;
			node_min.enthal.k = node[node_num].k;
		    }
		    if( max_node.ene < node[node_num].ene )
		    {
			max_node.ene = node[node_num].ene;
			node_max.ene.i = node[node_num].i;
			node_max.ene.j = node[node_num].j;
			node_max.ene.k = node[node_num].k;
		    }
		    if( min_node.ene > node[node_num].ene )
		    {
			min_node.ene = node[node_num].ene;
			node_min.ene.i = node[node_num].i;
			node_min.ene.j = node[node_num].j;
			node_min.ene.k = node[node_num].k;
		    }
		    if( max_node.vx < node[node_num].vx )
		    {
			max_node.vx = node[node_num].vx;
			node_max.vx.i = node[node_num].i;
			node_max.vx.j = node[node_num].j;
			node_max.vx.k = node[node_num].k;
		    }
		    if( min_node.vx > node[node_num].vx )
		    {
			min_node.vx = node[node_num].vx;
			node_min.vx.i = node[node_num].i;
			node_min.vx.j = node[node_num].j;
			node_min.vx.k = node[node_num].k;
		    }
		    if( max_node.vy < node[node_num].vy )
		    {
			max_node.vy = node[node_num].vy;
			node_max.vy.i = node[node_num].i;
			node_max.vy.j = node[node_num].j;
			node_max.vy.k = node[node_num].k;
		    }
		    if( min_node.vy > node[node_num].vy )
		    {
			min_node.vy = node[node_num].vy;
			node_min.vy.i = node[node_num].i;
			node_min.vy.j = node[node_num].j;
			node_min.vy.k = node[node_num].k;
		    }
		    if( max_node.vz < node[node_num].vz )
		    {
			max_node.vz = node[node_num].vz;
			node_max.vz.i = node[node_num].i;
			node_max.vz.j = node[node_num].j;
			node_max.vz.k = node[node_num].k;
		    }
		    if( min_node.vz > node[node_num].vz )
		    {
			min_node.vz = node[node_num].vz;
			node_min.vz.i = node[node_num].i;
			node_min.vz.j = node[node_num].j;
			node_min.vz.k = node[node_num].k;
		    }
		    if( max_node.mach < node[node_num].mach )
		    {
			max_node.mach = node[node_num].mach;
			node_max.mach.i = node[node_num].i;
			node_max.mach.j = node[node_num].j;
			node_max.mach.k = node[node_num].k;
		    }
		    if( min_node.mach > node[node_num].mach )
		    {
			min_node.mach = node[node_num].mach;
			node_min.mach.i = node[node_num].i;
			node_min.mach.j = node[node_num].j;
			node_min.mach.k = node[node_num].k;
		    }
		    if( max_node.vmag < node[node_num].vmag )
		    {
			max_node.vmag = node[node_num].vmag;
			node_max.vmag.i = node[node_num].i;
			node_max.vmag.j = node[node_num].j;
			node_max.vmag.k = node[node_num].k;
		    }
		    if( min_node.vmag > node[node_num].vmag )
		    {
			min_node.vmag = node[node_num].vmag;
			node_min.vmag.i = node[node_num].i;
			node_min.vmag.j = node[node_num].j;
			node_min.vmag.k = node[node_num].k;
		    }
		}
	}

  	return 1;    /* ANSI C requires main to return int. */
}
