/*
    This program writes out the parameters of of the meshview window.
    This is useful for when you want to do an animation and want to set
    the position and window dimensions of the animation.
  
                     Last Update 6/30/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>

#define init_far0      -2.0

extern int nmat, numnp, numel, dof, nx, ny, nz;
extern double time_current, time_current_input, time_current_post;
extern int nstep, cstep, cstep_input, cstep_post;
extern double step_sizex, step_sizey, step_sizez;
extern double left, right, top, bottom, near, far, fscale, coord_rescale, line_rescale;
extern int nxyz_max, nx_inc, ny_inc, nz_inc,
	nxyz_max0, nx_inc0, ny_inc0, nz_inc0;
extern int control_height, control_width, mesh_height, mesh_width;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom;
extern double xAngle, yAngle, zAngle;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
extern double AxisLength_x, AxisLength_y, AxisLength_z,
	AxisLength_max;
extern double AxisPoint_step;
extern double amplify_step0;
extern ANIM_DAT anim[1];

extern double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far;
extern GR_NODE del_node, max_node, min_node;
extern GLfloat line_width, point_size,
	line_width0, point_size0;

void fdWriteparameter_anim( void)
{
	int i, j, check;
	int dum, dum1;
	IJK_GR_NODE node_max, node_min;
	char char_dum[20], char_dum2[5], buf[ BUFSIZ ];
	double fdum;
	FILE *fddata;

	printf( "Orthographic viewport parameters(right, left, top, bottom, near, far)\n ");
	printf( "%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n", ortho_right, ortho_left,
		ortho_top, ortho_bottom, near, 1000.0);
	printf( "Perspective viewport parameters( mesh width and height)\n ");
	printf( "%6d %6d\n", mesh_width, mesh_height);
	printf( "Position left_right, up_down, in_out\n ");
	printf( "%14.6e %14.6e %14.6e\n", left_right, up_down, in_out);
	printf( "Rotation x, y, z\n ");
	printf( "%14.6e %14.6e %14.6e\n", xAngle, yAngle, zAngle);
	printf( "Current Step, Current Time\n ");
	printf( "%7d %14.6e\n",cstep, time_current);
	printf( "nx_inc, ny_inc, nz_inc\n ");
	printf( "%5d %5d %5d\n", nx_inc, ny_inc, nz_inc);
	printf( "line width, point size\n ");
	printf( "%14.6e %14.6e\n", line_width, point_size);
	printf( "Quantities to animate(pick 3, or write none)\n");
	printf( "Options for 1st quantity are: density, temperature, pressure, vx,");
	printf( " vx, vy, vz, enthalpy, energy, mach, vmag\n");
	printf( "Options for 2nd quantity are: pathlines, streaklines, streamlines\n");
	printf( "Options for 3rd quantity are: velocity (may add more later)\n");
	printf( "   none streaklines velocity\n");

/* Write data to the file "fdview.anim" */

	fddata = fopen( "fdview.anim","w" );

	fprintf( fddata, "Orthographic viewport parameters(right, left, top, bottom, near, far)\n ");
	fprintf( fddata, "%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n", ortho_right, ortho_left,
		ortho_top, ortho_bottom, near, 1000.0);
	fprintf( fddata, "Perspective viewport parameters( mesh width and height)\n ");
	fprintf( fddata, "%6d %6d\n", mesh_width, mesh_height);
	fprintf( fddata, "Position left_right, up_down, in_out\n ");
	fprintf( fddata, "%14.6e %14.6e %14.6e\n", left_right, up_down, in_out);
	fprintf( fddata, "Rotation x, y, z\n ");
	fprintf( fddata, "%14.6e %14.6e %14.6e\n", xAngle, yAngle, zAngle);
	fprintf( fddata, "Current Step, Current Time\n ");
	fprintf( fddata, "%7d %14.6e\n",cstep, time_current);
	fprintf( fddata, "nx_inc, ny_inc, nz_inc\n ");
	fprintf( fddata, "%5d %5d %5d\n", nx_inc, ny_inc, nz_inc);
	fprintf( fddata, "line width, point size\n ");
	fprintf( fddata, "%14.6e %14.6e\n", line_width, point_size);
	fprintf( fddata, "Quantities to animate(pick 3, or write none)\n");
	fprintf( fddata, "Options for 1st quantity are: density, temperature, pressure,");
	fprintf( fddata, " vx, vy, vz, enthalpy, energy, mach, vmag\n");
	fprintf( fddata, "Options for 2nd quantity are: pathlines, streaklines, streamlines\n");
	fprintf( fddata, "Options for 3rd quantity are: velocity (may add more later)\n");
	fprintf( fddata, "   none streaklines velocity\n");

	fclose(fddata);
}

void fdReadparameter_anim( void)
{
	int i, j, check;
	int dum, dum1;
	IJK_GR_NODE node_max, node_min;
	char *ccheck, char_dum[20], char_dum2[5], buf[ BUFSIZ ];
	double fdum;
	FILE *fddata;

	memset(anim[0].color_name,0,30*sizeof(char));
	memset(anim[0].flowtype_name,0,30*sizeof(char));
	memset(anim[0].velocity_name,0,30*sizeof(char));

/* These are the default values when the file "fdview.anim" doesn't exist. */

#if 0

/* I'll use the values calculated from "coord_param.c" as the defaults. 
   The values below are specific for the meshes that I've been using.
*/
	ortho_right = 2.101987e+00;
	ortho_left = -6.742179e-01;
	ortho_top = 2.182564e+00;
	ortho_bottom = -5.936409e-01;
	near = 2.000000e-01;
	mesh_width = 320;
	mesh_height = 320;
	left_right = -6.200000e-01;
	up_down = 7.000000e-02;
	in_out = -7.200000e+00;
#endif

	xAngle = 0.0;
	yAngle = 0.0;
	zAngle= 0.0;
	cstep = 0;
	time_current = 0.0;

/* To improve the display of the mesh, I don't want to draw the velocity vectors at
   every nodal point.  So I will skip points based on the code below.
*/
	nx_inc = 1;
	ny_inc = 1;
	nz_inc = 1;
	if( nx > 20 )
	{
		nx_inc = (int)nx/20;
		dum = nx%20;
		if(dum > 5) nx_inc += 1;
	}
	if( ny > 20 )
	{
		ny_inc = (int)ny/20;
		dum = ny%20;
		if(dum > 5) ny_inc += 1;
	}
	if( nz > 20 )
	{
		nz_inc = (int)nz/20;
		dum = nz%20;
		if(dum > 5) nz_inc += 1;
	}

/* Similar to the above, the display of the velocity vectors and boundary conditions
   are adjusted to the thickness of the lines which are used to draw them.  The CFD
   meshes tend to have a great number of nodal points which cause the display of
   lines which are too thick to not appear as individual lines but one solid block.
   So thickness will depend on the dimension with the largest number of nodal points.

   In addition, the lengths of the lines which draw the velocity vectors and
   boundary condition lines are shortened based on the number of nodal points.
*/
	nxyz_max = nx;
	if( nxyz_max < ny) nxyz_max = ny;
	if( nxyz_max < nz) nxyz_max = nz;

	line_width = 4.0;
	line_rescale = 4.0;
	if( nxyz_max > 20 )
	{
		line_width = 60.0/((double)nxyz_max);
		line_rescale = 0.05*((double)nxyz_max)/AxisLength_max;
	}
	point_size = 3.0*line_width;
	/*printf("  %5d %5d %5d %5d %16.8e %16.8e\n",
		nx_inc, ny_inc, nz_inc, nxyz_max, line_rescale, AxisLength_max);*/

	anim[0].color_choice = 3;
	anim[0].flowtype_choice = 2;
	anim[0].velocity_choice = 2;

/* Read data from the file "fdview.anim" */

	fddata = fopen( "fdview.anim","r" );
	if(fddata == NULL ) {
	    printf("Can't find file fdview.anim\n");
	    ccheck = strncpy( anim[0].color_name, "none", 4);
	    ccheck = strncpy( anim[0].flowtype_name, "streak", 6);
	    ccheck = strncpy( anim[0].velocity_name, "none", 4);
	    anim[0].flowtype_choice = 2;
	    anim[0].color_choice = 9;
	    anim[0].velocity_choice = 0;
	    anim[0].node_print_flag = 1;
	}
	else
	{
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%lf %lf %lf %lf %lf %lf\n", &ortho_right, &ortho_left,
		&ortho_top, &ortho_bottom, &near, &fdum);
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%d %d\n", &mesh_width, &mesh_height);
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%lf %lf %lf\n", &left_right, &up_down, &in_out);
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%lf %lf %lf\n", &xAngle, &yAngle, &zAngle);
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%d %lf\n", &cstep, &time_current);
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%d %d %d\n", &nx_inc, &ny_inc, &nz_inc);
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%f %f\n", &line_width, &point_size);
	    fgets( buf, BUFSIZ, fddata );
	    fgets( buf, BUFSIZ, fddata );
	    fgets( buf, BUFSIZ, fddata );
	    fgets( buf, BUFSIZ, fddata );
	    fscanf( fddata, "%s %s %s\n", anim[0].color_name, anim[0].flowtype_name, anim[0].velocity_name);

	    anim[0].color_choice = 0;
	    check = strncmp(anim[0].color_name,"densit", 6);
	    if(!check) anim[0].color_choice = 1;
	    check = strncmp(anim[0].color_name,"temper", 6);
	    if(!check) anim[0].color_choice = 2;
	    check = strncmp(anim[0].color_name,"pressu", 6);
	    if(!check) anim[0].color_choice = 3;
	    check = strncmp(anim[0].color_name,"vx", 2);
	    if(!check) anim[0].color_choice = 4;
	    check = strncmp(anim[0].color_name,"vy", 2);
	    if(!check) anim[0].color_choice = 5;
	    check = strncmp(anim[0].color_name,"vz", 2);
	    if(!check) anim[0].color_choice = 6;
	    check = strncmp(anim[0].color_name,"enthal", 6);
	    if(!check) anim[0].color_choice = 7;
	    check = strncmp(anim[0].color_name,"energy", 6);
	    if(!check) anim[0].color_choice = 8;
	    check = strncmp(anim[0].color_name,"mach", 4);
	    if(!check) anim[0].color_choice = 9;
	    check = strncmp(anim[0].color_name,"vmag", 4);
	    if(!check) anim[0].color_choice = 10;

	    anim[0].flowtype_choice = 0;
	    check = strncmp(anim[0].flowtype_name,"path", 4);
	    if(!check) anim[0].flowtype_choice = 1;
	    check = strncmp(anim[0].flowtype_name,"streak", 6);
	    if(!check) anim[0].flowtype_choice = 2;
	    check = strncmp(anim[0].flowtype_name,"stream", 6);
	    if(!check) anim[0].flowtype_choice = 3;

	    anim[0].velocity_choice = 0;
	    check = strncmp(anim[0].velocity_name,"veloci", 6);
	    if(!check) anim[0].velocity_choice = 2;

	    fclose(fddata);
	}


	printf( "Orthographic viewport parameters(right, left, top, bottom, near, far)\n ");
	printf( "%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n", ortho_right, ortho_left,
		ortho_top, ortho_bottom, near, 1000.0);
	printf( "Perspective viewport parameters( mesh width and height)\n ");
	printf( "%6d %6d\n", mesh_width, mesh_height);
	printf( "Position left_right, up_down, in_out\n ");
	printf( "%14.6e %14.6e %14.6e\n", left_right, up_down, in_out);
	printf( "Rotation x, y, z\n ");
	printf( "%14.6e %14.6e %14.6e\n", xAngle, yAngle, zAngle);
	printf( "Current Step, Current Time\n ");
	printf( "%7d %14.6e\n",cstep, time_current);
	printf( "nx_inc, ny_inc, nz_inc\n ");
	printf( "%5d %5d %5d\n", nx_inc, ny_inc, nz_inc);
	printf( "line width, point size\n ");
	printf( "%14.6e %14.6e\n", line_width, point_size);
	printf( "Quantities to animate(pick 3, or write none)\n");
	printf( "Options for 1st quantity are: density, temperature, pressure, vx,");
	printf( " vx, vy, vz, enthalpy, energy, mach, vmag\n");
	printf( "Options for 2nd quantity are: pathlines, streaklines, streamlines\n");
	printf( "Options for 3rd quantity are: velocity (may add more later)\n");
	printf( "%s %s %s\n", anim[0].color_name, anim[0].flowtype_name, anim[0].velocity_name);
}

