/*
    This program shows the 2-D or 3-D model of the finite
    difference mesh.
  
                        Last Update 6/28/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "cfdgui.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"
#include "../common_gr/color_gr.h"
#include "../AMR/amr.h"

/* glut header files */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#if LINUX
#include <GL/glx.h>

/* X11 header files. */
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/StdCmap.h>
#include <X11/keysym.h>

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#endif

/* The header files below can be used to incorporate
   widgets into your code */
#if 0
/* X11 Widget header files. */
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <GL/xmesa.h>
#include <GL/MesaDrawingArea.h>

#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <GL/xmesa.h>
#include <GL/MesaMDrawingArea.h>

#define GLwMakeCurrent GLwMMakeCurrent
#endif


/********************* These are all the subroutines **************************/

/******** Data management and calculations ********/


void flowline_points(int , FLOWLINES );

void velocity_vectors(int , BOUND , double *, XYZF *);

void bound_vectors(GLuint *, BOUND , double *);

void agvMakeAxesList(GLuint);

int cfd_set( BOUND , XYZF *, GR_NODE *, I_GR_NODE *, GR_NODE *);

int fdparameter( GR_NODE *);

void fdReadparameter_anim( void);

int coord_parameter( double *);

int compare_body ( const void *,  const void *);

int create_body( BODY *);

int bcset_noslip( BOUND *, BODY *, int *, GR_NODE * );

int bcset( BOUND *, int *, GR_NODE * );

int fdnormal_vectors (int *, double *, NORM * );

int ConnectSurfMesher2( int *, int *, GR_NODE *);

int ConnectSurfMesher( int *, int *, GR_NODE *);

int fdMesher( int *, double *, GR_NODE *);

int qikreader( MATL *, char *, GR_NODE *, FILE *, FLOWLINES, FLOWLINES, FLOWLINES );

int flwreader( MATL *, char *, GR_NODE *, FILE *, FLOWLINES, FLOWLINES, FLOWLINES );

int fdMemory_gr( I_GR_NODE **, int, NORM **, int );

int cfdMemory( double **, int , int **, int , MATL **, int , XYZF **, int , XYZI **, int ,
	XYZI2 **, int , XY_ZX_YZ **, int , GR_NODE **, int, SUBGRID **, int );

int Flowlines_File( FLOWLINES * );

int flow_anim_files( ANIM_DAT *, int , char *, FLOWLINES , FLOWLINES , FLOWLINES  );

int filecheck( char *, char *, FILE **, FILE **, FILE **, char *, int );

int parse_options(int , char **, ANIM_DAT *, char *, int );

/******** For the Mesh ********/

void MeshKey_Special2(int , int , int );

void cfd_MeshKeys( unsigned char , int , int  );

void MeshReshape(int , int );

void cfd_MeshDisplay(void);

void MeshIdle(void);

void MeshInit(void);

/******** For the Control Panel ********/

void cfd_ControlMouse(int , int , int , int );

void ControlReshape(int , int );

void cfd_ControlDisplay(void);

void cfd_Menu();

void ControlInit(void);

int ControlDimInit( void );

/******** For the Screen Dump ********/

void ScreenShot(int, int);

/* These are functions provided by Phillip Winston for
   movement of the mesh.*/

void agvHandleButton(int , int , int , int );

void agvHandleMotion(int , int );


/******************************* GLOBAL VARIABLES **************************/

/****** CFD globals ******/
int dof, sdof, nmat, nmode, numel, numnp, nx, ny, nz, num_bodies;
double dx, dy, dz;
double dt, err = 0.0;
double time_current, time_current_input, time_current_post;
int nstep, cstep, cstep_input, cstep_post, step_counter, step_counter3;
int anim_counter = 0, save_anim_counter = 0;
int stream_nstep = 700;


XYZI2 *mem_XYZI2;
XYZF *mem_XYZF;
XYZI *mem_XYZI;
XY_ZX_YZ *mem_XY_ZX_YZ;
int *mem_int;
double *mem_double;
NORM *mem_NORM;
double *coord;
double *U;
int *connecter;
BOUND bc;
BODY *body;
MATL *matl;
int *el_matl;
double *velocity, *Voln;
GR_NODE *node0, *node;
SUBGRID *subgrid1, *mem_SUBGRID;
FLOWLINES path0, path, streak0, streak, stream0, stream;
ANIM_DAT anim[1];


/* Global variables for the mesh color and nodal data */

I_GR_NODE *node_color0, *node_color;
int *el_matl_color;
NORM *norm, *norm0;
MATL *matl_crtl;

/* This used to be in MeshInit */

GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat mat_shininess[] = { 50.0 };
GLfloat light_position[] = { 1.0, 1.0, 6.0, 0.0 };

/****** graphics globals ******/

int ControlWindow, MeshWindow;

/* Determines the step size for keyboard and Control panel movement */

double step_sizex = .1, step_sizey = .1, step_sizez = .1;

/****** Translation Variables ******/
double left_right, up_down, in_out, left_right0, up_down0, in_out0;

/****** Rotation Variables ******/
double xAngle = 0.0, yAngle = 0.0, zAngle = 0.0;

/****** Cross Section Plane Translation Variables ******/
double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
	cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;

/* Global variables for drawing the axes */
GLuint AxesList, BodyList, BoundList[10], VelocityList0, VelocityList,   /* Display lists */
	PathlineList0, PathlineList, StreaklineList0, StreaklineList,
	StreamlineList0, StreamlineList;
double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
double AxisLength_x, AxisLength_y, AxisLength_z, AxisLength_max, AxisPoint_step;

/* Global variables for drawing the velocity vectors */
XYZF *velocity_vec, *velocity_vec0;
GLfloat line_width, point_size,
	line_width0, point_size0;

/****** For drawing the Mesh Window ******/
double left, right, top, bottom, near, far, fscale, coord_rescale, line_rescale;
double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
int mesh_width, mesh_height;
int com_mesh_width0 = mesh_width0;
int com_mesh_height0 = mesh_height0;

/* These Variables are for the Control Panle */

int sofi = sizeof(int);
int row_number = rowdim;
int ControlDiv_y[rowdim + 2], ControlDiv_x[rowdim + 2];
int control_width, control_height;
int com_control_width0 = control_width0;
int com_control_height0 =  control_height0;
int current_width, current_height;

double ratio = scaleFactor, ratio2 = 1.0;
double ratio_width = 1.0;
double ratio_height = 1.0;

int boxMove_x, boxMove_y, boxTextMove_x, textMove_x, textMove_y[rowdim+2];
int Color_flag[rowdim];

char RotateData[3][25] = { "    0.00", "    0.00", "    0.00" };
char MoveData[3][25] = { "    0.00", "    0.00", "    0.00" };
char NameData[25] = { " Name"};
char TimeData[25] = { " 1.000e+00"};
char StepData[25] = { " 1"};
char BoxData[2*boxnumber+2][25] = { "", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "", "", "" };
char BoxText[25];

int del_height = 0;
int del_width = 0;

double matl_choicef = 0, node_choicef = 0, ele_choicef = 0, bound_choicef = 0;

int textDiv_xa = textDiv_xa0;
int textDiv_xb = textDiv_xb0;
int boxTextMove_x = boxTextMove_x0;

/* These Variables partition the fluid quantities */

GR_NODE node_div[boxnumber+1], node_div0[boxnumber+1];
double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far, dim_max;
int nxyz_max, nx_inc, ny_inc, nz_inc,
	nxyz_max0, nx_inc0, ny_inc0, nz_inc0, reget_VelocityList_flag;
GR_NODE del_node, max_node, min_node;
double absolute_max_coord;

/* These are the flags */

int input_flag = 1,          /* Tells whether an input file exists or not */
    post_flag = 1,           /* Tells whether a post file exists or not   */
    color_choice = 1,        /* Set to desired color analysis(range 1 - 24) */
    choice = 0,              /* currently unused */
    matl_choice = -1,        /* Set to which material to view  */
    node_choice = -1,        /* Set to which node to view  */
    bound_choice = -1,       /* Set to which boundary to view  */
    ele_choice = -1,         /* Set to which element to view  */
    mode_choice = 0,         /* Set to the desired modal post file */
    app_type_choice = 0;     /* Determines the type of application (e.g. GUI, animation, save animation, etc.) */
int flag_2D = 0,             /* Tells whether we are reading in a 2-D file */
    flag_3D = 1,             /* Tells whether we are reading in a 3-D file */
    mach_read_flag = 1,      /* Tells whether there is mach number data to read */
    vmag_read_flag = 0,      /* Tells whether there is velocity magnitude data to read */
    print_flag = 0,          /* Tells whether to print out data that is being read in */
    codetype_flag = 3,       /* Tells the 3 types of code (compressible = 1, incompressible = 2, graphics = 3) */ 
    gr_optimize_flag = 1;    /* Optimizes the graphics by only drawing one face of a 3-D element. */

int input_color_flag = 0;    /* Used with input_flag to determine how to draw input mesh */
int BackGround_color = 0;    /* Determines the background color of the mesh display window */
int ortho_redraw_flag = 0;   /* calls MeshReshape(currently not used) */
int Solid_flag = 1,          /* Selects between wire frame or solid model */
    Perspective_flag = 1,    /* Selects between orthographic and perspecive views */
    Render_flag = 0,         /* Selects between rendering or analyses */
    AppliedDisp_flag = 0,    /* Turns applied on and off */
    VelocityVec_flag = 0,    /* Turns applied velocity on and off */
    Pathline_flag = 0,       /* Turns pathlines on and off */
    Streakline_flag = 2,     /* Turns streaklines on and off */
    Streamline_flag = 0,     /* Turns streamlines on and off */
    Body_flag = 1,           /* Turns the bodies on and off */
    Material_flag = 0,       /* Turns material on and off */
    Node_flag = 0,           /* Turns Node ID on and off */
    Element_flag = 0,        /* Turns Element ID on and off */
    Boundary_flag = 0,       /* Turns Boundary ID on and off */
    Axes_flag = 0,           /* Turns Axes on and off  */
    Outline_flag = 0,        /* Turns Element Outline on and off  */
    Transparent_flag = 0,    /* Turns Element Transparency on and off  */
    CrossSection_flag = 0;   /* Turns CrossSection_flag on and off  */
int Before_flag = 0,         /* Turns before mesh on and off */
    After_flag = 1,          /* Turns after mesh on and off */
    Both_flag = 0,           /* Turns both before and after on and off */
    Time_flag = 0;           /* Turns Amplification on and off */

double amplify_factor = 1.0; /* Amplifies deformed mesh for better viewing */
double amplify_step = 0.1;   /* Value of amplification icrement */
double amplify_step0 = 0.1;  /* Value of initial calculated amplification icrement */

int nodal_quantities_flag = 0;  /* Tells whether viewing of the nodal quantities
                                   density, pressure, temperature, etc. is on or off */

int compressible_flag = 0,   /* Tells whether we are dealing with a compressible data file. */
    incompressible_flag = 1; /* Tells whether we are dealing with an incompressible data file. */

int main(int argc, char** argv)
{
	int i, j, node_num, check, file_loc, name_length, wait_flag;
	double dns, temp, vx, vy, vz, prs, ene, enthal, mach, vmag, R, gam;
	char *ccheck;
	int dum, dum1, dum2, dum3, numel0, bod_num, pt;
	double fdum, fdum2, fdum3;
	double fpointx, fpointy, fpointz;
	int sofmi, sofmf, sofmGR_NODE, sofmI_GR_NODE, sofmXYZF, sofmXYZI, sofmXYZI2, sofmXY_ZX_YZ,
		sofmNORM, sofmSUBGRID, ptr_inc;
	char name[30], name2[30], ofd_exten[4], buf[ BUFSIZ ], one_char,
		char_dum[20], object_name[30];
	int ofd_exten_length = 4;
	FILE *o1, *o2, *o3, *o_check;

/* The variables are used for off-screen rendering. */ 

	GLXContext ctx;
	Display *dpy;
	Window win;
	XVisualInfo *visinfo;
	Pixmap pix;
	unsigned char *pixels;
	GLXPixmap px;

	int attrib[] = { GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1,
	    GLX_BLUE_SIZE, 1, GLX_DOUBLEBUFFER, None };

	right=0;
	top=0;
	left=1000;
	bottom=1000;
	fscale=0;
	near=1.0;
	far=10.0;
	/*mesh_width=500;
	mesh_height=20;
	control_width=1000;
	control_height=1500;*/

/* Initialize filenames */

	memset(name,0,30*sizeof(char));
	memset(name2,0,30*sizeof(char));
	memset(ofd_exten,0,ofd_exten_length*sizeof(char));

	ccheck = strncpy(ofd_exten,".ofd",ofd_exten_length);
	if(!ccheck) printf( " Problems with strncpy \n");

/*   o1 contains all the structural data for input
     o2 is used to determine the existance of input and post files
     o3 contains all the structural data for postprocessing
     path.file_in contains the pathline data from the post calculation
     path0.file_in contains the pathline data from the input calculation
     streak.file_in contains the streakline data from the post calculation
     streak0.file_in contains the streakline data from the input calculation
     stream.file_in contains the streamline data from the post calculation
     stream0.file_in contains the streamline data from the input calculation
*/

/* The lines below are for looking at the two files which represent the beginning
   and end of a flow.  
*/

	codetype_flag = 3;
	check = parse_options( argc , argv, anim, name2, codetype_flag );

	if( !anim[0].flag && !anim[0].save_flag)  /* Run as GUI only to look at before and after states. */
		app_type_choice = 0;
	if( anim[0].flag && !anim[0].save_flag)   /* Run as animation processor only. */
		app_type_choice = 1;
	if( anim[0].flag && anim[0].save_flag)    /* Run as animation and save frames. */
		app_type_choice = 2;
	if( !anim[0].flag && anim[0].save_flag)   /* Save frames without showing animation. */
		app_type_choice = 3;

	/*app_type_choice = 3;*/

	if( app_type_choice == 0 )
	{
/* Check the command line options passed by the reader. */

	    o2 = fopen( name2,"r" );
	    if(o2 == NULL ) {
		printf("Can't find file %30s\n", name2);
		exit(1);
	    }
	    /*printf( "%3d %30s\n ",name2_length,name2);*/

	    fgets( buf, BUFSIZ, o2 );
	    /*fscanf( o2, "%d %d %d\n ",&numel,&numnp,&nmat);*/
	    fscanf( o2, "%d %d %d", &numnp, &nx, &ny);

/* Check if there is additional coordinate z data and that this is a 3-D file.  */

	    flag_3D = 0;
	    flag_2D = 1;
	    nz = 1;
	    while(( one_char = (unsigned char) fgetc(o2)) != '\n')
	    {
		if(one_char != ' ' )
		{
		    ungetc( one_char, o2);
		    fscanf( o2,"%d", &nz);
		    flag_3D = 1;
		    Perspective_flag = 1;    /* Turns on perspecive for 3-D flows */
		    flag_2D = 0;
		    break;
		}
	    }
	    numnp = nx*ny*nz;

	    numel = (nx-1)*(ny-1);
	    if(flag_3D) numel = (nx-1)*(ny-1)*(nz-1);
	    numel0 = numel; /* numel0 is needed because numel is changed in fdMesher. */

	    nmat = 1;
	    fscanf( o2,"\n");

	    if(flag_3D) printf( "total nodes:%d   nodes in x:%d nodes in y:%d nodes in z:%d\n", numnp, nx, ny, nz);
	    else        printf( "total nodes:%d   nodes in x:%d nodes in y:%d\n", numnp, nx, ny);
	    fgets( buf, BUFSIZ, o2 );
	    printf( "\n");

	    fscanf( o2, "%lf %d %d %lf", &dt, &nstep, &cstep, &time_current);
	    printf( "Time step:     %16.8e\n", dt);
	    printf( "number of steps:  %4d\n", nstep);
	    printf( "current step:     %4d\n", cstep);
	    printf( "current time:  %16.8e\n", time_current);
	    printf( "\n");

	    compressible_flag = 1;
	    incompressible_flag = 0;
	    while(( one_char = (unsigned char) fgetc(o2)) != '\n')
	    {
		if(one_char != ' ' )
		{
		    ungetc( one_char, o2);
		    fscanf( o2,"%lf\n", &err);
		    printf( "SOR tolerence: %16.8e\n", err);
		    incompressible_flag = 1;
		    compressible_flag = 0;
		    mach_read_flag = 0;
		    vmag_read_flag = 1;
		    break;
		}
	    }
	    fgets( buf, BUFSIZ, o2 );
	    printf( "\n");
	    /*printf( "8888888 %4d %4d\n", compressible_flag, incompressible_flag);*/

	    dof=numnp*ndof;
	    sdof=numnp*nsd;
	    nmode = 0;

/* Begin exmaining and checking for the existence of data files */

	    check = filecheck( name, name2, &o1, &o2, &o3, ofd_exten, ofd_exten_length );
	    if(!check) printf( " Problems with filecheck \n");
	}

/* The lines below are for compiling an animation.  */

	if( app_type_choice > 0 )
	{
#if 0
	    numnp = 4590; 
	    nx = 102; 
	    ny = 45;
	    nz = 1;
	    dx = 2.66000000e-01;
	    dy = 2.66000000e-01;
	    dt = 4.00000000e-02;
	    nstep = 100;
	    ccheck = strncpy( name2, "a0", 2);
#endif

	    anim[0].color_choice = 3;
	    anim[0].flowtype_choice = 2;
	    anim[0].velocity_choice = 0;
	    anim[0].node_print_flag = 0;

/* Read in data from "fdview.anim" if it exists. This contains the data for the desired
   quantities to display.
*/
	    check = flow_anim_files( anim, codetype_flag, name2, path, streak, stream);

	    step_counter = cstep;
	    flag_2D = 1;
	    flag_3D = 0;
	    nz = 1;
	    printf( "total nodes:%d   nodes in x:%d nodes in y:%d nodes in z:%d\n", numnp, nx, ny, nz);
	    printf( "nnnn %s %4d\n", name2, incompressible_flag);
	    post_flag = 1;
	    input_flag = 0;

	    compressible_flag = 1;
	    if(incompressible_flag) compressible_flag = 0;

	    /*scanf( "%d %d %d %s", &numnp, &nx, &ny, name2);*/

	    name_length = strlen(name2);
	    ccheck = strncpy( name, name2, name_length);

	    dz = 1.0;
	    numel = (nx-1)*(ny-1);
	    numel0 = numel; /* numel0 is needed because numel is changed in fdMesher. */
	    nmat = 1;
	    dof=numnp*ndof;
	    sdof=numnp*nsd;
	    nmode = 0;
	    color_choice = 3;
	    Perspective_flag = 0;

/* Check to see if the science code has produced the first 9 set of data files by
   looking for the existence of the last one "anim09".
   If not, wait.
*/

	    if( anim[0].color_choice || anim[0].velocity_choice)
	    {
		wait_flag = 1;
		while(wait_flag)
		{
		    wait_flag = 0;
		    o_check = fopen( "anim_node09", "r");
		    if(o_check == NULL)
		    {
			wait_flag = 1;
			sleep(3);
			printf( "waiting anim_node09\n");
		    }
		}
		fclose(o_check);
	    }

	    if( anim[0].flowtype_choice )
	    {
		wait_flag = 1;
		while(wait_flag)
		{
		    wait_flag = 0;
		    o_check = fopen( "anim_flowline09", "r");
		    if(o_check == NULL)
		    {
			wait_flag = 1;
			sleep(3);
			printf( "waiting anim_flowline09\n");
		    }
		}
		fclose(o_check);
	    }


/* Now, open the file, "anim_node", which contains the nodal data needed to do
   preliminary graphics set up.
*/
	    anim[0].node_in = fopen( "anim_node", "r" ); /* Open animation file */
	    if(anim[0].node_in == NULL ) {
		printf("Can't find file anim_node\n");
		exit(1);
	    }

	}

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	if( app_type_choice == 0 )
	{
	    if( input_flag )
	    {
		fgets( buf, BUFSIZ, o1 );
		if(flag_2D) {
			fscanf( o1, "%d %d %d\n ",&dum,&dum1,&dum2);
			printf( "%d %d %d\n",dum,dum1,dum2);
		}
		else {
			fscanf( o1, "%d %d %d %d\n ",&dum,&dum1,&dum2,&dum3);
			printf( "%d %d %d %d\n",dum,dum1,dum2,dum3);
		}
		fgets( buf, BUFSIZ, o1 );
		if(compressible_flag) {
		    fscanf( o1, "%lf %d %d %lf\n ",&fdum,&dum1,&dum2,&fdum2);
		    /*printf( "%lf %d %d %lf\n",fdum,dum1,dum2,fdum2);*/
		}
		if(incompressible_flag) {
		    fscanf( o1, "%lf %d %d %lf %d %lf\n ",&fdum,&dum1,&dum2,&fdum2,&dum3,&fdum3);
		    /*printf( "%lf %d %d %lf %d %lf\n",fdum,dum1,dum2,fdum2,dum3,fdum3);*/
		}
		fgets( buf, BUFSIZ, o1 );
		/*printf( "name %30s\n ",name);*/
	    }

	    /*post_flag = 0;*/
	    if( post_flag )
	    {
		fgets( buf, BUFSIZ, o3 );
		if(flag_2D) {
			fscanf( o3, "%d %d %d\n ",&dum,&dum1,&dum2);
			printf( "%d %d %d\n",dum,dum1,dum2);
		}
		else {
			fscanf( o3, "%d %d %d %d\n ",&dum,&dum1,&dum2,&dum3);
			printf( "%d %d %d %d\n",dum,dum1,dum2,dum3);
		}
		fgets( buf, BUFSIZ, o3 );
		if(compressible_flag) {
		    fscanf( o3, "%lf %d %d %lf\n ",&fdum,&dum1,&dum2,&fdum2);
		    /*printf( "%lf %d %d %lf\n",fdum,dum1,dum2,fdum2);*/
		}
		if(incompressible_flag) {
		    fscanf( o3, "%lf %d %d %lf %d %lf\n ",&fdum,&dum1,&dum2,&fdum2,&dum3,&fdum3);
		    /*printf( "%lf %d %d %lf %d %lf\n",fdum,dum1,dum2,fdum2,dum3,fdum3);*/
		}
		fgets( buf, BUFSIZ, o3 );
		/*printf( "out %30s\n ",out);*/
	    }
	}

                      /* PATHLINES */

/* The code below is for pathlines coming from the post mesh. */

/* Set the name of the post pathline file */

	memset(path.name_in,0,41*sizeof(char));

	ccheck = strncpy(path.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(path.name_in+name_length, ".path.ofd", 11);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the pathline post file as well as its properties */

	check = Flowlines_File( &path );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(path.new_flag) path.nstep_in = 1;

/* The code below is for pathlines coming from the input mesh. */

/* Set the name of the input pathline file */

	memset(path0.name_in,0,41*sizeof(char));

	ccheck = strncpy(path0.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(path0.name_in+name_length, ".path", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the pathline input file as well as its properties */

	check = Flowlines_File( &path0 );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(path0.new_flag) path0.nstep_in = 1;
	if( app_type_choice > 0 ) if(path0.nstep_in < nstep ) path0.nstep_in = nstep;


                  /* STREAKLINES */

/* The code below is for streaklines coming from the post mesh. */

/* Set the name of the post streakline file */

	memset(streak.name_in,0,41*sizeof(char));

	ccheck = strncpy(streak.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(streak.name_in+name_length, ".streak.ofd", 11);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streakline post file as well as its properties */

	check = Flowlines_File( &streak );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(streak.new_flag) streak.nstep_in = 1;

/* The code below is for streaklines coming from the input mesh. */

/* Set the name of the input streakline file */

	memset(streak0.name_in,0,41*sizeof(char));

	ccheck = strncpy(streak0.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(streak0.name_in+name_length, ".streak", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streakline input file as well as its properties */

	check = Flowlines_File( &streak0 );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(streak0.new_flag) streak0.nstep_in = 1;
	if( app_type_choice > 0 ) if(streak0.nstep_in < nstep ) streak0.nstep_in = nstep;


                  /* STREAMLINES */

/* The code below is for streamlines coming from the post mesh. */

/* Set the name of the post streamline file */

	memset(stream.name_in,0,41*sizeof(char));

	ccheck = strncpy(stream.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(stream.name_in+name_length, ".stream.ofd", 11);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streamline post file as well as its properties */

	check = Flowlines_File( &stream );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(stream.new_flag) stream.nstep_in = 1;

/* The code below is for streamlines coming from the input mesh. */

/* Set the name of the input streamline file */

	memset(stream0.name_in,0,41*sizeof(char));

	ccheck = strncpy(stream0.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(stream0.name_in+name_length, ".stream", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streamline input file as well as its properties */

	check = Flowlines_File( &stream0 );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	if(stream0.new_flag) stream0.nstep_in = 1;
	if( app_type_choice > 0 ) if(stream0.nstep_in < nstep ) stream0.nstep_in = stream_nstep;


/*   Begin allocation of meomory */

/* For the doubles */
	sofmf = sdof + 2*dof;

/* For the integers */
	if(flag_2D) sofmi= numel0*npel2 + numnp + 2 + 4*numnp;
	    else sofmi= (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2 + 4*numnp;

/* Note is that the last value, 2*numnp, is based on the fact that some nodes
   may have more than 1 boundary condition, specifically the no-slip corner
   boundary which results in a node being no-slip in more than 1 direction.  So
   I can have, for instance:

      bc[0].noslp.xp[3] = 5;
      bc[0].noslp.yp[3] = 5;

   This potential for having more than one boundary condition is why I have allocated
   2*numnp.
*/

/* For the XYZF doubles */

	dum = 0;
	if( path0.read_flag ) dum += path0.num_pt0 + path0.num_pt0*(path0.nstep_in+1);
	if( path.read_flag ) dum += path.num_pt0 + path.num_pt0*(path.nstep_in+1);
	if( streak0.read_flag ) dum += streak0.num_pt0 + streak0.num_pt0*streak0.nstep_in;
	if( streak.read_flag ) dum += streak.num_pt0 + streak.num_pt0*streak.nstep_in;
	if( stream0.read_flag ) dum += stream0.num_pt0 + stream0.num_pt0*stream0.nstep_in;
	if( stream.read_flag ) dum += stream.num_pt0 + stream.num_pt0*stream.nstep_in;
	sofmXYZF=2*numnp + dum;

/* For the XYZI integers */
	sofmXYZI = 1;

/* For the XYZI2 integers */
	sofmXYZI2=7;

/* For the XY_ZX_YZ integers */
	sofmXY_ZX_YZ=1;

/* For the GR_NODE */
	sofmGR_NODE=numnp;
	if( input_flag && post_flag ) sofmGR_NODE=2*numnp;

/* For the I_GR_NODE */
	sofmI_GR_NODE=numnp;
	if( input_flag && post_flag ) sofmI_GR_NODE=2*numnp;

/* For the NORMS */
	sofmNORM=numel0;
	if(gr_optimize_flag) sofmNORM = 2*nx*ny + 2*nx*nz + 2*ny*nz;

/* For the SUBGRID structs*/
	sofmSUBGRID=1;

	check = cfdMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat, &mem_XYZF, sofmXYZF,
		&mem_XYZI, sofmXYZI, &mem_XYZI2, sofmXYZI2, &mem_XY_ZX_YZ, sofmXY_ZX_YZ, &node0,
		sofmGR_NODE, &mem_SUBGRID, sofmSUBGRID );
	if(!check) printf( " Problems with cfdMemory\n");

	check = fdMemory_gr( &node_color0, sofmI_GR_NODE, &mem_NORM, sofmNORM );
	if(!check) printf( " Problems with fdMemory_gr \n");

/* For the doubles */
	                                ptr_inc=0;
	coord=(mem_double+ptr_inc);     ptr_inc += sdof;
	velocity=(mem_double+ptr_inc);  ptr_inc += dof;
	U=(mem_double+ptr_inc);         ptr_inc += dof;

/* For the materials */

	matl_crtl = matl;

/* For the integers */
	                                      ptr_inc = 0;
	connecter=(mem_int+ptr_inc);
	                          if(flag_2D) ptr_inc += numel0*npel2;
	                                 else ptr_inc += (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2;
	el_matl=(mem_int+ptr_inc);            ptr_inc += numnp;
	bc.num_none=(mem_int+ptr_inc);        ptr_inc += 1;
	bc.num_prs0=(mem_int+ptr_inc);        ptr_inc += 1;

/* Note that bcset below has to be corrected for mem_int so that it has the correct
   offset for mem_int, i.e.:

	if(flag_2D) dum = numel0*npel2 + numnp + 2;
	else dum = (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2;

	check = bcset( &bc, (mem_int + dum), node);
	if(!check) printf( " Problems with bcset\n");
*/

	memset(el_matl,0,numnp*sofi);
	el_matl_color = el_matl;

/* For the XYZI integers */
	                                      ptr_inc = 0;
	bc.num_vel0=(mem_XYZI+ptr_inc);       ptr_inc += 1;

/* For the XYZI2 integers */
	                                       ptr_inc = 0;
	bc.num_symm=(mem_XYZI2+ptr_inc);       ptr_inc += 1;
	bc.num_far=(mem_XYZI2+ptr_inc);        ptr_inc += 1;
	bc.num_free=(mem_XYZI2+ptr_inc);       ptr_inc += 1;
	bc.num_inflow=(mem_XYZI2+ptr_inc);     ptr_inc += 1;
	bc.num_frestr=(mem_XYZI2+ptr_inc);     ptr_inc += 1;
	bc.num_free2=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_noslp=(mem_XYZI2+ptr_inc);      ptr_inc += 1;

/* For the XY_ZX_YZ integers */
	                                          ptr_inc = 0;
	bc.num_noslp_prs=(mem_XY_ZX_YZ+ptr_inc);  ptr_inc += 1;

/* For the GR_NODE doubles */
	                                      ptr_inc = numnp;
	if( input_flag && post_flag ) node =(node0+ptr_inc);

/* For the SUBGRID structs */
                                              ptr_inc = 0;
	subgrid1=(mem_SUBGRID+ptr_inc);       ptr_inc += 1;

/* 
Remember that bc.free, bc.symm, etc. are still waiting for how you would allocate for
them in bcset.c.
*/

/* For the I_GR_NODE integers */
	                                      ptr_inc = numnp;
	if( input_flag && post_flag ) node_color =(node_color0+ptr_inc);

/* For the NORM doubles */
	                                      ptr_inc = 0;
	norm =(mem_NORM+ptr_inc);
#if 0
	if( input_flag && post_flag )         ptr_inc += numel0;
	norm0 =(mem_NORM+ptr_inc);            ptr_inc += numel0;
#endif

/* For the XYZF doubles */
	                                          ptr_inc = 0;
	velocity_vec =(mem_XYZF+ptr_inc);         ptr_inc += numnp;
	velocity_vec0 =(mem_XYZF+ptr_inc);        ptr_inc += numnp;
	if(path0.read_flag)
	{
		path0.pt0=(mem_XYZF+ptr_inc);   ptr_inc += path0.num_pt0;
		path0.pt=(mem_XYZF+ptr_inc);    ptr_inc += path0.num_pt0*(path0.nstep_in+1);
	}
	if(path.read_flag)
	{
		path.pt0=(mem_XYZF+ptr_inc);    ptr_inc += path.num_pt0;
		path.pt=(mem_XYZF+ptr_inc);     ptr_inc += path.num_pt0*(path.nstep_in+1);
	}
	if(streak0.read_flag)
	{
		streak0.pt0=(mem_XYZF+ptr_inc);   ptr_inc += streak0.num_pt0;
		streak0.pt=(mem_XYZF+ptr_inc);    ptr_inc += streak0.num_pt0*streak0.nstep_in;
	}
	if(streak.read_flag)
	{
		streak.pt0=(mem_XYZF+ptr_inc);    ptr_inc += streak.num_pt0;
		streak.pt=(mem_XYZF+ptr_inc);     ptr_inc += streak.num_pt0*streak.nstep_in;
	}
	if(stream0.read_flag)
	{
		stream0.pt0=(mem_XYZF+ptr_inc);   ptr_inc += stream0.num_pt0;
		stream0.pt=(mem_XYZF+ptr_inc);    ptr_inc += stream0.num_pt0*stream0.nstep_in;
	}
	if(stream.read_flag)
	{
		stream.pt0=(mem_XYZF+ptr_inc);    ptr_inc += stream.num_pt0;
		stream.pt=(mem_XYZF+ptr_inc);     ptr_inc += stream.num_pt0*stream.nstep_in;
	}

	if( !post_flag )
	{
	    node = node0;
	    node_color = node_color0;
	    After_flag = 0;
	    Before_flag = 1;
	}

	if( !input_flag )
	{
	    node = node0;
	    node_color = node_color0;
	    After_flag = 1;
	    Before_flag = 0;
	}

	dum = strlen(name);
	memset(char_dum,0,20*sizeof(char));
	if( dum < 9 )
	{
		ccheck = strncpy(char_dum, name, 8);
		sprintf( NameData, "%s", char_dum);
	}
	else
	{
		ccheck = strncpy(char_dum, name, 5);
		sprintf( NameData, "%s...", char_dum);
	}

	if( compressible_flag )
	{
		ccheck = strncpy(ControlText[58], " density    ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[60], " temperature", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[70], " enthalpy   ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[72], " energy     ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[74], " mach number", 12);
		if(!ccheck) printf( " Problems with strncpy \n");
	}
	if( incompressible_flag )
	{
		ccheck = strncpy(ControlText[58], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[60], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[70], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[72], "            ", 12);
		if(!ccheck) printf( " Problems with strncpy \n");

		ccheck = strncpy(ControlText[74], " velocity mag", 13);
		if(!ccheck) printf( " Problems with strncpy \n");
	}

	if( app_type_choice > 0 )
	{
	    fread( node, sizeof(GR_NODE), numnp, anim[0].node_in);
	    fclose( anim[0].node_in );

	    if( anim[0].flowtype_choice == 1)
	    {
		for( i = 0; i < step_counter; ++i)
		{
		    for( j = 0; j < path0.num_pt0; ++j)
		    {
			dum = path0.num_pt0*i + j;
			fscanf( path0.file_in, "%lf %lf", &path0.pt0[dum].x, &path0.pt0[dum].y);
			/*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
				i, j, path0.pt[dum].x, path0.pt[dum].y, path0.pt[dum].z, coord_rescale);*/
		    }
		    fscanf( path0.file_in, "\n");
		}

	    }
	}

	if( app_type_choice == 0 )
	{
	    if( post_flag )
	    {
		if( compressible_flag )
		{
			check = flwreader( matl, name, node, o3, path, streak, stream);
			if(!check) printf( " Problems with flwreader\n");
		}
		if( incompressible_flag )
		{
			check = qikreader( matl, name, node, o3, path, streak, stream);
			if(!check) printf( " Problems with qikreader\n");
		}

/* The 33 lines below are for a case that I don't think will ever come up.  The only case
   is when a file like (filename).path.ofd would still have the format of a starting file.
   I probably put it here because I did a direct copy from the "if( input_flag )" case
   where it will often come up.  But on the off chance that this case will happen for post
   files like (filename).path.ofd files, I am leaving it in.
*/
		if(path.read_flag)
		{
			if(path.new_flag)
			{
			    for( j = 0; j < path.num_pt0; ++j )
			    {
				path.pt[j].x = path.pt0[j].x;
				path.pt[j].y = path.pt0[j].y;
			    }
			}
		}
		if(streak.read_flag)
		{
			if(streak.new_flag)
			{
			    for( j = 0; j < streak.num_pt0; ++j )
			    {
				streak.pt[j].x = streak.pt0[j].x;
				streak.pt[j].y = streak.pt0[j].y;
			    }
			}
		}
		if(stream.read_flag)
		{
			if(stream.new_flag)
			{
			    for( j = 0; j < stream.num_pt0; ++j )
			    {
				stream.pt[j].x = stream.pt0[j].x;
				stream.pt[j].y = stream.pt0[j].y;
			    }
			}
		}

		time_current_post = time_current;
		cstep_post = cstep;

		if(!mach_read_flag && compressible_flag)
		{
		    R = matl[0].R;
		    gam = matl[0].gam;
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			temp   = node[node_num].temp;
			vx     = node[node_num].vx;
			vy     = node[node_num].vy;
			vz     = node[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz)/(R*gam*temp + SMALL);
			node[node_num].mach = sqrt(fdum);
		   }
		}

		if(!vmag_read_flag && incompressible_flag)
		{
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			vx     = node[node_num].vx;
			vy     = node[node_num].vy;
			vz     = node[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz);
			node[node_num].vmag = sqrt(fdum);
		   }
		}
	    }
	    if( input_flag )
	    {
		if( compressible_flag )
		{
			check = flwreader( matl, name, node0, o1, path0, streak0, stream0);
			if(!check) printf( " Problems with flwreader\n");
		}
		if( incompressible_flag )
		{
			check = qikreader( matl, name, node0, o1, path0, streak0, stream0);
			if(!check) printf( " Problems with qikreader\n");
		}

		if(path0.read_flag)
		{
			if(path0.new_flag)
			{
			    for( j = 0; j < path0.num_pt0; ++j )
			    {
				path0.pt[j].x = path0.pt0[j].x;
				path0.pt[j].y = path0.pt0[j].y;
			    }
			}
		}
		if(streak0.read_flag)
		{
			if(streak0.new_flag)
			{
			    for( j = 0; j < streak0.num_pt0; ++j )
			    {
				streak0.pt[j].x = streak0.pt0[j].x;
				streak0.pt[j].y = streak0.pt0[j].y;
			    }
			}
		}
		if(stream0.read_flag)
		{
			if(stream0.new_flag)
			{
			    for( j = 0; j < stream0.num_pt0; ++j )
			    {
				stream0.pt[j].x = stream0.pt0[j].x;
				stream0.pt[j].y = stream0.pt0[j].y;
			    }
			}
		}

		time_current_input = time_current;
		cstep_input = cstep;

		if(!mach_read_flag && compressible_flag)
		{
		    R = matl[0].R;
		    gam = matl[0].gam;
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			temp   = node0[node_num].temp;
			vx     = node0[node_num].vx;
			vy     = node0[node_num].vy;
			vz     = node0[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz)/(R*gam*temp + SMALL);
			node0[node_num].mach = sqrt(fdum);
		   }
		}

		if(!vmag_read_flag && incompressible_flag)
		{
		    for ( node_num = 0; node_num < numnp; node_num++)
		    {
			vx     = node0[node_num].vx;
			vy     = node0[node_num].vy;
			vz     = node0[node_num].vz;

			fdum = (vx*vx + vy*vy + vz*vz);
			node0[node_num].vmag = sqrt(fdum);
		   }
		}
	    }
	}

/* Create the connectivity and nodal coordinates for a standard finite difference uniform
   mesh.  This mesh will not change and applies to all data over all time. */

	check = fdMesher( connecter, coord, node);
	if(!check) printf( " Problems with fdMesher \n");

/* Create the normal vectors for the above mesh. */

	if( flag_3D )
	{
	    if(!gr_optimize_flag)
	    {
		check = ConnectSurfMesher( connecter, el_matl, node);
		if(!check) printf( " Problems with ConnectSurfMesher \n");
	    }
	    else
	    {
		coord_rescale = 10000.0;
		cross_sec_left_right = 10.0;
		cross_sec_up_down = 10.0;
		cross_sec_in_out = 10.0;
		check = ConnectSurfMesher2( connecter, el_matl, node);
		if(!check) printf( " Problems with ConnectSurfMesher \n");
	    }

	    check = fdnormal_vectors(connecter, coord, norm );
	    if(!check) printf( " Problems with normal_vectors \n");
	}

	if(flag_2D) dum = numel0*npel2 + numnp + 2;
	else dum = (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2;

	check = bcset( &bc, (mem_int + dum), node);
	if(!check) printf( " Problems with bcset\n");

/* Allocate memory for the bodies in the flow.  */

	/*printf(  "pppp  %7d\n", num_bodies);*/

	if(num_bodies)
	{
	    body = (BODY *)calloc(num_bodies,sizeof(BODY));
	    if(!body )
	    {
		printf( "failed to allocate memory for body structures\n ");
		exit(1);
	    }
	    dum = 2*bc.num_noslp[0].xm;   /* twice the amount necessary, just in case. */
	    for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	    {
		body[bod_num].nd = (J_I1_I2_INDX *)calloc(dum,sizeof(J_I1_I2_INDX));
		body[bod_num].coord = (double *)calloc(2*dum*nsd,sizeof(double));
	    }
	}

	if(flag_2D) dum = numel0*npel2 + numnp + 2 + 2*numnp;
	else dum = (2*nx*ny + 2*nx*nz + 2*ny*nz)*npel2 + numnp + 2 + 2*numnp;

	check = bcset_noslip( &bc, body, (mem_int+dum), node);
	if(!check) printf( " Problems with bcset_noslip\n");

/* This is only used for generating points which draw the body. */

	if(num_bodies)
	{
	    check = create_body( body);
	    if(!check) printf( " Problems with create_body\n");
	}

/* Search for extreme values */
 
/* In mesh viewer, search for extreme values of nodal points, density,
   pressure, temperature, etc. to obtain viewing parameters and set color
   assignments.  Also initialize variables */

	check = coord_parameter( coord);
	if(!check) printf( " Problems with coord_parameter \n");

	if( app_type_choice > 0 )
	{
	    fdReadparameter_anim();
	    if(anim[0].color_choice)
		if(anim[0].flowtype_choice || anim[0].velocity_choice) Transparent_flag = 1;
		    printf(  "pppp  %7d %4d %4d\n", anim[0].color_choice, anim[0].flowtype_choice, anim[0].velocity_choice);
	}

	if(path.read_flag)
	{
	    for( i = 0; i < path.nstep_in; ++i)
	    {
		for( j = 0; j < path.num_pt0; ++j)
		{
		    dum = path.num_pt0*i + j;
		    path.pt[dum].x /= coord_rescale;
		    path.pt[dum].y /= coord_rescale;
		    path.pt[dum].z /= coord_rescale;
		    /*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, path.pt[dum].x, path.pt[dum].y, path.pt[dum].z, coord_rescale);*/
		}
	    }
	}
	if(path0.read_flag)
	{
	    for( i = 0; i < path0.nstep_in; ++i)
	    {
		for( j = 0; j < path0.num_pt0; ++j)
		{
		    dum = path0.num_pt0*i + j;
		    path0.pt[dum].x /= coord_rescale;
		    path0.pt[dum].y /= coord_rescale;
		    path0.pt[dum].z /= coord_rescale;
		    /*printf(  "vvvv  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, path0.pt[dum].x, path0.pt[dum].y, path0.pt[dum].z, coord_rescale);*/
		}
	    }
	}

	if(streak.read_flag)
	{
	    for( i = 0; i < streak.nstep_in; ++i)
	    {
		for( j = 0; j < streak.num_pt0; ++j)
		{
		    dum = streak.num_pt0*i + j;
		    streak.pt[dum].x /= coord_rescale;
		    streak.pt[dum].y /= coord_rescale;
		    streak.pt[dum].z /= coord_rescale;
		    /*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, streak.pt[dum].x, streak.pt[dum].y, streak.pt[dum].z, coord_rescale);*/
		}
	    }
	}
	if(streak0.read_flag)
	{
	    for( i = 0; i < streak0.nstep_in; ++i)
	    {
		for( j = 0; j < streak0.num_pt0; ++j)
		{
		    dum = streak0.num_pt0*i + j;
		    streak0.pt[dum].x /= coord_rescale;
		    streak0.pt[dum].y /= coord_rescale;
		    streak0.pt[dum].z /= coord_rescale;
		    /*printf(  "vvvv  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, streak0.pt[dum].x, streak0.pt[dum].y, streak0.pt[dum].z, coord_rescale);*/
		}
	    }
	}


	if(stream.read_flag)
	{
	    for( i = 0; i < stream.nstep_in; ++i)
	    {
		for( j = 0; j < stream.num_pt0; ++j)
		{
		    dum = stream.num_pt0*i + j;
		    stream.pt[dum].x /= coord_rescale;
		    stream.pt[dum].y /= coord_rescale;
		    stream.pt[dum].z /= coord_rescale;
		    /*printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, stream.pt[dum].x, stream.pt[dum].y, stream.pt[dum].z, coord_rescale);*/
		}
	    }
	}
	if(stream0.read_flag)
	{
	    for( i = 0; i < stream0.nstep_in; ++i)
	    {
		for( j = 0; j < stream0.num_pt0; ++j)
		{
		    dum = stream0.num_pt0*i + j;
		    stream0.pt[dum].x /= coord_rescale;
		    stream0.pt[dum].y /= coord_rescale;
		    stream0.pt[dum].z /= coord_rescale;
		    /*printf(  "vvvv  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
			i, j, stream0.pt[dum].x, stream0.pt[dum].y, stream0.pt[dum].z, coord_rescale);*/
		}
	    }
	}

	for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	{
	    for( pt = 0; pt < 2*body[bod_num].num_points; ++pt )
	    {
		body[bod_num].coord[2*pt] /= coord_rescale;
		body[bod_num].coord[2*pt+1] /= coord_rescale;
	    }
	}
	
	if( app_type_choice == 0 )
	{
	    if( input_flag )
	    {
		check = fdparameter( node0);
		if(!check) printf( " Problems with fdparameter \n");
	
		check = cfd_set( bc, velocity_vec0, node0, node_color0, node_div0);
		if(!check) printf( " Problems with cfd_set \n");
	    }

	    if( post_flag )
	    {
		check = fdparameter( node);
		if(!check) printf( " Problems with fdparameter \n");
	
		check = cfd_set( bc, velocity_vec, node, node_color, node_div);
		if(!check) printf( " Problems with cfd_set \n");
	    }
	}

/* Initialize the mesh viewer */

	glutInit(&argc, argv);
	if( app_type_choice == 0 )glutInitWindowSize(mesh_width0, mesh_height0);
	if( app_type_choice > 0 )glutInitWindowSize(mesh_width, mesh_height);
	if( app_type_choice != 3)
	{
	    glutInitWindowPosition(400, 215);
	    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	    MeshWindow = glutCreateWindow("SLFCFD");
	    MeshInit();
	}

	AxesList = glGenLists(1);
	agvMakeAxesList(AxesList);

/* create display list for the bodies which are the obstructions in the flow */

	if(num_bodies)
	{
	    BodyList = glGenLists(1);
	    bodies_in_flow(BodyList, body);
	}

	if( app_type_choice == 0 )
	{
/* create display list for boundary conditions */

	    BoundList[0] = glGenLists(1);
	    BoundList[1] = glGenLists(1);
	    BoundList[2] = glGenLists(1);
	    BoundList[3] = glGenLists(1);
	    BoundList[4] = glGenLists(1);
	    BoundList[5] = glGenLists(1);
	    BoundList[6] = glGenLists(1);
	    BoundList[7] = glGenLists(1);
	    BoundList[8] = glGenLists(1);
	    BoundList[9] = glGenLists(1);
	    bound_vectors(BoundList, bc, coord);

/* Note to San:  The code to draw the velocity vectors is analogous to what I did in
   SLFFEA for the force vectors, but the implementation is slightly different in that
   velocity_vec does not represent the global position of one end of the vector, but
   simply its value from the origin.  The reason I do it differently here is that these
   vectors are drawn using display lists as well as the fact that I need the magnitude
   of the vectors in the function "velocity_vectors".  To read more on it, see the code
   in SLFFEA in the *post.c file.
*/

	    if( post_flag )
	    {
/* create display list for velocity grapics vectors on post mesh*/

		VelocityList = glGenLists(1);
		velocity_vectors(VelocityList, bc, coord, velocity_vec);
		if(path.read_flag)
		{
		    PathlineList = glGenLists(1);
		    flowline_points(PathlineList, path);
		}
		if(streak.read_flag)
		{
		    StreaklineList = glGenLists(1);
		    flowline_points(StreaklineList, streak);
		}
		if(stream.read_flag)
		{
		    StreamlineList = glGenLists(1);
		    flowline_points(StreamlineList, stream);
		}
	    }

	    if( input_flag )
	    {

/* create display list for velocity grapics vectors on input mesh*/

		VelocityList0 = glGenLists(1);
		velocity_vectors(VelocityList0, bc, coord, velocity_vec0);
		if(path0.read_flag)
		{
		    PathlineList0 = glGenLists(1);
		    flowline_points(PathlineList0, path0);
		}
		if(streak0.read_flag)
		{
		    StreaklineList0 = glGenLists(1);
		    flowline_points(StreaklineList0, streak0);
		}
		if(stream0.read_flag)
		{
		    StreamlineList0 = glGenLists(1);
		    flowline_points(StreamlineList0, stream0);
		}
	    }
	}

/* Initiate variables in Control Panel */

	memset(Color_flag,0,rowdim*sofi);

	check = ControlDimInit();
	if(!check) printf( " Problems with ControlDimInit \n");

	if( app_type_choice != 3 )
	{
/* call display function  */

	    glutDisplayFunc(cfd_MeshDisplay);

	    glutReshapeFunc(MeshReshape);
	}

	if( app_type_choice > 0 ) glutIdleFunc( MeshIdle );  /* This keeps the animation running. */

/* Initialize Keyboard Functions */

	if( app_type_choice == 0 )
	{
/* Initialize Mouse Functions */

	    glutMouseFunc(agvHandleButton);
	    glutMotionFunc(agvHandleMotion);

	    glutKeyboardFunc(cfd_MeshKeys);
	    glutSpecialFunc(MeshKey_Special2);

/* Initialize the Control Panel */

	    glutInitWindowSize(control_width0, control_height0);
	    glutInitWindowPosition(0, 0);
	    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB );
	    ControlWindow = glutCreateWindow("SLFCFD Control Panel");

	    ControlInit();
	    cfd_Menu();
	    glutDisplayFunc(cfd_ControlDisplay);
	    glutReshapeFunc(ControlReshape);

	    glutMouseFunc(cfd_ControlMouse);
	}

	if( app_type_choice != 3 ) glutMainLoop();

/*
         ------------ OFF-SCREEN BEGIN ------------

   The code below are for off-screen rendering.  I learned a lot from the Mesa demo code:

      /usr/local/Mesa-7.0.3/progs/xdemos/glxheads.c

   and an online code called do_zoom.c which I found at:

      svn.osgeo.org/grass/grass/branches/releasebranch_6_3/visualization/nviz/src/do_zoom.c
*/ 

	if( app_type_choice == 3 )
	{
	    dpy = XOpenDisplay(0);
	    if (!dpy) {
		printf( "\nUnable to open display\n");
		exit(1);
	    }

	    visinfo = glXChooseVisual(dpy, DefaultScreen(dpy), attrib);

	    ctx = glXCreateContext( dpy, visinfo, NULL, True );
	    if (!ctx) {
		printf("Error: glXCreateContext failed\n");
		exit(1);
	    }

	    pix = XCreatePixmap(dpy, RootWindow(dpy, visinfo->screen), mesh_width,
		 mesh_height, visinfo->depth);
	    px = glXCreateGLXPixmap(dpy, visinfo, pix);

	    if (!(glXMakeCurrent(dpy, px, ctx))) {
	    fprintf(stderr, "glXMakeCurrent failed (window)!\n");
	    exit(0);
	    }

	    MeshInit();
	    MeshReshape(mesh_width, mesh_height);
	    for( step_counter3 = cstep; step_counter3 < nstep; ++step_counter3)
	    {
		 cfd_MeshDisplay();
	    }

	    glXDestroyGLXPixmap(dpy, px);
	    XFreePixmap(dpy, pix);
	    glXDestroyContext(dpy, ctx);
	    XCloseDisplay(dpy);

	}

/*       ------------ OFF-SCREEN END ------------ */

/* call function for hotkeys
 */
#if 0
	glutKeyboardFunc(ControlHandleKeys);
#endif

	free(mem_NORM);
	free(matl);
	free(mem_double);
	free(mem_int);
	free(mem_XYZF);
	free(mem_XYZI);
	free(mem_XYZI2);
	free(mem_XY_ZX_YZ);
	free(node0);
	free(mem_SUBGRID);
	return 1;    /* ANSI C requires main to return int. */
}
