/*
    This program does the file management of the nodal data for each frame
    of the animation.  It looks for and reads 10 data files, one at a time,
    that were generated by the science codes, calls "fdparameter_anim" and
    "cfd_set" to set the parameters, and then deletes these files when
    finished.  The science codes will wait until these files are deleted to
    continue. 
  
                  Last Update 7/14/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

extern int numnp, nx, ny;
extern double dt;
extern int nstep, step_counter, anim_counter, save_anim_counter;
extern int flag_2D, flag_3D;
extern double *coord;
extern BOUND bc;
extern GR_NODE *node;
extern FLOWLINES path0, streak0, stream0;
extern int stream_nstep;

extern I_GR_NODE *node_color;
extern GR_NODE node_div[boxnumber+1];
extern XYZF *velocity_vec;

extern int mesh_width, mesh_height;
extern double coord_rescale;
extern ANIM_DAT anim[1];
extern int app_type_choice;

int fdparameter_anim( GR_NODE *);

void ScreenShot_anim( int, int, int );

void MeshIdle(void)
{
	glGetError();
	glutPostRedisplay();
}


void MeshAnimate(void)
{

	int i, j, k, node_num, dum, dum2;
	int wait_flag, counter, check;
	double time;
	FILE *o_check;
	char *ccheck;

	memset(anim[0].node_name,0,30*sizeof(char));
	memset(anim[0].flowline_name,0,30*sizeof(char));

	printf( "The current step is %5d\n", step_counter);
	time = dt*step_counter;
	printf( " Time: %16.8e  step: %5d\n", time, step_counter);
	++step_counter;
	if(step_counter > nstep) exit(1);

	ccheck = strncpy(anim[0].node_name, "anim_node0", 10);
	sprintf((anim[0].node_name+10), "%d", anim_counter);

	ccheck = strncpy(anim[0].flowline_name, "anim_flowline0", 14);
	sprintf((anim[0].flowline_name+14), "%d", anim_counter);

	if( anim[0].color_choice || anim[0].velocity_choice)
	{
	    wait_flag = 1;
	    while(wait_flag)
	    {
		wait_flag = 0;
		anim[0].node_in = fopen( anim[0].node_name, "r");
		if(anim[0].node_in == NULL)
		{
		    wait_flag = 1;
		    sleep(1);
		    printf( " waiting %s\n", anim[0].node_name);
		}
		else
		{
		    fread( node, sizeof(GR_NODE), numnp, anim[0].node_in);
		    fclose(anim[0].node_in);
		    wait_flag = 0;
		}
	    }

	    check = fdparameter_anim( node);
	    if(!check) printf( " Problems with fdparameter \n");

	    check = cfd_set( bc, velocity_vec, node, node_color, node_div);
	    if(!check) printf( " Problems with cfd_set \n");
	}

	printf(" force %12.5f %12.5f\n", node[50].vx, velocity_vec[50].x);

#if 1
	if( anim[0].flowtype_choice )
	{
	    wait_flag = 1;
	    while(wait_flag)
	    {
		wait_flag = 0;
		anim[0].flowline_in = fopen( anim[0].flowline_name, "r");
		if(anim[0].flowline_in == NULL)
		{
		    wait_flag = 1;
		    sleep(1);
		    printf( " waiting %s\n", anim[0].flowline_name);
		}
		else
		{
		    if( anim[0].flowtype_choice == 1)
		    {
			dum2 = path0.num_pt0*(step_counter-1);
			fread( path0.pt + dum2, sizeof(XYZF), path0.num_pt0, anim[0].flowline_in);
			fclose(anim[0].flowline_in);

			/*printf(  "pppp  %7d   %7d   %4d \n", step_counter, anim_counter, dum2);*/

/* The path0line points have to be rescaled just like every other quantity for display. */

			for( j = 0; j < path0.num_pt0; ++j)
			{
			    dum = path0.num_pt0*(step_counter-1) + j;
			    path0.pt[dum].x /= coord_rescale;
			    path0.pt[dum].y /= coord_rescale;
			    path0.pt[dum].z /= coord_rescale;
			}

			for( i = 0; i < step_counter; ++i)
			{
			    for( j = 0; j < path0.num_pt0; ++j)
			    {
				dum = path0.num_pt0*i + j;
				/*if ( step_counter > 399 && i >  counter-4 && j < 10 )
				printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
					i, j, path0.pt[dum].x, path0.pt[dum].y, path0.pt[dum].z, coord_rescale);*/
			    }
			}
			wait_flag = 0;
		    }

		    if( anim[0].flowtype_choice == 2)
		    {
			dum2 = streak0.num_pt0*step_counter;
			fread( streak0.pt, sizeof(XYZF), dum2, anim[0].flowline_in);
			fclose(anim[0].flowline_in);

			/*printf(  "pppp  %7d   %7d   %4d \n", step_counter, anim_counter, dum2);*/

/* The streakline points have to be rescaled just like every other quantity for display. */

			for( i = 0; i < step_counter; ++i)
			{
			    for( j = 0; j < streak0.num_pt0; ++j)
			    {
				dum = streak0.num_pt0*i + j;
				streak0.pt[dum].x /= coord_rescale;
				streak0.pt[dum].y /= coord_rescale;
				streak0.pt[dum].z /= coord_rescale;
				/*if ( step_counter > 150 && i >  counter-4 && j < 10 )
				printf(  "pppp  %7d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum,
					i, j, streak0.pt[dum].x, streak0.pt[dum].y, streak0.pt[dum].z, coord_rescale);*/
			    }
			}
			wait_flag = 0;
		    }

		    if( anim[0].flowtype_choice == 3)
		    {
			dum2 = stream0.num_pt0*stream_nstep;
			fread( stream0.pt, sizeof(XYZF), dum2, anim[0].flowline_in);
			fclose(anim[0].flowline_in);

			/*printf(  "pppp  %7d   %7d   %4d \n", step_counter, anim_counter, dum2);*/

/* The streamline points have to be rescaled just like every other quantity for display. */

			for( i = 0; i < stream_nstep; ++i)
			{
			    for( j = 0; j < stream0.num_pt0; ++j)
			    {
				dum = stream0.num_pt0*i + j;
				stream0.pt[dum].x /= coord_rescale;
				stream0.pt[dum].y /= coord_rescale;
				stream0.pt[dum].z /= coord_rescale;
				/*if ( i > 350 && i >  counter-4 && j < 10 )
				printf(  "pppp  %7d %4d %4d %4d %16.8e %16.8e %16.8e %16.8e\n", dum, stream_nstep,
					i, j, stream0.pt[dum].x, stream0.pt[dum].y, stream0.pt[dum].z, coord_rescale);*/
			    }
			}
			wait_flag = 0;
		    }
		}
	    }
	}
#endif


	if( app_type_choice > 1 )
	{
	    printf("\n step: %5d  anim counter: %5d save anim counter: %5d \n",
		step_counter, anim_counter, anim_counter);
	    ScreenShot_anim( mesh_width, mesh_height, anim_counter );
	}


	++anim_counter;
	if(anim_counter > 9)
	{
	    if( anim[0].node_print_flag )
	    {
		check = system( "rm anim_node00");
		check = system( "rm anim_node01");
		check = system( "rm anim_node02");
		check = system( "rm anim_node03");
		check = system( "rm anim_node04");
		check = system( "rm anim_node05");
		check = system( "rm anim_node06");
		check = system( "rm anim_node07");
		check = system( "rm anim_node08");
		check = system( "rm anim_node09");
	    }

	    if( anim[0].flowtype_choice )
	    {
		check = system( "rm anim_flowline00");
		check = system( "rm anim_flowline01");
		check = system( "rm anim_flowline02");
		check = system( "rm anim_flowline03");
		check = system( "rm anim_flowline04");
		check = system( "rm anim_flowline05");
		check = system( "rm anim_flowline06");
		check = system( "rm anim_flowline07");
		check = system( "rm anim_flowline08");
		check = system( "rm anim_flowline09");
	    }

/* I want to stop producing images after each set of 10 steps until ffanim deletes
   these files.  Once they are deleted, we can continue.

   save_anim_counter is also used to keep the names of the dumped images between pic00.tga
   and pic20.tga.
*/
	    if( app_type_choice > 1 )
	    {
		wait_flag = 1;
		while(wait_flag)
		{
		    wait_flag = 0;
		    o_check = fopen( "pic09.tga", "r");
		    if(o_check != NULL)
		    {
			fclose(o_check);
			wait_flag = 1;
			sleep(3);
		    }
		}
	    }

	    anim_counter = 0;
	}

}

