/* The programs contained in this file are for when animation is called.
   The first subroutine "flow_anim_files", does file handling.  The second,
   "flow_anim_write" is used by the science codes to write out the data.
 

                     Updated 7/1/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"

extern int numnp, nx, ny, nz, num_bodies;
extern double dt, time_current, err, dx, dy, dz;
extern int incompressible_flag;
extern int nstep, cstep;
extern int stream_nstep;

int flow_anim_files( ANIM_DAT *anim, int codetype_flag, char *name,
	FLOWLINES path, FLOWLINES streak, FLOWLINES stream )
{
/*
   This program handles the management of the animation parameter data
   files "fdview.anim" and "anim_name".  The first part applies to both
   the science and graphics codes which read from "fdview.anim" to
   determine what type of quantities will be displayed for the animation.

   For the second part, if this program is called from a science code, then
   the parameters of the mesh will be written into "anim_name".  If called
   from graphics, then the data in "anim_name" will be read in. 

                     Updated 7/1/08
*/
	int i, dum;
	int check;
	char buf[ BUFSIZ ];
	FILE *anim_name, *anim_node_out, *anim_flowline_out, *fddata;


/* Read in data from "fdview.anim" if it exists. This contains the data for the desired 
   quantities to display.
*/

	memset(anim[0].color_name,0,30*sizeof(char));
	memset(anim[0].flowtype_name,0,30*sizeof(char));
	memset(anim[0].velocity_name,0,30*sizeof(char));


	fddata = fopen( "fdview.anim","r" );
	if(fddata == NULL ) {
	    printf("Can't find file fdview.anim\n");
	    anim[0].flowtype_choice = 2;
	    anim[0].color_choice = 9;
	    anim[0].velocity_choice = 0;
	    anim[0].node_print_flag = 1;
	}
	else
	{
	    for( i = 0; i < 18; ++i )
	    {
		fgets( buf, BUFSIZ, fddata );
	    }
	    fscanf( fddata, "%s %s %s\n", anim[0].color_name, anim[0].flowtype_name, anim[0].velocity_name);

/* Determine whether node data such as pressure and velocity are desired. */

	    anim[0].color_choice = 1;
	    check = strncmp(anim[0].color_name,"none", 4);
	    if(!check) anim[0].color_choice = 0;

	    anim[0].velocity_choice = 1;
	    check = strncmp(anim[0].velocity_name,"none", 4);
	    if(!check) anim[0].velocity_choice = 0;

	    anim[0].node_print_flag = anim[0].color_choice + anim[0].velocity_choice;

/* Determine whether flowline data is desired. */

	    anim[0].flowtype_choice = 0;
	    check = strncmp(anim[0].flowtype_name,"path", 4);
	    if(!check) anim[0].flowtype_choice = 1;
	    check = strncmp(anim[0].flowtype_name,"streak", 6);
	    if(!check) anim[0].flowtype_choice = 2;
	    check = strncmp(anim[0].flowtype_name,"stream", 6);
	    if(!check) anim[0].flowtype_choice = 3;

/* If neither nodal data or flowline data are chosen, quit. */

	    dum = anim[0].node_print_flag + anim[0].flowtype_choice;
	    if(!dum)
	    {
		printf( "You must choose quantities to animate in the last line of fdview.dat.\n");
		exit(1);
	    }

	    fclose(fddata);
	}

/* If called from a science code, print out the parameters of the mesh for animation. */

	if(codetype_flag == 2 || codetype_flag == 1)
	{

/* Write out the data that will be needed by "fdpost" to do the animation. */

	    anim[0].name = fopen( "anim_name", "w" ); /* Open animation data file */
	    fprintf( anim[0].name, "input file name\n");
	    fprintf( anim[0].name, "%s\n", name);
	    fprintf( anim[0].name, "numnp  nx  ny  incompressible_flag\n");
	    fprintf( anim[0].name, "%5d %5d %5d %5d\n", numnp, nx, ny, incompressible_flag);
	    fprintf( anim[0].name, "dt  nstep  cstep  time_current  err\n");
	    fprintf( anim[0].name, "%12.6e %5d %5d %12.6e %12.6e\n", dt, nstep, cstep, time_current, err);
	    fprintf( anim[0].name, "spatial discretization dx, dy\n");
	    fprintf( anim[0].name, "%16.8e %16.8e\n", dx, dy);
	    fprintf( anim[0].name, "number of bodies\n");
	    fprintf( anim[0].name, "%4d\n", num_bodies);
	    fprintf( anim[0].name, "number of flowlines\n");
	    if( anim[0].flowtype_choice == 1) fprintf( anim[0].name, "%4d\n", path.num_pt0);
	    if( anim[0].flowtype_choice == 2) fprintf( anim[0].name, "%4d\n", streak.num_pt0);
	    if( anim[0].flowtype_choice == 3) fprintf( anim[0].name, "%4d\n", stream.num_pt0);
	    fprintf( anim[0].name, "\n");
	    fclose(anim[0].name);

	    printf( "\n");
	    printf( "The file \"fdview.anim\" contains the input data used by\n");
	    printf( "the graphics code, \"fdpost\", to do animation.  You should\n");
	    printf( "look in this file and modify it to get a display of the\n");
	    printf( "quantities you want to see animated.\n");
	    printf( "\n");
	}

/* If called from graphics code, read in the parameters of the mesh for animation. */

	if(codetype_flag == 3 )
	{
	    anim[0].name = fopen( "anim_name", "r" ); /* Open animation name file */
	    if(anim[0].name == NULL ) {
		printf("Can't find file anim_name\n");
		exit(1);
	    }
	    fgets( buf, BUFSIZ, anim[0].name );
	    printf( "input file name\n");
	    fscanf( anim[0].name, "%s\n", name);
	    printf( "%s\n", name);
	    fgets( buf, BUFSIZ, anim[0].name );
	    printf( "numnp  nx  ny\n");
	    fscanf( anim[0].name, "%d %d %d %d\n", &numnp, &nx, &ny, &incompressible_flag);
	    printf( "%d %d %d\n", numnp, nx, ny);
	    fgets( buf, BUFSIZ, anim[0].name );
	    fscanf( anim[0].name, "%lf %d %d %lf %lf\n", &dt, &nstep, &cstep, &time_current, &err);

	    printf( "Time step:     %16.8e\n", dt);
	    printf( "number of steps:  %4d\n", nstep);
	    printf( "current step:     %4d\n", cstep);
	    printf( "current time:  %16.8e\n", time_current);
	    printf( "SOR tolerence: %16.8e\n", err);

	    fgets( buf, BUFSIZ, anim[0].name );
	    printf( "spatial discretization dx, dy\n");
	    fscanf( anim[0].name, "%lf %lf\n", &dx, &dy);
	    printf( "%16.8e %16.8e\n", dx, dy);
	    fgets( buf, BUFSIZ, anim[0].name );
	    printf( "number of bodies\n");
	    fscanf( anim[0].name, "%d\n", &num_bodies);
	    printf( "%d\n", num_bodies);
	    fgets( buf, BUFSIZ, anim[0].name );
	    printf( "\n");
	    fclose( anim[0].name );
	}

	return 1;

}


int flow_anim_write( int anim_counter, int counter, ANIM_DAT *anim, FLOWLINES flowline, GR_NODE *gr_node )
{
/*
   This program is called from the science codes to write out nodal data 
   containing quantaties such as density, pressure, velocity, etc. as
   well as flowline data(either pathlines, streaklines, or streamlines).

   This data is read into the graphics code for animation.

                     Updated 7/1/08
*/
	int check, wait_flag, dum;
	FILE *o_check;

	if(anim[0].node_print_flag)
	{
	    sprintf((anim[0].node_name+10), "%1d", anim_counter);
	    anim[0].node_out = fopen( anim[0].node_name, "w" ); /* Open animation file containing node data*/
	    fwrite( gr_node, sizeof(GR_NODE), numnp, anim[0].node_out);
	    fclose( anim[0].node_out );                      /* Close node data animation file */
	}
#if 0
	fwrite( gr_node, sizeof(GR_NODE), numnp, stdout); /* Test writing to stdout rather than a file. */
#endif

	if( anim[0].flowtype_choice )
	{
/* Open animation file containing flowline data if selected.  */

	    sprintf((anim[0].flowline_name+14), "%1d", anim_counter);
	    anim[0].flowline_out = fopen( anim[0].flowline_name, "w" );

	    if( anim[0].flowtype_choice == 1 )
	    {
/* Only the most recent point in the pathline changes so we only have to update the most recently updated points. */

		dum = flowline.num_pt0*(counter-1);
		fwrite( flowline.pt + dum, sizeof(XYZF), flowline.num_pt0, anim[0].flowline_out);
	    }
	    if( anim[0].flowtype_choice == 2 )
	    {
/* All streakline points which have been released are updated so every released point is sent. */

		dum = flowline.num_pt0*counter;
		fwrite( flowline.pt, sizeof(XYZF), dum, anim[0].flowline_out);
	    }
	    if( anim[0].flowtype_choice == 3 )
	    {
/* All streamline points are updated so every point(I set the number of points to 700) is sent. */

		/*check = streamlines( stream_nstep, flowline, node);*/
		dum = flowline.num_pt0*stream_nstep;
		fwrite( flowline.pt, sizeof(XYZF), dum, anim[0].flowline_out);
	    }

	    fclose( anim[0].flowline_out );  /* Close flowline animation file */
	}

	wait_flag = 1;

	if( anim_counter > 8)
	{
/* I want to stop producing images after each set of 10 steps until "fdpost" deletes
   these files indicating that it is finished compiling the set into the animation.
   Once they are deleted, we can continue.

   anim_counter is also used to keep the names of the dumped images between anim_node00
   and anim_node09.
*/
	    if(anim[0].node_print_flag)
	    {
		wait_flag = 1;
		while(wait_flag)
		{
/* Check for the existence of "anim_node00".  If it exists, wait.  If not, start the
   process again.
*/
		    wait_flag = 0;
		    o_check = fopen( "anim_node09", "r");
		    if(o_check != NULL)
		    {
			fclose(o_check);
			wait_flag = 1;
			sleep(3);
		    }
		}
	    }

	    if( anim[0].flowtype_choice )
	    {
/* Wait for animation file containing flowline data if selected.  */

		wait_flag = 1;
		while(wait_flag)
		{
/* Check for the existence of "anim_flowline00".  If it exists, wait.  If not, start the
   process again.
*/
		    wait_flag = 0;
		    o_check = fopen( "anim_flowline09", "r");
		    if(o_check != NULL)
		    {
			fclose(o_check);
			wait_flag = 1;
			sleep(3);
		    }
		}
	    }
	    anim_counter = 0;
	}

	return 1;

}


