/*
    This subroutine processes the information of the streakline or pathline
    data files to determine whether they exist and their properties.  Streaklines
    and pathlines may be either an initial input file or one where many points
    have already been calculated.

    From the code below, "flow" can mean either "streak" or "path".

                  Last Update 1/7/10

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2008-2010  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdstruct.h"

int Flowlines_File( FLOWLINES *flow )
{
	int i, j, check, dum, file_loc, line_count_number;
	char *ccheck, one_char, buf[ BUFSIZ ];
	double fdum;

/*   *flow[0].file_in contains the initial points of the flowlines
     *flow[0].file_out will contain the newly generated points of the flowlines
*/

	flow[0].read_flag = 1;
	flow[0].new_flag = 0;
	flow[0].file_in = fopen( flow[0].name_in, "r" );
	if(flow[0].file_in == NULL ) {
	    printf("Can't find file %s\n", flow[0].name_in);
	    flow[0].read_flag = 0;
	    flow[0].num_pt0 = 0;
	}
	else
	{
		file_loc = ftell(flow[0].file_in);
		line_count_number = line_count( flow[0].file_in );
		if( line_count_number < 2)
		{
		    printf("There is only one data point in file  %s\n", flow[0].name_in);
		    flow[0].nstep_in = 1;
		    flow[0].read_flag = 0;
		    return 1;
		}
		fseek(flow[0].file_in, file_loc, 0);

/* Determine whether the flowline input file (either  *.path, *.streak, or *.stream) is an initial file:

		flow[0].new_flag = 1;

   or whether it is the continuation of a previous calculation:

		flow[0].new_flag = 0;
*/

		file_loc = ftell(flow[0].file_in);
		flow[0].new_flag = 1;
		fgets( buf, BUFSIZ, flow[0].file_in );
		fscanf( flow[0].file_in, "%lf", &fdum);
		dum = 1;
		while(( one_char = (unsigned char) fgetc(flow[0].file_in)) != '\n')
		{
		    if(one_char != ' ' )
		    {
			ungetc( one_char, flow[0].file_in);
			fscanf( flow[0].file_in,"%lf", &fdum);
			++dum;
			flow[0].new_flag = 0;
		    }
		}
		fseek(flow[0].file_in, file_loc, 0);
		if(flow[0].new_flag)
		{
		    fgets( buf, BUFSIZ, flow[0].file_in );
		    fscanf( flow[0].file_in, "%d\n", &flow[0].num_pt0);
		    flow[0].nstep_in = 1;
		}
		else
		{
		    flow[0].num_pt0 = dum/2;
		    printf( "The number of points from file %s %d\n",
			flow[0].name_in, flow[0].num_pt0);
		    file_loc = ftell(flow[0].file_in);
		    flow[0].nstep_in = line_count_number;
		}
	}


	return 1;
}

