/* This program compares two inputs x and y for the qsort call
   in the main function.  This is ultimately used to sort the
   nodes comprising the body by "j" int.

                        Updated 3/28/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/


#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#include "cfdstruct.h"

extern int num_bodies;
extern double dx, dy, dz;

int compare_body ( const void *x,  const void *y)
{
	J_I1_I2_INDX *a = (J_I1_I2_INDX *) x;
	J_I1_I2_INDX *b = (J_I1_I2_INDX *) y;
	if ( a[0].j < b[0].j ) return -1;
	if ( a[0].j == b[0].j ) return 0;
	if ( a[0].j > b[0].j ) return 1;
}


int create_body ( BODY *body)
{
/* Below, I calculate the coordinates of the outline of the obstructions and store it in
   the body structure.  This is only for drawing the object and not for calculation.
*/
 
	int i, j, k, dum, bod_num, pt;
	double fdum, fdum2;

	for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	{

/* The body points are sorted based on the j indicie drawing of the obstructions. */

	    qsort( body[bod_num].nd, body[bod_num].num_points, sizeof(J_I1_I2_INDX), compare_body);

#if 0
	    printf( "Here are the resorted points for the obstruction.\n\n");
	    for( pt = 0; pt < body[bod_num].num_points; ++pt )
	    {
		printf( "%4d %4d %4d %4d %4d %4d %4d\n", pt, bod_num,
		    body[bod_num].nd[pt].j, body[bod_num].nd[pt].i1, body[bod_num].nd[pt].i2,
		    body[bod_num].num_points, body[bod_num].nd[pt].index);
	    }
#endif
	}

/* Based on the nodes which make up a boundary, the code below calculates the coordinates based on
   these nodes.
*/

	for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	{
		for( pt = body[bod_num].num_points-1; pt >= 0; --pt )
		{
		    body[bod_num].coord[2*pt] = dx*((double)body[bod_num].nd[pt].i1) + .5*dx;
		    body[bod_num].coord[2*pt+1] = dy*((double)body[bod_num].nd[pt].j);
		}
		dum = body[bod_num].num_points;
		for( pt = 0; pt < body[bod_num].num_points; ++pt )
		{
		    body[bod_num].coord[2*(pt+dum)] = dx*((double)body[bod_num].nd[pt].i2) + .5*dx;
		    body[bod_num].coord[2*(pt+dum)+1] = dy*((double)body[bod_num].nd[pt].j);
		}
		pt = body[bod_num].num_points-1;
		fdum  = dx*((double)body[bod_num].nd[pt].i1) + .5*dx;
		fdum2 = dx*((double)body[bod_num].nd[pt].j);
	}

	return 1;

}

