/*
   This is the include file "flwconst.h" for the for a compressible
   CFD flow field.  It reads in the data and calculates the various
   quantities for each point in the rectangular CFD mesh.

                     Updated 3/22/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2005, 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define PT166 1.0/6.0

#define nsd           3                /* number of spatial dimensions per node */
#define ndof          3                /* degrees of freedom per node */
#define npel          8                /* nodes per element for 3-D mesh*/
#define npel2         4                /* nodes per element for 2-D mesh*/
#define numbc        10                /* number of boundary condition types */

#define min(x,y) ((x)<(y)? (x) : (y))
#define max(x,y) ((x)>(y)? (x) : (y))
#define sign(x,y)  ((y)>0 ? (fabs(x)) : -(fabs(x)))


/* There are quantities below which you can adjust to
   run different problems.  
  
   time_final  - this is the final time which obviously is a multiple of dt
 
   nx                  - x dimension of mesh
   ny                  - y dimension of mesh
   nz                  - z dimension of mesh
  
   The quantities which you need to change simultaneously are:
  
   dt         - time step size
   gam        - ratio of specific heats
   R          - Ideal Gas Constant
  
 */


/*

There are 8 types of boundary conditions(and a skipped condition):

type                 number value         label           type of analysis

 None                       0             none            both
 symmetric in x -           1             smxm            compressible
 symmetric in x +           2             smxp            compressible
 symmetric in y -           3             smym            compressible
 symmetric in y +           4             smyp            compressible
 symmetric in z -           5             smzm            compressible
 symmetric in z +           6             smzp            compressible
 far field in x -           7             ffxm            compressible
 far field in x +           8             ffxp            compressible
 far field in y -           9             ffym            compressible
 far field in y +          10             ffyp            compressible
 far field in z -          11             ffzm            compressible
 far field in z +          12             ffzp            compressible
 free in x -               13             frxm            compressible
 free in x +               14             frxp            compressible
 free in y -               15             frym            compressible
 free in y +               16             fryp            compressible
 free in z -               17             frzm            compressible
 free in z +               18             frzp            compressible
 inflow in x -             19             ifxm            both
 inflow in x +             20             ifxp            both
 inflow in y -             21             ifym            both
 inflow in y +             22             ifyp            both
 inflow in z -             23             ifzm            both
 inflow in z +             24             ifzp            both
 freestream in x -         25             fsxm            incompressible
 freestream in x +         26             fsxp            incompressible
 freestream in y -         27             fsym            incompressible
 freestream in y +         28             fsyp            incompressible
 freestream in z -         29             fszm            incompressible
 freestream in z +         30             fszp            incompressible
 free 2 in x -             31             fr2xm           incompressible
 free 2 in x +             32             fr2xp           incompressible
 free 2 in y -             33             fr2ym           incompressible
 free 2 in y +             34             fr2yp           incompressible
 free 2 in z -             35             fr2zm           incompressible
 free 2 in z +             36             fr2zp           incompressible
 noslip in x -             37             nsxm            both ( Only for 2-D flows )
 noslip in x +             38             nsxp            both ( Only for 2-D flows )
 noslip in y -             39             nsym            both ( Only for 2-D flows )
 noslip in y +             40             nsyp            both ( Only for 2-D flows )
 noslip in z -             41             nszm            both ( Only for 2-D flows )
 noslip in z +             42             nszp            both ( Only for 2-D flows )
 noslip corner in x - y -  43             nsxmym          both ( Only for 2-D flows )
 noslip corner in x - y +  44             nsxmyp          both ( Only for 2-D flows )
 noslip corner in x + y -  45             nsxpym          both ( Only for 2-D flows )
 noslip corner in x + y +  46             nsxpyp          both ( Only for 2-D flows )
 pressure node set to 0.0  63             prs0            incompressible
 Discarded                100             dis             compressible

Note that for the corners, it is given only for the 2-D case.  To keep this
code easily modifiable for 3-D, I should keep a sufficient number of noslip
corner positions for the 3-D case.  So for a new BC, begin numbering somewhere
far beyond 47.

*/
