/* 

   This utility function allocates the memory for the program which
   uses the finite difference method to calculate a compressible CFD flow field. 

                     Updated 1/30/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2005, 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cfdconst.h"
#include "cfdstruct.h"
#include "../AMR/amr.h"

#if COMPRESSIBLE
#define NODE    COM_NODE
#endif
#if INCOMPRESSIBLE
#define NODE    INC_NODE
#endif
#if GUI
#define NODE    GR_NODE
#endif

int cfdMemory( double **mem_double, int sofmf, int **mem_int, int sofmi, MATL **matl,
	int nmat, XYZF **mem_XYZF, int sofmXYZF, XYZI **mem_XYZI, int sofmXYZI,
	XYZI2 **mem_XYZI2, int sofmXYZI2, XY_ZX_YZ **mem_XY_ZX_YZ, int sofmXY_ZX_YZ, NODE **node,
        int sofmNODE, SUBGRID **mem_SUBGRID, int sofmSUBGRID )
{
/* For the doubles */
	*mem_double=(double *)calloc(sofmf,sizeof(double));

	if(!mem_double )
	{
	    printf( "failed to allocate memory for doubles\n ");
	    exit(1);
	}

/* For the materials */
	*matl=(MATL *)calloc(nmat,sizeof(MATL));
	if(!matl )
	{
		printf( "failed to allocate memory for matl doubles\n ");
		exit(1);
	}

/* For the NODE struct */

	*node=(NODE *)calloc(sofmNODE,sizeof(NODE));
	if(!node )
	{
		printf( "failed to allocate memory for node structs\n ");
		exit(1);
	}

/* For the SUBGRID struct */

	*mem_SUBGRID=(SUBGRID *)calloc(sofmSUBGRID,sizeof(SUBGRID));
	if(!mem_SUBGRID )
	{
		printf( "failed to allocate memory for subgrid structs\n ");
		exit(1);
	}

/* For the integers */
	*mem_int=(int *)calloc(sofmi,sizeof(int));
	if(!mem_int )
	{
		printf( "failed to allocate memory for integers\n ");
		exit(1);
	}

/* For the XYZF doubles */

	*mem_XYZF=(XYZF *)calloc(sofmXYZF,sizeof(XYZF));
	if(!mem_XYZF )
	{
		printf( "failed to allocate memory for XYZF doubles\n ");
		exit(1);
	}

/* For the XYZI doubles */

	*mem_XYZI=(XYZI *)calloc(sofmXYZI,sizeof(XYZI));
	if(!mem_XYZI )
	{
		printf( "failed to allocate memory for XYZI doubles\n ");
		exit(1);
	}

/* For the XYZI2 integers */

	*mem_XYZI2=(XYZI2 *)calloc(sofmXYZI2,sizeof(XYZI2));
	if(!mem_XYZI2 )
	{
		printf( "failed to allocate memory for XYZI2 integers\n ");
		exit(1);
	}

/* For the XY_ZX_YZ integers */

	*mem_XY_ZX_YZ=(XY_ZX_YZ *)calloc(sofmXY_ZX_YZ,sizeof(XY_ZX_YZ));
	if(!mem_XY_ZX_YZ )
	{
		printf( "failed to allocate memory for XY_ZX_YZ integers\n ");
		exit(1);
	}

	return 1;
}


int cfdReGetMemory( double **mem_double, int sofmf, int **mem_int, int sofmi, MATL **matl,
	int nmat, XYZF **mem_XYZF, int sofmXYZF, XYZI **mem_XYZI, int sofmXYZI,
	XYZI2 **mem_XYZI2, int sofmXYZI2, XY_ZX_YZ **mem_XY_ZX_YZ, int sofmXY_ZX_YZ, NODE **node,
        int sofmNODE, SUBGRID **mem_SUBGRID, int sofmSUBGRID )
{
/* For the doubles */
	*mem_double=(double *)realloc(*mem_double, sofmf*sizeof(double));

	if(!mem_double )
	{
	    printf( "failed to allocate memory for doubles\n ");
	    exit(1);
	}
	memset(*mem_double,0,sofmf*sizeof(double));

/* For the materials */
	*matl=(MATL *)realloc(*matl, nmat*sizeof(MATL));
	if(!matl )
	{
		printf( "failed to allocate memory for matl doubles\n ");
		exit(1);
	}
	memset(*matl,0,nmat*sizeof(MATL));

/* For the NODE struct */
	*node=(NODE *)realloc(*node, sofmNODE*sizeof(NODE));

	if(!node )
	{
	    printf( "failed to allocate memory for node\n ");
	    exit(1);
	}
	memset(*node,0,sofmNODE*sizeof(NODE));

/* For the SUBGRID struct */
	*mem_SUBGRID=(SUBGRID *)realloc(*mem_SUBGRID, sofmSUBGRID*sizeof(SUBGRID));

	if(!mem_SUBGRID )
	{
	    printf( "failed to allocate memory for mem_SUBGRID\n ");
	    exit(1);
	}
	memset(*mem_SUBGRID,0,sofmSUBGRID*sizeof(SUBGRID));

/* For the integers */
	*mem_int=(int *)realloc(*mem_int, sofmi*sizeof(int));
	if(!mem_int )
	{
		printf( "failed to allocate memory for integers\n ");
		exit(1);
	}
	memset(*mem_int,0,sofmi*sizeof(int));

/* For the XYZF doubles */

	*mem_XYZF=(XYZF *)realloc(*mem_XYZF, sofmXYZF*sizeof(XYZF));
	if(!mem_XYZF )
	{
		printf( "failed to allocate memory for XYZF doubles\n ");
		exit(1);
	}
	memset(*mem_XYZF,0,sofmXYZF*sizeof(XYZF));

/* For the XYZI integers */

	*mem_XYZI=(XYZI *)realloc(*mem_XYZI, sofmXYZI*sizeof(XYZI));
	if(!mem_XYZI )
	{
		printf( "failed to allocate memory for XYZI doubles\n ");
		exit(1);
	}
	memset(*mem_XYZI,0,sofmXYZI*sizeof(XYZI));

/* For the XYZI2 integers */

	*mem_XYZI2=(XYZI2 *)realloc(*mem_XYZI2, sofmXYZI2*sizeof(XYZI2));
	if(!mem_XYZI2 )
	{
		printf( "failed to allocate memory for XYZI2 integers\n ");
		exit(1);
	}
	memset(*mem_XYZI2,0,sofmXYZI2*sizeof(XYZI2));

/* For the XY_ZX_YZ integers */

	*mem_XY_ZX_YZ=(XY_ZX_YZ *)realloc(*mem_XY_ZX_YZ, sofmXY_ZX_YZ*sizeof(XY_ZX_YZ));
	if(!mem_XY_ZX_YZ )
	{
		printf( "failed to allocate memory for XY_ZX_YZ integers\n ");
		exit(1);
	}
	memset(*mem_XY_ZX_YZ,0,sofmXY_ZX_YZ*sizeof(XY_ZX_YZ));

	return 1;
}

