/* This library function writes to a file the following data for a CFD code:

   1) streakline data
   2) obstructions in the flow
   3) the input file which will display 1) and 2) in "gunplot".

This data is also used by "fdpost" to view the results.


                     Updated 6/25/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2005, 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cfdconst.h"
#include "cfdstruct.h"

#define SMALL   1.0e-20

extern int nstep, num_bodies;
extern int flag_2D, flag_3D, reverse_flag;
extern int sof, nx, ny, nz;
extern int flag_2D, flag_3D;
extern double dx, dy, dz;
extern double dt, time_current, err;
extern int nstep, cstep;

int GnuplotWriter( BODY *body, char *name, FLOWLINES path, FLOWLINES streak, FLOWLINES stream)
{
	int i, i2, j, k, n, n2, dum, bod_num, num_pt0, pt;
	int check, name_length, counter;
	double fdum, fdum2, length, width;
	char object_name[30], gplot_name[41], buf[ BUFSIZ ], *ccheck, one_char;
	FILE *body_file[10], *gplot, *gplot2;
	XYZF min_path_pt, max_path_pt, min_streak_pt, max_streak_pt, min_stream_pt, max_stream_pt;

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

/* Initialize the variables which search for the extreme pathline and streakline
   points.
*/

	min_path_pt.x   = 1000000.0;    max_path_pt.x   = -1000000.0;
	min_path_pt.y   = 1000000.0;    max_path_pt.y   = -1000000.0;
	min_streak_pt.x = 1000000.0;    max_streak_pt.x = -1000000.0;
	min_streak_pt.y = 1000000.0;    max_streak_pt.y = -1000000.0;
	min_stream_pt.x = 1000000.0;    max_stream_pt.x = -1000000.0;
	min_streak_pt.y = 1000000.0;    max_stream_pt.y = -1000000.0;


	if( bod_num )  /* Only write out body data if there are one or more objects. */
	{
/* Draw the obstruction */

	    memset(object_name,0,30*sizeof(char));

	    ccheck = strncpy(object_name, name, name_length);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(object_name+name_length, ".ob", 3);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    for( bod_num = 0; bod_num < num_bodies; ++bod_num )
	    {
		sprintf((object_name+name_length+3), "%d",bod_num);
		body_file[bod_num] = fopen( object_name, "w" );
		for( pt = body[bod_num].num_points-1; pt >= 0; --pt )
		{
		    fprintf( body_file[bod_num], "%4d %9.5f %9.5f\n",
			bod_num, body[bod_num].coord[2*pt], body[bod_num].coord[2*pt+1]);
		}
		dum = body[bod_num].num_points;
		for( pt = 0; pt < body[bod_num].num_points; ++pt )
		{
		    fprintf( body_file[bod_num], "%4d %9.5f %9.5f\n",
			bod_num, body[bod_num].coord[2*(pt+dum)], body[bod_num].coord[2*(pt+dum)+1]);
		}
		pt = body[bod_num].num_points-1;
		fdum  = dx*((double)body[bod_num].nd[pt].i1) + .5*dx;
		fdum2 = dx*((double)body[bod_num].nd[pt].j);
		fprintf( body_file[bod_num], "%4d %9.5f %9.5f\n", bod_num, fdum, fdum2);
	    }
	}

					      /* PATHLINES */

	if(path.read_flag)  /* Only write out pathline data if there was pathline data read in. */
	{
/* Set the name of the pathline output file */

	    memset(path.name_out,0,41*sizeof(char));

	    ccheck = strncpy(path.name_out, name, name_length);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(path.name_out+name_length, ".path.ofd", 9);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* Set the name of the gplot pathline output file */

	    memset(gplot_name,0,41*sizeof(char));

	    ccheck = strncpy(gplot_name, path.name_in, name_length+5);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(gplot_name+name_length+5, ".gplot", 7);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* The reason I'm opening 2 files with the identical content for displaying
   pathlines in gnuplot is because gnuplot allows for the use of arrow
   keys to bring up former commands.  This makes it convenient to have the output
   file always named "gplot.path".
*/
	    gplot = fopen( "gplot.path", "w" );
	    gplot2 = fopen( gplot_name, "w" );
	    path.file_out = fopen( path.name_out, "w" );

/* Plotting of the pathlines */

	    num_pt0 = path.num_pt0;

	    dum = 0;
	    for( i2 = 0; i2 < path.nstep_out; ++i2 )
	    {
		for( i = 0; i < num_pt0; ++i )
		{
		    fdum = path.pt[num_pt0*dum+i].x + path.pt[num_pt0*dum+i].y;
		    if(fdum > -10000)
		    {
			/*printf("%i %i %9.5f %9.5f \n",
			    dum,i,path.pt[num_pt0*dum+i].x, path.pt[num_pt0*dum+i].y);*/
			if(!reverse_flag) fprintf(path.file_out, "%9.5f %9.5f ",
			    path.pt[num_pt0*dum+i].x, path.pt[num_pt0*dum+i].y);
			if(reverse_flag) fprintf(path.file_out, "%9.5f %9.5f ",
			    path.pt[num_pt0*dum+i].y, path.pt[num_pt0*dum+i].x);

			if( max_path_pt.x < path.pt[num_pt0*dum+i].x )
			    max_path_pt.x = path.pt[num_pt0*dum+i].x;
			if( path.pt[num_pt0*dum+i].x > SMALL )
			{
			    if( min_path_pt.x > path.pt[num_pt0*dum+i].x )
				min_path_pt.x = path.pt[num_pt0*dum+i].x;
			}

			if( max_path_pt.y < path.pt[num_pt0*dum+i].y )
			    max_path_pt.y = path.pt[num_pt0*dum+i].y;
			if( path.pt[num_pt0*dum+i].y > SMALL )
			{
			    if( min_path_pt.y > path.pt[num_pt0*dum+i].y )
				min_path_pt.y = path.pt[num_pt0*dum+i].y;
			}
		    }
		    else
		    {
			fprintf(path.file_out, "%9.5f %9.5f ", 0.0, 0.0);
		    }
		}
		fprintf(path.file_out, "\n");
		dum += 1;
	    }

/* This prepares the gnuplot file named "gplot.path". */

	    length = dx*nx;
	    width = dy*ny;

	    max_path_pt.x += dx;
	    min_path_pt.x -= dx;

	    max_path_pt.y += dy;
	    min_path_pt.y -= dy;

	    if( max_path_pt.x > length )
		max_path_pt.x = length;
	    if( min_path_pt.x < 0.0 )
		min_path_pt.x = 0.0;

	    if( max_path_pt.y > width )
		max_path_pt.y = width;
	    if( min_path_pt.y < 0.0 )
		min_path_pt.y = 0.0;

	    fdum = (max_path_pt.x - min_path_pt.x)/(max_path_pt.y - min_path_pt.y + SMALL);
	    if( fdum > 2.0 )
	    {
		fdum = 1.5;
		max_path_pt.y = min_path_pt.y + (max_path_pt.x - min_path_pt.x)/fdum;
	    }
	    if( fdum < 1.0 )
	    {
		fdum = 1.4;
		max_path_pt.x = min_path_pt.x + (max_path_pt.y - min_path_pt.y)*fdum;
	    }

	    fprintf( gplot, "#set term postscript\n");
	    fprintf( gplot, "#set out 'test.ps'\n");
	    fprintf( gplot, "#set term png\n");
	    fprintf( gplot, "#set out 'test.png'\n");
	    fprintf( gplot, "set nokey\n");
	    /*fprintf( gplot, "set yrange [0:%12.6e]\n", width);
	    fprintf( gplot, "set xrange [2.5:%12.6f]\n", length);*/
	    fprintf( gplot, "set yrange [%12.6e:%12.6e]\n", min_path_pt.y, max_path_pt.y);
	    fprintf( gplot, "set xrange [%12.6e:%12.6e]\n", min_path_pt.x, max_path_pt.x);
	    fprintf( gplot, "plot  \"%s\" using  1: 2 with points  0, \\\n", path.name_out);
	    for( i = 1; i < num_pt0 - 1; ++i )
	    {
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			path.name_out, 2*i+1, 2*i+2, i);
	    }
	    if( bod_num )  /* Only write the lines below if there are one or more objects. */
	    {
		i = num_pt0 - 1;
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			path.name_out, 2*i+1, 2*i+2, i);
		for( i = 0; i < num_bodies-1; ++i )
		{
		    fprintf( gplot, "      \"%s.ob%d\" using 2: 3 with lines, \\\n", name, i);
		}
		fprintf( gplot, "      \"%s.ob%d\" using 2: 3 with lines\n", name, i);
	    }
	    else
	    {
		i = num_pt0 - 1;
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d\n",
			path.name_out, 2*i+1, 2*i+2, i);
	    }
	    fprintf( gplot, "#set out\n");

/* This prepares the second gnuplot file, (input file name).path.gplot. */

	    fprintf( gplot2, "#set term postscript\n");
	    fprintf( gplot2, "#set out 'test.ps'\n");
	    fprintf( gplot2, "#set term png\n");
	    fprintf( gplot2, "#set out 'test.png'\n");
	    fprintf( gplot2, "set nokey\n");
	    /*fprintf( gplot2, "set yrange [0:%12.6e]\n", width);
	    fprintf( gplot2, "set xrange [2.5:%12.6f]\n", length);*/
	    fprintf( gplot2, "set yrange [%12.6e:%12.6e]\n", min_path_pt.y, max_path_pt.y);
	    fprintf( gplot2, "set xrange [%12.6e:%12.6e]\n", min_path_pt.x, max_path_pt.x);
	    fprintf( gplot2, "plot  \"%s\" using  1: 2 with points  0, \\\n", path.name_out);
	    for( i = 1; i < num_pt0 - 1; ++i )
	    {
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			path.name_out, 2*i+1, 2*i+2, i);
	    }
	    if( bod_num )  /* Only write the lines below if there are one or more objects. */
	    {
		i = num_pt0 - 1;
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			path.name_out, 2*i+1, 2*i+2, i);
		for( i = 0; i < num_bodies-1; ++i )
		{
		    fprintf( gplot2, "      \"%s.ob%d\" using 2: 3 with lines, \\\n", name, i);
		}
		fprintf( gplot2, "      \"%s.ob%d\" using 2: 3 with lines\n", name, i);
	    }
	    else
	    {
		i = num_pt0 - 1;
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d\n",
			path.name_out, 2*i+1, 2*i+2, i);
	    }
	    fprintf( gplot2, "#set out\n");

	    fclose(gplot);
	    fclose(gplot2);
	}

						  /* STREAKLINES */

	if(streak.read_flag)  /* Only write out streakline data if there was streakline data read in. */
	{
/* Set the name of the streakline output file */

	    memset(streak.name_out,0,41*sizeof(char));

	    ccheck = strncpy(streak.name_out, name, name_length);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(streak.name_out+name_length, ".streak.ofd", 11);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* Set the name of the gplot streakline output file */

	    memset(gplot_name,0,41*sizeof(char));

	    ccheck = strncpy(gplot_name, streak.name_in, name_length+7);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(gplot_name+name_length+7, ".gplot", 7);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* The reason I'm opening 2 files with the identical content for displaying
   streaklines in gnuplot is because gnuplot allows for the use of arrow
   keys to bring up former commands.  This makes it convenient to have the output
   file always named "gplot.streak".
*/
	    gplot = fopen( "gplot.streak", "w" );
	    gplot2 = fopen( gplot_name, "w" );
	    streak.file_out = fopen( streak.name_out, "w" );

/* Plotting of the streaklines */

	    num_pt0 = streak.num_pt0;

/* The 17 lines below are needed for the streaklines but not the pathlines.  They
   are for the case where nstep = cstep and nstep != 0.  Otherwise:

	min_streak_pt.x = 1000000.0;    max_streak_pt.x = -1000000.0;
	min_streak_pt.y = 1000000.0;    max_streak_pt.y = -1000000.0;
*/

	    for( i = 0; i < num_pt0; ++i )
	    {
			if( max_streak_pt.x < streak.pt0[i].x )
			    max_streak_pt.x = streak.pt0[i].x;
			if( streak.pt0[i].x > SMALL )
			{
			    if( min_streak_pt.x > streak.pt0[i].x )
				min_streak_pt.x = streak.pt0[i].x;
			}

			if( max_streak_pt.y < streak.pt0[i].y )
			    max_streak_pt.y = streak.pt0[i].y;
			if( streak.pt0[i].y > SMALL )
			{
			    if( min_streak_pt.y > streak.pt0[i].y )
				min_streak_pt.y = streak.pt0[i].y;
			}
	    }

	    dum = 0;
	    for( i2 = 0; i2 < streak.nstep_out; ++i2 )
	    {
		for( i = 0; i < num_pt0; ++i )
		{
		    fdum = streak.pt[num_pt0*dum+i].x + streak.pt[num_pt0*dum+i].y;
		    if(fdum > -10000)
		    {
			/*fprintf(streak.file_out, "%i %i %9.5f %9.5f \n",
			    dum,i,streak.pt[num_pt0*dum+i].x, streak.pt[num_pt0*dum+i].y);*/
			if(!reverse_flag) fprintf(streak.file_out, "%9.5f %9.5f ",
			    streak.pt[num_pt0*dum+i].x, streak.pt[num_pt0*dum+i].y);
			if(reverse_flag) fprintf(streak.file_out, "%9.5f %9.5f ",
			    streak.pt[num_pt0*dum+i].y, streak.pt[num_pt0*dum+i].x);

			if( max_streak_pt.x < streak.pt[num_pt0*dum+i].x )
			    max_streak_pt.x = streak.pt[num_pt0*dum+i].x;
			if( streak.pt[num_pt0*dum+i].x > SMALL )
			{
			    if( min_streak_pt.x > streak.pt[num_pt0*dum+i].x )
				min_streak_pt.x = streak.pt[num_pt0*dum+i].x;
			}

			if( max_streak_pt.y < streak.pt[num_pt0*dum+i].y )
			    max_streak_pt.y = streak.pt[num_pt0*dum+i].y;
			if( streak.pt[num_pt0*dum+i].y > SMALL )
			{
			    if( min_streak_pt.y > streak.pt[num_pt0*dum+i].y )
				min_streak_pt.y = streak.pt[num_pt0*dum+i].y;
			}
		    }
		    else
		    {
			fprintf(streak.file_out, "%9.5f %9.5f ", 0.0, 0.0);
		    }
		}
		fprintf(streak.file_out, "\n");
		dum += 1;
	    }


/* This prepares the gnuplot file named "gplot.streak". */

	    length = dx*nx;
	    width = dy*ny;

	    max_streak_pt.x += dx;
	    min_streak_pt.x -= dx;

	    max_streak_pt.y += dy;
	    min_streak_pt.y -= dy;

	    if( max_streak_pt.x > length )
		max_streak_pt.x = length;
	    if( min_streak_pt.x < 0.0 )
		min_streak_pt.x = 0.0;

	    if( max_streak_pt.y > width )
		max_streak_pt.y = width;
	    if( min_streak_pt.y < 0.0 )
		min_streak_pt.y = 0.0;

	    fdum = (max_streak_pt.x - min_streak_pt.x)/(max_streak_pt.y - min_streak_pt.y + SMALL);
	    if( fdum > 2.0 )
	    {
		fdum = 1.5;
		max_streak_pt.y = min_streak_pt.y + (max_streak_pt.x - min_streak_pt.x)/fdum;
	    }
	    if( fdum < 1.0 )
	    {
		fdum = 1.4;
		max_streak_pt.x = min_streak_pt.x + (max_streak_pt.y - min_streak_pt.y)*fdum;
	    }

	    fprintf( gplot, "#set term postscript\n");
	    fprintf( gplot, "#set out 'test.ps'\n");
	    fprintf( gplot, "#set term png\n");
	    fprintf( gplot, "#set out 'test.png'\n");
	    fprintf( gplot, "set nokey\n");
	    /*fprintf( gplot, "set yrange [0:%12.6e]\n", width);
	    fprintf( gplot, "set xrange [%12.6f:%12.6f]\n", 2.5, length);*/
	    fprintf( gplot, "set yrange [%12.6e:%12.6e]\n", min_streak_pt.y, max_streak_pt.y);
	    fprintf( gplot, "set xrange [%12.6e:%12.6e]\n", min_streak_pt.x, max_streak_pt.x);
	    fprintf( gplot, "plot  \"%s\" using  1: 2 with points  0, \\\n", streak.name_out);
	    for( i = 1; i < num_pt0 - 1; ++i )
	    {
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			streak.name_out, 2*i+1, 2*i+2, i);
	    }
	    if( bod_num )  /* Only write the lines below if there are one or more objects. */
	    {
		i = num_pt0 - 1;
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			streak.name_out, 2*i+1, 2*i+2, i);
		for( i = 0; i < num_bodies-1; ++i )
		{
		    fprintf( gplot, "      \"%s.ob%d\" using 2: 3 with lines, \\\n", name, i);
		}
		fprintf( gplot, "      \"%s.ob%d\" using 2: 3 with lines\n", name, i);
	    }
	    else
	    {
		i = num_pt0 - 1;
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d\n",
			streak.name_out, 2*i+1, 2*i+2, i);
	    }
	    fprintf( gplot, "#set out\n");


/* This prepares the second gnuplot file, (input file name).streak.gplot. */

	    fprintf( gplot2, "#set term postscript\n");
	    fprintf( gplot2, "#set out 'test.ps'\n");
	    fprintf( gplot2, "#set term png\n");
	    fprintf( gplot2, "#set out 'test.png'\n");
	    fprintf( gplot2, "set nokey\n");
	    /*fprintf( gplot2, "set yrange [0:%12.6e]\n", width);
	    fprintf( gplot2, "set xrange [0:%12.6f]\n", length);*/
	    fprintf( gplot2, "set yrange [%12.6e:%12.6e]\n", min_streak_pt.y, max_streak_pt.y);
	    fprintf( gplot2, "set xrange [%12.6e:%12.6e]\n", min_streak_pt.x, max_streak_pt.x);
	    fprintf( gplot2, "plot  \"%s\" using  1: 2 with points  0, \\\n", streak.name_out);
	    for( i = 1; i < num_pt0 - 1; ++i )
	    {
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			streak.name_out, 2*i+1, 2*i+2, i);
	    }
	    if( bod_num )  /* Only write the lines below if there are one or more objects. */
	    {
		i = num_pt0 - 1;
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			streak.name_out, 2*i+1, 2*i+2, i);
		for( i = 0; i < num_bodies-1; ++i )
		{
		    fprintf( gplot2, "      \"%s.ob%d\" using 2: 3 with lines, \\\n", name, i);
		}
		fprintf( gplot2, "      \"%s.ob%d\" using 2: 3 with lines\n", name, i);
	    }
	    else
	    {
		i = num_pt0 - 1;
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d\n",
			streak.name_out, 2*i+1, 2*i+2, i);
	    }
	    fprintf( gplot2, "#set out\n");

	    fclose(gplot);
	    fclose(gplot2);
	}

						  /* STREAMLINES */

	if(stream.read_flag)  /* Only write out streamline data if there was streamline data read in. */
	{
/* Set the name of the streamline output file */

	    memset(stream.name_out,0,41*sizeof(char));

	    ccheck = strncpy(stream.name_out, name, name_length);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(stream.name_out+name_length, ".stream.ofd", 11);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* Set the name of the gplot streamline output file */

	    memset(gplot_name,0,41*sizeof(char));

	    ccheck = strncpy(gplot_name, stream.name_in, name_length+7);
	    if(!ccheck) printf( " Problems with strncpy \n");

	    ccheck = strncpy(gplot_name+name_length+7, ".gplot", 7);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* The reason I'm opening 2 files with the identical content for displaying
   streamlines in gnuplot is because gnuplot allows for the use of arrow
   keys to bring up former commands.  This makes it convenient to have the output
   file always named "gplot.stream".
*/
	    gplot = fopen( "gplot.stream", "w" );
	    gplot2 = fopen( gplot_name, "w" );
	    stream.file_out = fopen( stream.name_out, "w" );

/* Plotting of the streamlines */

	    num_pt0 = stream.num_pt0;

/* The 17 lines below are needed for the streamlines but not the pathlines.  They
   are for the case where nstep = cstep and nstep != 0.  Otherwise:

	min_stream_pt.x = 1000000.0;    max_stream_pt.x = -1000000.0;
	min_stream_pt.y = 1000000.0;    max_stream_pt.y = -1000000.0;
*/

	    for( i = 0; i < num_pt0; ++i )
	    {
			if( max_stream_pt.x < stream.pt0[i].x )
			    max_stream_pt.x = stream.pt0[i].x;
			if( stream.pt0[i].x > SMALL )
			{
			    if( min_stream_pt.x > stream.pt0[i].x )
				min_stream_pt.x = stream.pt0[i].x;
			}

			if( max_stream_pt.y < stream.pt0[i].y )
			    max_stream_pt.y = stream.pt0[i].y;
			if( stream.pt0[i].y > SMALL )
			{
			    if( min_stream_pt.y > stream.pt0[i].y )
				min_stream_pt.y = stream.pt0[i].y;
			}
	    }

	    dum = 0;
	    for( i2 = 0; i2 < stream.nstep_out; ++i2 )
	    {
		for( i = 0; i < num_pt0; ++i )
		{
		    fdum = stream.pt[num_pt0*dum+i].x + stream.pt[num_pt0*dum+i].y;
		    if(fdum > -10000)
		    {
			/*fprintf(stream.file_out, "%i %i %9.5f %9.5f \n",
			    dum,i,stream.pt[num_pt0*dum+i].x, stream.pt[num_pt0*dum+i].y);*/
			if(!reverse_flag) fprintf(stream.file_out, "%9.5f %9.5f ",
			    stream.pt[num_pt0*dum+i].x, stream.pt[num_pt0*dum+i].y);
			if(reverse_flag) fprintf(stream.file_out, "%9.5f %9.5f ",
			    stream.pt[num_pt0*dum+i].y, stream.pt[num_pt0*dum+i].x);

			if( max_stream_pt.x < stream.pt[num_pt0*dum+i].x )
			    max_stream_pt.x = stream.pt[num_pt0*dum+i].x;
			if( stream.pt[num_pt0*dum+i].x > SMALL )
			{
			    if( min_stream_pt.x > stream.pt[num_pt0*dum+i].x )
				min_stream_pt.x = stream.pt[num_pt0*dum+i].x;
			}

			if( max_stream_pt.y < stream.pt[num_pt0*dum+i].y )
			    max_stream_pt.y = stream.pt[num_pt0*dum+i].y;
			if( stream.pt[num_pt0*dum+i].y > SMALL )
			{
			    if( min_stream_pt.y > stream.pt[num_pt0*dum+i].y )
				min_stream_pt.y = stream.pt[num_pt0*dum+i].y;
			}
		    }
		    else
		    {
			fprintf(stream.file_out, "%9.5f %9.5f ", 0.0, 0.0);
		    }
		}
		fprintf(stream.file_out, "\n");
		dum += 1;
	    }


/* This prepares the gnuplot file named "gplot.stream". */

	    length = dx*nx;
	    width = dy*ny;

	    max_stream_pt.x += dx;
	    min_stream_pt.x -= dx;

	    max_stream_pt.y += dy;
	    min_stream_pt.y -= dy;

	    if( max_stream_pt.x > length )
		max_stream_pt.x = length;
	    if( min_stream_pt.x < 0.0 )
		min_stream_pt.x = 0.0;

	    if( max_stream_pt.y > width )
		max_stream_pt.y = width;
	    if( min_stream_pt.y < 0.0 )
		min_stream_pt.y = 0.0;

	    fdum = (max_stream_pt.x - min_stream_pt.x)/(max_stream_pt.y - min_stream_pt.y + SMALL);
	    if( fdum > 2.0 )
	    {
		fdum = 1.5;
		max_stream_pt.y = min_stream_pt.y + (max_stream_pt.x - min_stream_pt.x)/fdum;
	    }
	    if( fdum < 1.0 )
	    {
		fdum = 1.4;
		max_stream_pt.x = min_stream_pt.x + (max_stream_pt.y - min_stream_pt.y)*fdum;
	    }

	    fprintf( gplot, "#set term postscript\n");
	    fprintf( gplot, "#set out 'test.ps'\n");
	    fprintf( gplot, "#set term png\n");
	    fprintf( gplot, "#set out 'test.png'\n");
	    fprintf( gplot, "set nokey\n");
	    /*fprintf( gplot, "set yrange [0:%12.6e]\n", width);
	    fprintf( gplot, "set xrange [%12.6f:%12.6f]\n", 2.5, length);*/
	    fprintf( gplot, "set yrange [%12.6e:%12.6e]\n", min_stream_pt.y, max_stream_pt.y);
	    fprintf( gplot, "set xrange [%12.6e:%12.6e]\n", min_stream_pt.x, max_stream_pt.x);
	    fprintf( gplot, "plot  \"%s\" using  1: 2 with points  0, \\\n", stream.name_out);
	    for( i = 1; i < num_pt0 - 1; ++i )
	    {
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			stream.name_out, 2*i+1, 2*i+2, i);
	    }
	    if( bod_num )  /* Only write the lines below if there are one or more objects. */
	    {
		i = num_pt0 - 1;
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			stream.name_out, 2*i+1, 2*i+2, i);
		for( i = 0; i < num_bodies-1; ++i )
		{
		    fprintf( gplot, "      \"%s.ob%d\" using 2: 3 with lines, \\\n", name, i);
		}
		fprintf( gplot, "      \"%s.ob%d\" using 2: 3 with lines\n", name, i);
	    }
	    else
	    {
		i = num_pt0 - 1;
		fprintf( gplot, "      \"%s\" using %2d:%2d with points %2d\n",
			stream.name_out, 2*i+1, 2*i+2, i);
	    }
	    fprintf( gplot, "#set out\n");


/* This prepares the second gnuplot file, (input file name).stream.gplot. */

	    fprintf( gplot2, "#set term postscript\n");
	    fprintf( gplot2, "#set out 'test.ps'\n");
	    fprintf( gplot2, "#set term png\n");
	    fprintf( gplot2, "#set out 'test.png'\n");
	    fprintf( gplot2, "set nokey\n");
	    /*fprintf( gplot2, "set yrange [0:%12.6e]\n", width);
	    fprintf( gplot2, "set xrange [0:%12.6f]\n", length);*/
	    fprintf( gplot2, "set yrange [%12.6e:%12.6e]\n", min_stream_pt.y, max_stream_pt.y);
	    fprintf( gplot2, "set xrange [%12.6e:%12.6e]\n", min_stream_pt.x, max_stream_pt.x);
	    fprintf( gplot2, "plot  \"%s\" using  1: 2 with points  0, \\\n", stream.name_out);
	    for( i = 1; i < num_pt0 - 1; ++i )
	    {
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			stream.name_out, 2*i+1, 2*i+2, i);
	    }
	    if( bod_num )  /* Only write the lines below if there are one or more objects. */
	    {
		i = num_pt0 - 1;
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d, \\\n",
			stream.name_out, 2*i+1, 2*i+2, i);
		for( i = 0; i < num_bodies-1; ++i )
		{
		    fprintf( gplot2, "      \"%s.ob%d\" using 2: 3 with lines, \\\n", name, i);
		}
		fprintf( gplot2, "      \"%s.ob%d\" using 2: 3 with lines\n", name, i);
	    }
	    else
	    {
		i = num_pt0 - 1;
		fprintf( gplot2, "      \"%s\" using %2d:%2d with points %2d\n",
			stream.name_out, 2*i+1, 2*i+2, i);
	    }
	    fprintf( gplot2, "#set out\n");

	    fclose(gplot);
	    fclose(gplot2);
	}

	return 1;
}


