/*
    This program generates a shock bubble mesh for a CFD code.

		Updated 1/8/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int meshBCset(int *bc_type, char *text)
{

	char *ccheck;

	switch (bc_type[0]) {
		case 0:
			ccheck = strncpy(text, "none    ", 8); /* None */
		break;
		case 1:
			ccheck = strncpy(text, "smxm    ", 8); /* symmetric in x - */
		break;
		case 2:
			ccheck = strncpy(text, "smxp    ", 8); /* symmetric in x + */
		break;
		case 3:
			ccheck = strncpy(text, "smym    ", 8); /* symmetric in y - */
		break;
		case 4:
			ccheck = strncpy(text, "smyp    ", 8); /* symmetric in y + */
		break;
		case 5:
			ccheck = strncpy(text, "smzm    ", 8); /* symmetric in z - */
		break;
		case 6:
			ccheck = strncpy(text, "smzp    ", 8); /* symmetric in z + */
		break;
		case 7:
			ccheck = strncpy(text, "ffxm    ", 8); /* far field in x - */
		break;
		case 8:
			ccheck = strncpy(text, "ffxp    ", 8); /* far field in x + */
		break;
		case 9:
			ccheck = strncpy(text, "ffym    ", 8); /* far field in y - */
		break;
		case 10:
			ccheck = strncpy(text, "ffyp    ", 8); /* far field in y + */
		break;
		case 11:
			ccheck = strncpy(text, "ffzm    ", 8); /* far field in z - */
		break;
		case 12:
			ccheck = strncpy(text, "ffzp    ", 8); /* far field in z + */
		break;
		case 13:
			ccheck = strncpy(text, "frxm    ", 8); /* free in x - */
		break;
		case 14:
			ccheck = strncpy(text, "frxp    ", 8); /* free in x + */
		break;
		case 15:
			ccheck = strncpy(text, "frym    ", 8); /* free in y - */
		break;
		case 16:
			ccheck = strncpy(text, "fryp    ", 8); /* free in y + */
		break;
		case 17:
			ccheck = strncpy(text, "frzm    ", 8); /* free in z - */
		break;
		case 18:
			ccheck = strncpy(text, "frzp    ", 8); /* free in z + */
		break;
		case 19:
			ccheck = strncpy(text, "ifxm    ", 8); /* inflow in x - */
		break;
		case 20:
			ccheck = strncpy(text, "ifxp    ", 8); /* inflow in x + */
		break;
		case 21:
			ccheck = strncpy(text, "ifym    ", 8); /* inflow in y - */
		break;
		case 22:
			ccheck = strncpy(text, "ifyp    ", 8); /* inflow in y + */
		break;
		case 23:
			ccheck = strncpy(text, "ifzm    ", 8); /* inflow in z - */
		break;
		case 24:
			ccheck = strncpy(text, "ifzp    ", 8); /* inflow in z + */
		break;
		case 25:
			ccheck = strncpy(text, "fsxm    ", 8); /* freestream in x - */
		break;
		case 26:
			ccheck = strncpy(text, "fsxp    ", 8); /* freestream in x + */
		break;
		case 27:
			ccheck = strncpy(text, "fsym    ", 8); /* freestream in y - */
		break;
		case 28:
			ccheck = strncpy(text, "fsyp    ", 8); /* freestream in y + */
		break;
		case 29:
			ccheck = strncpy(text, "fszm    ", 8); /* freestream in z - */
		break;
		case 30:
			ccheck = strncpy(text, "fszp    ", 8); /* freestream in z + */
		break;
		case 31:
			ccheck = strncpy(text, "fr2xm   ", 8); /* free 2 in x - */
		break;
		case 32:
			ccheck = strncpy(text, "fr2xp   ", 8); /* free 2 in x + */
		break;
		case 33:
			ccheck = strncpy(text, "fr2ym   ", 8); /* free 2 in y - */
		break;
		case 34:
			ccheck = strncpy(text, "fr2yp   ", 8); /* free 2 in y + */
		break;
		case 35:
			ccheck = strncpy(text, "fr2zm   ", 8); /* free 2 in z - */
		break;
		case 36:
			ccheck = strncpy(text, "fr2zp   ", 8); /* free 2 in z + */
		break;
		case 37:
			ccheck = strncpy(text, "nsxm    ", 8); /* noslip in x - */
		break;
		case 38:
			ccheck = strncpy(text, "nsxp    ", 8); /* noslip in x + */
		break;
		case 39:
			ccheck = strncpy(text, "nsym    ", 8); /* noslip in y - */
		break;
		case 40:
			ccheck = strncpy(text, "nsyp    ", 8); /* noslip in y + */
		break;
		case 41:
			ccheck = strncpy(text, "nszm    ", 8); /* noslip in z - */
		break;
		case 42:
			ccheck = strncpy(text, "nszp    ", 8); /* noslip in z + */
		break;

/* For the corner nodes */

		case 43:
			ccheck = strncpy(text, "nsxmym  ", 8); /* corner noslip in x - and y - */
		break;
		case 44:
			ccheck = strncpy(text, "nsxmyp  ", 8); /* corner noslip in x - and y + */
		break;
		case 45:
			ccheck = strncpy(text, "nsxpym  ", 8); /* corner noslip in x + and y - */
		break;
		case 46:
			ccheck = strncpy(text, "nsxpyp  ", 8); /* corner noslip in x + and y + */
		break;

		case 63:
			ccheck = strncpy(text, "prs0    ", 8); /* pressure node set to 0.0 */
		break;

		case 100:
			ccheck = strncpy(text, "dis     ", 8); /* discarded */
		break;
	}

        return 1;
}

