/*
    This program generates a free jet mesh for a CFD code.

		Updated 3/15/06

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define pi              3.141592654

int meshBCset(int *, char *);

int main(int argc, char** argv)
{
	int dof, nmat, numel, numnp, nmode, nstep, flag_2D, check;
	int i, j, k, dum, dum2, dum3, dum4;
	double fdum, fdum1, fdum2, fdum3, fdum4, ux[9], uy[9];
	double dnsA, dnsB, vlA, vlB, tempA, tempB;
	double dns, temp, prs, vx, vy, vz, enthal, ene;
	double gam, R, dt, dx, dy, dz;
	int nx, ny, nz; 
	char name[30], *ccheck;
	char buf[ BUFSIZ ];
	FILE *o1;
	char text[6];
	int *bc_type, node_num;

	gam = 1.4;
	R = 1.0;
	dnsA = 1.0; dnsB = 0.2008;
	vlA = 1.0; vlB = 0.0;
	tempA = 1.0/gam; tempB = 1.2/gam;

	o1 = fopen( "dat","w" );
/*
	printf( "\n What is the value of the discretization?\n ");
	scanf( "%lf",&dx);
	printf( "\n How many elements on the length side?\n ");
	scanf( "%d",&nx);
	printf( "\n How many elements on the height?\n ");
	scanf( "%d",&ny);
	printf( "\n How many elements on the width?\n ");
	scanf( "%d",&nz);
*/
	printf( "\n Is the mesh you want to generate 3-D or 2-D?\n ");
	scanf( "%d",&dum);
	flag_2D = 0;
	if(dum < 3) flag_2D = 1;
	if(flag_2D) nz = 1;

	nx = 106; ny = 32; nz = 32;
	dt = 0.0094071;
	nstep = 1;
	dx = 0.1; dy = 0.1; dz = 0.1;

	if(!flag_2D)
	{
		numel = nx*ny*nz;
		numnp = nx*ny*nz;
		fprintf( o1, "total nodes, nodes in x, nodes in y, nodes in z for a free jet\n");
		fprintf( o1, "%4d    %4d %4d %4d\n", numnp, nx, ny, nz);
		fprintf( o1, "Time step, number of steps, current step, current time\n");
		fprintf( o1, "%16.8e %7d %7d %16.8e\n", dt, nstep, 0, 0.0);
		fprintf( o1, "spatial discretization dx, dy, dz\n");
		fprintf( o1, "%16.8e %16.8e %16.8e\n", dx, dy, dz);
		fprintf( o1, "Ideal Gas Constant, Ratio of specific heats\n");
		fprintf( o1, "%16.8e %16.8e\n", R, gam);
	}
	else
	{
		numel = nx*ny;
		numnp = nx*ny;
		fprintf( o1, "total nodes, nodes in x, nodes in y\n");
		fprintf( o1, "%4d    %4d %4d\n", numnp, nx, ny);
		fprintf( o1, "Time step, number of steps, current step, current time\n");
		fprintf( o1, "%16.8e %7d %7d %16.8e\n", dt, nstep, 0, 0.0);
		fprintf( o1, "spatial discretization dx, dy\n");
		fprintf( o1, "%16.8e %16.8e\n", dx, dy);
		fprintf( o1, "Ideal Gas Constant, Ratio of specific heats\n");
		fprintf( o1, "%16.8e %16.8e\n", R, gam);

	}

	bc_type = (int *)calloc(numnp, sizeof(int));

	memset(text, 0, 6*sizeof(char));

	if(!flag_2D) {
		fprintf( o1, "   i    j    k  bc         density      temperature  pressure     velocity x");
		fprintf( o1, "   velocity y   velocity z   enthalpy     energy\n");
	}
	else {
		fprintf( o1, "   i    j    bc       density      temperature  pressure     velocity x");
		fprintf( o1, "   velocity y   enthalpy     energy\n");
	}
		

/* For the 3-D mesh */

	if(!flag_2D)
	{
	    if(nz == ny)
	    {
		for ( i=0; i < nx; i++)
		{
		    for ( j=0; j < ny; j++)
		    {
			for ( k=0; k < nz; k++)
			{
			    node_num = nz*ny*i + nz*j + k;

/*     Note that in assigning the boundary conditions below, I discard every node
       where there is overlap of boundary conditions.  This is why the if statements
       are of the form:

			    if( i == 0) {
			       bc_type[node_num] += n;    
			       if( bc_type[node_num] > n ) bc_type[node_num] = 100;
			    }

       where bc_type[node_num] = 100 is means a discarded node.  For an internal node with
       no boundary conditions, bc_type[node_num] = 0.  If bc_type[node_num] != 0, then
       bc_type[node_num] += n is greater than n and this means there is overlap.

*/
			    if( i == 0) {
			       bc_type[node_num] += 19;       /* inflow in x -  */
			       if( bc_type[node_num] > 19 ) bc_type[node_num] = 100;
			    }
			    if( i == nx - 1) {
			       bc_type[node_num] += 14;       /* Free in x + */
			       if( bc_type[node_num] > 14 ) bc_type[node_num] = 100;
			    }
			    if( j == 0 ) {
			       bc_type[node_num] += 3;	/* symmetric in y - */
			       if( bc_type[node_num] > 3 ) bc_type[node_num] = 100;
			    }
			    if( j == ny - 1) {
			       bc_type[node_num] += 10;       /* far field in y + */
			       if( bc_type[node_num] > 10 ) bc_type[node_num] = 100;
			    }
			    if( k == 0 ) {
			       bc_type[node_num] += 5;	/* symmetric in z - */
			       if( bc_type[node_num] > 5 ) bc_type[node_num] = 100;
			    }
			    if( k == nz - 1) {
			       bc_type[node_num] += 12;       /* far field in z + */
			       if( bc_type[node_num] > 12 ) bc_type[node_num] = 100;
			    }

			    check = meshBCset( (bc_type + node_num), text);
			    if(!check) printf( " Problems with meshBCset\n");
#if     0
/*     All the points below are when the nodes above overlap */

			    if( i == 0 && j == 0 )	    bc_type[node_num] = 100;       /* discarded */
			    if( i == 0 && k == 0 )	    bc_type[node_num] = 100;       /* discarded */
			    if( i == 0 && j == ny - 1 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == 0 && k == nz - 1 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && j == 0 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && k == 0 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && j == ny - 1 )  bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && k == nz - 1 )  bc_type[node_num] = 100;       /* discarded */
			    if( j == 0 && k == 0 )	    bc_type[node_num] = 100;       /* discarded */
			    if( j == 0 && k == nz - 1 )       bc_type[node_num] = 100;       /* discarded */
			    if( j == ny - 1 && k == 0 )	  bc_type[node_num] = 100;       /* discarded */
			    if( j == ny - 1 && k == nz - 1 )  bc_type[node_num] = 100;       /* discarded */
#endif

			    dns = dnsB;
			    temp = tempB;
			    prs = R*dns*temp;
			    vx = vlB; vy = vlB; vz = vlB;
			    enthal = R*gam*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);
			    ene = R*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);

			    fprintf( o1, "%4d %4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
				    i, j, k, text, dns, temp, prs, vx, vy, vz, enthal, ene);

			}
		    }
		}

		for  ( j=0; j < 7; j++) {
		    for ( k=0; k < 7; k++) {
			dum = k*k + j*j;
			fdum = sqrt((double)dum);
			if( fdum < 7.000)
			{
			    for ( i=0; i < nx; i++) {
				int node_num = nz*ny*i + nz*j + k;

				check = meshBCset( (bc_type + node_num), text);
				if(!check) printf( " Problems with meshBCset\n");

				dns = dnsA;
				temp = tempA;
				prs = R*dns*temp;
				vx = vlA; vy = vlB; vz = vlB;
				enthal = R*gam*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);
				ene = R*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);

				fprintf( o1, "%4d %4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
				    i, j, k, text, dns, temp, prs, vx, vy, vz, enthal, ene);
			    }
			}
		    }
		}
	    }
	    else
	    {
		for ( i=0; i < nx; i++)
		{
		    for ( j=0; j < ny; j++)
		    {
			for ( k=0; k < nz; k++)
			{
			    node_num = nz*ny*i + nz*j + k;

/*     Note that in assigning the boundary conditions below, I discard every node
       where there is overlap of boundary conditions.  This is why the if statements
       are of the form:

			    if( i == 0) {
			       bc_type[node_num] += n;    
			       if( bc_type[node_num] > n ) bc_type[node_num] = 100;
			    }

       where bc_type[node_num] = 100 is means a discarded node.  For an internal node with
       no boundary conditions, bc_type[node_num] = 0.  If bc_type[node_num] != 0, then
       bc_type[node_num] += n is greater than n and this means there is overlap.

*/
			    if( i == 0) {
			       bc_type[node_num] += 19;       /* inflow in x -  */
			       if( bc_type[node_num] > 19 ) bc_type[node_num] = 100;
			    }
			    if( i == nx - 1) {
			       bc_type[node_num] += 14;       /* Free in x + */
			       if( bc_type[node_num] > 14 ) bc_type[node_num] = 100;
			    }
			    if( j == 0 ) {
			       bc_type[node_num] += 3;	/* symmetric in y - */
			       if( bc_type[node_num] > 3 ) bc_type[node_num] = 100;
			    }
			    if( j == ny - 1) {
			       bc_type[node_num] += 10;       /* far field in y + */
			       if( bc_type[node_num] > 10 ) bc_type[node_num] = 100;
			    }

			    check = meshBCset( (bc_type + node_num), text);
			    if(!check) printf( " Problems with meshBCset\n");
#if     0
/*     All the points below are when the nodes above overlap */

			    if( i == 0 && j == 0 )	    bc_type[node_num] = 100;       /* discarded */
			    if( i == 0 && k == 0 )	    bc_type[node_num] = 100;       /* discarded */
			    if( i == 0 && j == ny - 1 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == 0 && k == nz - 1 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && j == 0 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && k == 0 )       bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && j == ny - 1 )  bc_type[node_num] = 100;       /* discarded */
			    if( i == nx - 1 && k == nz - 1 )  bc_type[node_num] = 100;       /* discarded */
			    if( j == 0 && k == 0 )	    bc_type[node_num] = 100;       /* discarded */
			    if( j == 0 && k == nz - 1 )       bc_type[node_num] = 100;       /* discarded */
			    if( j == ny - 1 && k == 0 )	  bc_type[node_num] = 100;       /* discarded */
			    if( j == ny - 1 && k == nz - 1 )  bc_type[node_num] = 100;       /* discarded */
#endif

			    dns = dnsB;
			    temp = tempB;
			    prs = R*dns*temp;
			    vx = vlB; vy = vlB; vz = vlB;
			    enthal = R*gam*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);
			    ene = R*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);

			    fprintf( o1, "%4d %4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
				    i, j, k, text, dns, temp, prs, vx, vy, vz, enthal, ene);

			}
		    }
		}

		for  ( j=0; j < 7; j++) {
		    for ( k=0; k < nz; k++) {
			for ( i=0; i < nx; i++) {
				int node_num = nz*ny*i + nz*j + k;

				check = meshBCset( (bc_type + node_num), text);
				if(!check) printf( " Problems with meshBCset\n");

				dns = dnsA;
				temp = tempA;
				prs = R*dns*temp;
				vx = vlA; vy = vlB; vz = vlB;
				enthal = R*gam*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);
				ene = R*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy + vz*vz);

				fprintf( o1, "%4d %4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
				    i, j, k, text, dns, temp, prs, vx, vy, vz, enthal, ene);
			}
		    }
		}
	    }
	}
	
/* For the 2-D mesh */

	if(flag_2D)
	{
	    for ( i=0; i < nx; i++)
	    {
		for ( j=0; j < ny; j++)
		{
			node_num = ny*i + j;

/* Note that in assigning the boundary conditions below, I discard every node
   where there is overlap of boundary conditions.  This is why the if statements
   are of the form:

			if( i == 0) {
			   bc_type[node_num] += n;    
			   if( bc_type[node_num] > n ) bc_type[node_num] = 100;
			}

   where bc_type[node_num] = 100 is means a discarded node.  For an internal node with
   no boundary conditions, bc_type[node_num] = 0.  If bc_type[node_num] != 0, then
   bc_type[node_num] += n is greater than n and this means there is overlap.

*/
			if( i == 0) {
			   bc_type[node_num] += 19;       /* inflow in x -  */
			   if( bc_type[node_num] > 19 ) bc_type[node_num] = 100;
			}
			if( i == nx - 1) {
			   bc_type[node_num] += 14;       /* Free in x + */
			   if( bc_type[node_num] > 14 ) bc_type[node_num] = 100;
			}
			if( j == 0 ) {
			   bc_type[node_num] += 3;	/* symmetric in y - */
			   if( bc_type[node_num] > 3 ) bc_type[node_num] = 100;
			}
			if( j == ny - 1) {
			   bc_type[node_num] += 10;       /* far field in y + */
			   if( bc_type[node_num] > 10 ) bc_type[node_num] = 100;
			}

			check = meshBCset( (bc_type + node_num), text);
			if(!check) printf( " Problems with meshBCset\n");

			dns = dnsB;
			temp = tempB;
			prs = R*dns*temp;
			vx = vlB; vy = vlB;;
			enthal = R*gam*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy);
			ene = R*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy);

			fprintf( o1, "%4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
				i, j, text, dns, temp, prs, vx, vy, enthal, ene);

		}
	    }

	    for  ( j=0; j < 7; j++) {
		for ( i=0; i < nx; i++) {
		    int node_num = ny*i + j;

		    check = meshBCset( (bc_type + node_num), text);
		    if(!check) printf( " Problems with meshBCset\n");

		    dns = dnsA;
		    temp = tempA;
		    prs = R*dns*temp;
		    vx = vlA; vy = vlB;;
		    enthal = R*gam*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy);
		    ene = R*temp/(gam - 1.0) + 0.5*(vx*vx + vy*vy);

		    fprintf( o1, "%4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
			i, j, text, dns, temp, prs, vx, vy, enthal, ene);
		}
	    }
	}

	fprintf( o1, "%4d\n ",-10);

	free(bc_type);

	return 1;
}

