/* This program switches the i and j components of the nodes which has the
   effect of switching the x and y axis.  I wrote this so that I could
   use symmetry to help test and debug the problem.

   As I say in the main program "cfdqik.c":

   Most of the example files have flows running from left to right.  To see if the
   code would produce the same results with the fluid running from bottom to top,
   I transpose the quantities in the x and y directions. 

   When I run this code, I type:

      drev

   and give the input:

      a-15

   The output file is called: abcd.ofd
   which I later named ay-15.  So the two files to compare are:

           a-15    flow from left to right with a -15 degree angle of attack
           ay-15   flow from bottom to top with a 45 degree angle of attack

                     Updated 4/8/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "../AMR/amr.h"

int writer_2D( BOUND , MATL *, char *, INC_NODE *);

int bcset( BOUND *, int *, INC_NODE * );

int cfdMemory( double **, int , int **, int , MATL **, int , XYZF **, int , XYZI **, int ,
	XYZI2 **, int , XY_ZX_YZ **, int , INC_NODE **, int, SUBGRID **, int );

int Flowlines_File( FLOWLINES * );

int qikreader( MATL *, char *, INC_NODE *, FILE *, FLOWLINES, FLOWLINES );

int dof, nmat, numel, numnp, sof, nx, ny, nz, num_bodies;
double dx, dy, dz;
int flag_2D = 1, flag_3D = 0, print_flag = 0, debug_flag = 0, reverse_flag = 0;
double dt, time_current, err;         /* time step for time integration */
int vmag_read_flag = 0;
int nstep, cstep, ANGLE;
int imax, jmax;
double q, alp, Re;
double umax;
int umax_i, umax_j;
int stream_nstep = 700;


int main(int argc, char **argv)
{
	int i, i2, j, j2, k, n, n2, jxp, kyp, dum, dum2, node_num, node_num2;
	int check, name_length, counter, MemoryCounter, file_loc;
	int ce, ce2, xm1, xm2, xp1, xp2, xp3, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
	double time, time_final, length, width;
	double uxm1, vym1, pxm1, pxp1, pym1, pyp1;
	INC_NODE *node, *node1, *mem_INC_NODE;
	SUBGRID subgrid0, *mem_SUBGRID;
	XYZI *mem_XYZI;
	XYZI2 *mem_XYZI2; 
	XYZF *mem_XYZF;
	FLOWLINES streak, path;
	XY_ZX_YZ *mem_XY_ZX_YZ;
	int *mem_int, sofmi, sofmf, sofmXYZF, sofmXYZI, sofmXYZI2, sofmXY_ZX_YZ,
		sofmINC_NODE, sofmSUBGRID, ptr_inc;
	MATL *matl; 
	double fdum, fdum2, factor;
	double dns, temp, vx, vy, pres, ene, enthal, vmag;
	double *mem_double;
	double usq1, usq2, vsq1, vsq2, uv1, uvQ;
	double px, py; 
	char name[30], buf[ BUFSIZ ], *ccheck, one_char;
	FILE *o1, *o3, *o4;
	BOUND bc;

	sof = sizeof(double);

	nz = 1;

	printf("What is the name of the file containing the \n");
	printf("finite difference CFD data? \n");
	scanf( "%30s",name);

/* o1 contains all the structural data */

	o1 = fopen( name,"r" );
	if(o1 == NULL ) {
		printf("Can't find file %30s\n", name);
		exit(1);
	}

	fgets( buf, BUFSIZ, o1 );
	fscanf( o1, "%d %d %d", &numnp, &nx, &ny);
	imax = nx;
	jmax = ny;
	nmat = 1;

/* Check if there is additional coordinate z data and that this is a 3-D file.  */

	flag_3D = 0;
	flag_2D = 1;
	nz = 1;
	while(( one_char = (unsigned char) fgetc(o1)) != '\n')
	{
		if(one_char != ' ' )
		{
		    ungetc( one_char, o1);
		    fscanf( o1,"%d", &nz);
		    flag_3D = 1;
		    flag_2D = 0;
		    break;
		}
	}
	numnp = nx*ny*nz;
	dof=numnp;
	fscanf( o1,"\n");

/* Look for streakline input file */

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	memset(streak.name_in,0,30*sizeof(char));

	ccheck = strncpy(streak.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(streak.name_in+name_length, ".streak", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check if there is a streakline data file. */

/* Set the name of the input streakline file */

	memset(streak.name_in,0,30*sizeof(char));

	ccheck = strncpy(streak.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(streak.name_in+name_length, ".streak", 7);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the streakline input file as well as its properties */

	check = Flowlines_File( &streak );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

/* Set the name of the input pathline file */

	memset(path.name_in,0,30*sizeof(char));

	ccheck = strncpy(path.name_in, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(path.name_in+name_length, ".path", 5);
	if(!ccheck) printf( " Problems with strncpy \n");

/* Check for the existence of the pathline input file as well as its properties */

	check = Flowlines_File( &path );
	if(!ccheck) printf( " Problems with Flowlines_File \n");

	print_flag = 0;
	printf("%d\n", argc);
	if(argc > 2)
	{
		printf("usage %s -p\n", name);
		printf("Too many arguments.\n");
		exit(1);
	}
	if(argc == 2 )
	{
	    if( !strncmp(argv[1], "-p", 2))
	    {
		print_flag = 1;
	    }
	    else
	    {
		printf("usage %s -p only\n",name);
		exit(1);
	    }
	}

	if(flag_3D) printf( "total nodes:%d   nodes in x:%d nodes in y:%d nodes in z:%d\n", numnp, nx, ny, nz);
	else        printf( "total nodes:%d   nodes in x:%d nodes in y:%d\n", numnp, nx, ny);
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	fscanf( o1, "%lf %d %d %lf %lf\n", &dt, &nstep, &cstep, &time_current, &err);

	printf( "Time step:     %16.8e\n", dt);
	printf( "number of steps:  %4d\n", nstep);
	printf( "current step:     %4d\n", cstep);
	printf( "current time:  %16.8e\n", time_current);
	printf( "SOR tolerence: %16.8e\n", err);

	printf( "\n");

	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	time = 0.0;
	counter = 0;

	/* Establishing initial conditions for u,v, and pressure */

	dum = jmax;
	numnp = nx*ny;
	nmat = 1;
	streak.nstep_out = nstep;
	path.nstep_out = nstep;

/*   Begin allocation of meomory */

	MemoryCounter = 0;

/* For the doubles */
	sofmf = 1;
	MemoryCounter += sofmf*sizeof(double);
	printf( "\n Memory requrement for doubles is %15d bytes\n",MemoryCounter);

/* For the integers */
	sofmi= 2 + 2*numnp;
	MemoryCounter += sofmi*sizeof(int);
	printf( "\n Memory requrement for integers is %15d bytes\n",MemoryCounter);

/* Note is that the last value, 2*numnp, is based on the fact that some nodes
   may have more than 1 boundary condition, specifically the no-slip corner
   boundary which results in a node being no-slip in more than 1 direction.  So
   I can have, for instance:

      bc[0].noslp.xm[3] = 5;
      bc[0].noslp.ym[3] = 5;

   This potential for having more than one boundary condition is why I have allocated
   2*numnp.
*/

/* For the XYZF doubles */
	sofmXYZF = streak.num_pt0*streak.nstep_out + streak.num_pt0 +
		path.num_pt0 + path.num_pt0*path.nstep_out;
	MemoryCounter += sofmXYZF*sizeof(XYZF);
	printf( "\n Memory requrement for XYZF doubles is %15d bytes\n",MemoryCounter);

/* For the XYZI integers */
	sofmXYZI = 1;
	MemoryCounter += sofmXYZI*sizeof(XYZI);
	printf( "\n Memory requrement for XYZI integers is %15d bytes\n",MemoryCounter);

/* For the XYZI2 integers */
	sofmXYZI2=7;
	MemoryCounter += sofmXYZI2*sizeof(XYZI2);
	printf( "\n Memory requrement for XYZI2 integers is %15d bytes\n",MemoryCounter);

/* For the XY_ZX_YZ integers */
	sofmXY_ZX_YZ=1;
	MemoryCounter += sofmXY_ZX_YZ*sizeof(XY_ZX_YZ);
	printf( "\n Memory requrement for XY_ZX_YZ integers is %15d bytes\n",MemoryCounter);

/* For the INC_NODE structs */
	sofmINC_NODE=2*numnp + 1;
	MemoryCounter += sofmINC_NODE*sizeof(INC_NODE);
	printf( "\n Memory requrement for INC_NODE integers is %15d bytes\n",MemoryCounter);

/* For the SUBGRID structs */
	sofmSUBGRID=1;
	MemoryCounter += sofmSUBGRID*sizeof(SUBGRID);
	printf( "\n Memory requrement for SUBGRID integers is %15d bytes\n",MemoryCounter);

	check = cfdMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat, &mem_XYZF, sofmXYZF,
		&mem_XYZI, sofmXYZI, &mem_XYZI2, sofmXYZI2, &mem_XY_ZX_YZ, sofmXY_ZX_YZ, &mem_INC_NODE,
		sofmINC_NODE, &mem_SUBGRID, sofmSUBGRID );
	if(!check) printf( " Problems with cfdMemory\n");

/* For the INC_NODE structs*/
	                                      ptr_inc = 0;
	node=(mem_INC_NODE+ptr_inc);          ptr_inc += numnp;
	node1=(mem_INC_NODE+ptr_inc);         ptr_inc += numnp;

/* For the integers */
	                                      ptr_inc = 0;
	bc.num_none=(mem_int+ptr_inc);        ptr_inc += 1;
	bc.num_prs0=(mem_int+ptr_inc);        ptr_inc += 1;

/* Note that bcset below has to be corrected for mem_int so that it has the correct
   offset for mem_int, i.e.:

        check = bcset( &bc, (mem_int+2), node);
*/

/* For the XYZF doubles */
	                                      ptr_inc = 0;
	streak.pt=(mem_XYZF+ptr_inc);         ptr_inc += streak.num_pt0*streak.nstep_out;
	streak.pt0=(mem_XYZF+ptr_inc);        ptr_inc += streak.num_pt0;
	path.pt0=(mem_XYZF+ptr_inc);          ptr_inc += path.num_pt0;
	path.pt=(mem_XYZF+ptr_inc);           ptr_inc += path.num_pt0*path.nstep_out;

/* For the XYZI2 integers */
	                                      ptr_inc = 0;
	bc.num_symm=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_far=(mem_XYZI2+ptr_inc);       ptr_inc += 1;
	bc.num_free=(mem_XYZI2+ptr_inc);      ptr_inc += 1;
	bc.num_inflow=(mem_XYZI2+ptr_inc);    ptr_inc += 1;
	bc.num_frestr=(mem_XYZI2+ptr_inc);    ptr_inc += 1;
	bc.num_free2=(mem_XYZI2+ptr_inc);     ptr_inc += 1;
	bc.num_noslp=(mem_XYZI2+ptr_inc);     ptr_inc += 1;

/* For the XY_ZX_YZ integers */
	                                          ptr_inc = 0;
	bc.num_noslp_prs=(mem_XY_ZX_YZ+ptr_inc);  ptr_inc += 1;

	for( i = 0; i < numnp; ++i )
	{
	     node[i].bc_type = 0;
	}

/* Below is the initial condition for the pressure */

	for( i = 0; i < imax; ++i )
	{
		for( j = 0; j < jmax; ++j )
		{
		   ce  = jmax*i + j;
		   node[ce].prs = 0.0;
		}
	}

	check = qikreader( matl, name, node, o1, path, streak );
	if(!check) printf( " Problems with qikreader\n");

	for( i = 0; i < nx; ++i )
	{
	    for( j = 0; j < ny; ++j )
	    {
		ce = ny*i + j;
		node[ce].vx = node[ce].vx;
		node[ce].vy = node[ce].vy;
		node[ce].vxQ = node[ce].vx;
		node[ce].vyQ = node[ce].vy;
		node[ce].prs = node[ce].prs;
		/*printf(  "llll %16.8e %16.8e %16.8e %4d\n", node[ce].vx, node[ce].vy, node[ce].prs, ce);*/
	    }
	}

/* Establishing initial conditions for u,v, and pressure */

	Re = matl[0].Re;
	matl[0].gam = 1.0;

	q = 0.375;
	alp = 1.0/Re;

/* Set the boundary condition variable, bc, as well as allocate memory for each individual
   boundary condition.  
*/

	check = bcset( &bc, (mem_int+2), node);
	if(!check) printf( " Problems with bcset\n");

	o3 = fopen( "abcd.streak","w" );
	o4 = fopen( "abcd.path","w" );

	if(streak.read_flag);
	{
	    if(streak.new_flag)
	    {
		fprintf( o3, "number of streaklines\n", streak.num_pt0);
		fprintf( o3, "%4d\n", streak.num_pt0);
		fprintf( o3, "streakline number, initial x, initial y\n", streak.num_pt0);
		for( i = 0; i < streak.num_pt0; ++i )
		{
			printf( "%4d %16.8e %16.8e\n", i, streak.pt0[i].x, streak.pt0[i].y);
			fprintf( o3, "%4d %16.8e %16.8e\n", i, streak.pt0[i].y, streak.pt0[i].x);
		}
	    }
	    else
	    {
		for( i = 0; i < streak.nstep_in; ++i )
		{
		    for( j = 0; j < streak.num_pt0; ++j )
		    {
			dum = streak.num_pt0*i + j;
			fprintf( o3, "%9.5f %9.5f ", streak.pt[dum].y, streak.pt[dum].x);
		    }
		    fprintf( o3, "\n");
		}
	    }
	}


	if(path.read_flag);
	{
	    if(path.new_flag)
	    {
		fprintf( o4, "number of pathlines\n", path.num_pt0);
		fprintf( o4, "%4d\n", path.num_pt0);
		fprintf( o4, "pathline number, initial x, initial y\n", path.num_pt0);
		for( i = 0; i < path.num_pt0; ++i )
		{
			printf( "%4d %16.8e %16.8e\n", i, path.pt0[i].x, path.pt0[i].y);
			fprintf( o4, "%4d %16.8e %16.8e\n", i, path.pt0[i].y, path.pt0[i].x);
		}
	    }
	    else
	    {
		for( i = 0; i < path.nstep_in; ++i )
		{
		    for( j = 0; j < path.num_pt0; ++j )
		    {
			dum = path.num_pt0*i + j;
			fprintf( o4, "%9.5f %9.5f ", path.pt[dum].y, path.pt[dum].x);
		    }
		    fprintf( o4, "\n");
		}
	    }
	}

/* I define subgrid0 to be the original coarsest mesh. */

	subgrid0.jstart = 0;
	subgrid0.jend = jmax-1;
	subgrid0.istart = 0;
	subgrid0.iend = imax-1;
	subgrid0.dx = dx;
	subgrid0.dy = dy;
	subgrid0.dt = dt;
	subgrid0.nx = nx;
	subgrid0.ny = ny;
	subgrid0.refine_num = 0;

/* Transpose the data */

	for( i = 0; i < nx; ++i )
	{
	    for( j = 0; j < ny; ++j )
	    {
		node_num = ny*i + j;
		node_num2 = nx*j + i;

		/*(node1 + node_num2) = (node + node_num);*/

		node1[node_num2].j = i;
		node1[node_num2].i = j;
		node1[node_num2].k = 0;

		node1[node_num2].vy         = node[node_num].vx;
		node1[node_num2].vx         = node[node_num].vy;
		node1[node_num2].vz         = node[node_num].vz;
		node1[node_num2].prs        = node[node_num].prs;
		node1[node_num2].vmag       = node[node_num].vmag;
		dum = node[node_num].bc_type%6;
		dum2 = 0;
		if(dum == 1 || dum == 2) dum2 = 2;
		if(dum == 3 || dum == 4) dum2 = -2;
		node1[node_num2].bc_type    = node[node_num].bc_type + dum2;
	    }
	}

	node1[0].bc_type = 63;

	dum = nx;
	dum2 = ny;
	nx = dum2;
	ny = dum;

	memset(name,0,30*sizeof(char));
	ccheck = strncpy(name, "abcd", 4);
	if(!ccheck) printf( " Problems with strncpy \n");

	check = writer_2D(bc, matl, name, node1);
	if(!check) printf( " Problems with writer_2D\n");

	fclose(o1);
	fclose(o3);
	fclose(o4);

	return 1;
}


