/*
    This library function reads in data for the program which
    uses the finite difference method to calculate a compressible CFD flow
    field.

		Updated 4/31/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2005, 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"

#if INCOMPRESSIBLE
#define NODE    INC_NODE
#endif
#if GUI
#define NODE    GR_NODE
#endif

extern int dof, numel, numnp, sof, nx, ny, nz, num_bodies;
extern int flag_2D, flag_3D, print_flag;
extern int vmag_read_flag;
extern double dx, dy, dz;
extern double dt, time_current;
extern int nstep, cstep;

int qikreader( MATL *matl, char *name, NODE *node, FILE *o1, FLOWLINES path, FLOWLINES streak,
	FLOWLINES stream)
{
	int i, j, k, dum, dum1, dum2, dum3, name_length, check, file_loc;
	int node_num, nsid_read_flag, num_pt0;
	double dns, temp, vx, vy, vz, prs, ene, enthal, vmag;
	double fdum, fdum1, fdum2, fdum3, fdum4, fdum5, fdum6, fdum7, fdum8, fdum9, fdum10;
	char *ccheck, one_char;
	char buf[ BUFSIZ ];
	char text[6], bc_text[6];

	memset(bc_text, 0, 6*sizeof(char));

	if(flag_3D)
	{
	    printf( "spatial discretization dx, dy, dz\n");
	    fscanf( o1, "%lf %lf %lf\n", &dx, &dy, &dz);
	    printf( "%16.8e %16.8e %16.8e\n", dx, dy, dz);
	}
	else
	{
	    printf( "spatial discretization dx, dy\n");
	    fscanf( o1, "%lf %lf\n", &dx, &dy);
	    printf( "%16.8e %16.8e\n", dx, dy);
	    dz = 1.0;
	}
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	printf( "Reynolds Number\n");
	fscanf( o1, "%lf\n", &matl[0].Re);
	printf( " %7.3e\n", matl[0].Re);
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	num_bodies = 0;

/* For the 3D mesh */

	if(flag_3D)
	{
	    if(print_flag) printf( "node              bc        prs          vx           vy ");

	    file_loc = ftell(o1);

	    fscanf( o1,"%d", &i);
	    if( i > -10 )
	    {

		fscanf( o1,"%d %d", &j, &k);

		fscanf( o1,"%6s %lf %lf %lf %lf", bc_text,
		    &prs, &vx, &vy, &vz);

		vmag_read_flag = 0;
		while(( one_char = (unsigned char) fgetc(o1)) != '\n')
		{
		    if(one_char != ' ' )
		    {
			ungetc( one_char, o1);
			fscanf( o1,"%lf", &vmag);
			vmag_read_flag = 1;
			break;
		    }
		}

		fseek(o1, file_loc, 0);

		fscanf( o1,"%d",&i);
	    }
	    if( !vmag_read_flag )
	    {
		if(print_flag) printf( "\n");
		while( i > -1 )
		{
		    fscanf( o1,"%d %d", &j, &k);
		    node_num = i*ny*nz + j*nz + k;
		    node[node_num].i = i; node[node_num].j = j; node[node_num].k = k;
		    if(print_flag) printf( "node (%6d)   ", node_num);
		    fscanf( o1,"%6s ", bc_text);
		    if(print_flag) printf( "%6s ", bc_text);

/* Record the boundary condition type. */

		    check = strncmp(bc_text, "none", 4); if(!check) node[node_num].bc_type = 0;   /* None */
		    check = strncmp(bc_text, "smxm", 4); if(!check) node[node_num].bc_type = 1;   /* symmetric in x - */
		    check = strncmp(bc_text, "smxp", 4); if(!check) node[node_num].bc_type = 2;   /* symmetric in x + */
		    check = strncmp(bc_text, "smym", 4); if(!check) node[node_num].bc_type = 3;   /* symmetric in y - */
		    check = strncmp(bc_text, "smyp", 4); if(!check) node[node_num].bc_type = 4;   /* symmetric in y + */
		    check = strncmp(bc_text, "smzm", 4); if(!check) node[node_num].bc_type = 5;   /* symmetric in z - */
		    check = strncmp(bc_text, "smzp", 4); if(!check) node[node_num].bc_type = 6;   /* symmetric in z + */
		    check = strncmp(bc_text, "ffxm", 4); if(!check) node[node_num].bc_type = 7;   /* far field in x - */
		    check = strncmp(bc_text, "ffxp", 4); if(!check) node[node_num].bc_type = 8;   /* far field in x + */
		    check = strncmp(bc_text, "ffym", 4); if(!check) node[node_num].bc_type = 9;   /* far field in y - */
		    check = strncmp(bc_text, "ffyp", 4); if(!check) node[node_num].bc_type = 10;  /* far field in y + */
		    check = strncmp(bc_text, "ffzm", 4); if(!check) node[node_num].bc_type = 11;  /* far field in z - */
		    check = strncmp(bc_text, "ffzp", 4); if(!check) node[node_num].bc_type = 12;  /* far field in z + */
		    check = strncmp(bc_text, "frxm", 4); if(!check) node[node_num].bc_type = 13;  /* free in x - */
		    check = strncmp(bc_text, "frxp", 4); if(!check) node[node_num].bc_type = 14;  /* free in x + */
		    check = strncmp(bc_text, "frym", 4); if(!check) node[node_num].bc_type = 15;  /* free in y - */
		    check = strncmp(bc_text, "fryp", 4); if(!check) node[node_num].bc_type = 16;  /* free in y + */
		    check = strncmp(bc_text, "frzm", 4); if(!check) node[node_num].bc_type = 17;  /* free in z - */
		    check = strncmp(bc_text, "frzp", 4); if(!check) node[node_num].bc_type = 18;  /* free in z + */
		    check = strncmp(bc_text, "ifxm", 4); if(!check) node[node_num].bc_type = 19;  /* inflow in x - */
		    check = strncmp(bc_text, "ifxp", 4); if(!check) node[node_num].bc_type = 20;  /* inflow in x + */
		    check = strncmp(bc_text, "ifym", 4); if(!check) node[node_num].bc_type = 21;  /* inflow in y - */
		    check = strncmp(bc_text, "ifyp", 4); if(!check) node[node_num].bc_type = 22;  /* inflow in y + */
		    check = strncmp(bc_text, "ifzm", 4); if(!check) node[node_num].bc_type = 23;  /* inflow in z - */
		    check = strncmp(bc_text, "ifzp", 4); if(!check) node[node_num].bc_type = 24;  /* inflow in z + */
		    check = strncmp(bc_text, "fsxm", 4); if(!check) node[node_num].bc_type = 25;  /* freestream in x - */
		    check = strncmp(bc_text, "fsxp", 4); if(!check) node[node_num].bc_type = 26;  /* freestream in x + */
		    check = strncmp(bc_text, "fsym", 4); if(!check) node[node_num].bc_type = 27;  /* freestream in y - */
		    check = strncmp(bc_text, "fsyp", 4); if(!check) node[node_num].bc_type = 28;  /* freestream in y + */
		    check = strncmp(bc_text, "fszm", 4); if(!check) node[node_num].bc_type = 29;  /* freestream in z - */
		    check = strncmp(bc_text, "fszp", 4); if(!check) node[node_num].bc_type = 30;  /* freestream in z + */
		    check = strncmp(bc_text, "fr2xm", 5); if(!check) node[node_num].bc_type = 31; /* free 2 in x - */
		    check = strncmp(bc_text, "fr2xp", 5); if(!check) node[node_num].bc_type = 32; /* free 2 in x + */
		    check = strncmp(bc_text, "fr2ym", 5); if(!check) node[node_num].bc_type = 33; /* free 2 in y - */
		    check = strncmp(bc_text, "fr2yp", 5); if(!check) node[node_num].bc_type = 34; /* free 2 in y + */
		    check = strncmp(bc_text, "fr2zm", 5); if(!check) node[node_num].bc_type = 35; /* free 2 in z - */
		    check = strncmp(bc_text, "fr2zp", 5); if(!check) node[node_num].bc_type = 36; /* free 2 in z + */

		    check = strncmp(bc_text, "nsxm", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 37;  /* noslip in x - */
		    }

		    check = strncmp(bc_text, "nsxp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 38;  /* noslip in x + */
		    }

		    check = strncmp(bc_text, "nsym", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 39;  /* noslip in y - */
		    }

		    check = strncmp(bc_text, "nsyp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 40;  /* noslip in y + */
		    }

		    check = strncmp(bc_text, "nszm", 4); if(!check) node[node_num].bc_type = 41;  /* noslip in z - */
		    check = strncmp(bc_text, "nszp", 4); if(!check) node[node_num].bc_type = 42;  /* noslip in z + */

/* Corner nodes */
		    check = strncmp(bc_text, "nsxmym", 6); if(!check) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
		    check = strncmp(bc_text, "nsxmyp", 6); if(!check) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
		    check = strncmp(bc_text, "nsxpym", 6); if(!check) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
		    check = strncmp(bc_text, "nsxpyp", 6); if(!check) node[node_num].bc_type = 46;  /* corner noslip in x + y + */

		    if(node[node_num].bc_type > 36 && node[node_num].bc_type < 63)
		    {

/* I test whether a body number is given for the noslip node.  If there is no body id given, then
   it is assumed that all the noslip nodes are associated with 1 body.
*/ 
			file_loc = ftell(o1);

			fscanf( o1,"%lf %lf %lf %lf",
				&fdum, &fdum2, &fdum3, &fdum4);

			nsid_read_flag = 0;
			while(( one_char = (unsigned char) fgetc(o1)) != '\n')
			{
			    if(one_char != ' ' )
			    {
				ungetc( one_char, o1);
				fscanf( o1,"%lf", &fdum5);
				nsid_read_flag = 1;
				break;
			    }
			}

			fseek(o1, file_loc, 0);

			if( nsid_read_flag == 1 )
			{
				fscanf( o1,"%3d", &node[node_num].nsid);
				if(print_flag) printf( "%3d", node[node_num].nsid);
				if(num_bodies < node[node_num].nsid) num_bodies = node[node_num].nsid;
			}
			else
			{
				node[node_num].nsid = 1;
				if(print_flag) printf( "%3d", node[node_num].nsid);
				num_bodies = 1;
			}
		    }
		    else if(print_flag) printf( "   ");

		    fscanf( o1,"%lf %lf %lf %lf\n%d",
			&node[node_num].prs, &node[node_num].vx, &node[node_num].vy, &node[node_num].vz, &i);
		    if(print_flag) printf( "%12.4e %12.4e %12.4e %12.4e\n",
			node[node_num].prs, node[node_num].vx, node[node_num].vy, node[node_num].vz);

/* Record the boundary condition type. */

		    check = strncmp(bc_text, "prs0", 4); if(!check) node[node_num].bc_type = 63;  /* pressure set to 0.0 */
		    check = strncmp(bc_text, "dis", 3);  if(!check) node[node_num].bc_type = 100; /* discarded */
		}
		if(print_flag) printf( "\n\n");
	    }
	    else
	    {
		if(print_flag) printf( "          vmag\n");
		while( i > -1 )
		{
		    fscanf( o1,"%d %d", &j, &k);
		    node_num = i*ny*nz + j*nz + k;
		    node[node_num].i = i; node[node_num].j = j; node[node_num].k = k;
		    if(print_flag) printf( "node (%6d)   ", node_num);
		    fscanf( o1,"%6s ", bc_text);
		    if(print_flag) printf( "%6s ", bc_text);

/* Record the boundary condition type. */

		    check = strncmp(bc_text, "none", 4); if(!check) node[node_num].bc_type = 0;   /* None */
		    check = strncmp(bc_text, "smxm", 4); if(!check) node[node_num].bc_type = 1;   /* symmetric in x - */
		    check = strncmp(bc_text, "smxp", 4); if(!check) node[node_num].bc_type = 2;   /* symmetric in x + */
		    check = strncmp(bc_text, "smym", 4); if(!check) node[node_num].bc_type = 3;   /* symmetric in y - */
		    check = strncmp(bc_text, "smyp", 4); if(!check) node[node_num].bc_type = 4;   /* symmetric in y + */
		    check = strncmp(bc_text, "smzm", 4); if(!check) node[node_num].bc_type = 5;   /* symmetric in z - */
		    check = strncmp(bc_text, "smzp", 4); if(!check) node[node_num].bc_type = 6;   /* symmetric in z + */
		    check = strncmp(bc_text, "ffxm", 4); if(!check) node[node_num].bc_type = 7;   /* far field in x - */
		    check = strncmp(bc_text, "ffxp", 4); if(!check) node[node_num].bc_type = 8;   /* far field in x + */
		    check = strncmp(bc_text, "ffym", 4); if(!check) node[node_num].bc_type = 9;   /* far field in y - */
		    check = strncmp(bc_text, "ffyp", 4); if(!check) node[node_num].bc_type = 10;  /* far field in y + */
		    check = strncmp(bc_text, "ffzm", 4); if(!check) node[node_num].bc_type = 11;  /* far field in z - */
		    check = strncmp(bc_text, "ffzp", 4); if(!check) node[node_num].bc_type = 12;  /* far field in z + */
		    check = strncmp(bc_text, "frxm", 4); if(!check) node[node_num].bc_type = 13;  /* free in x - */
		    check = strncmp(bc_text, "frxp", 4); if(!check) node[node_num].bc_type = 14;  /* free in x + */
		    check = strncmp(bc_text, "frym", 4); if(!check) node[node_num].bc_type = 15;  /* free in y - */
		    check = strncmp(bc_text, "fryp", 4); if(!check) node[node_num].bc_type = 16;  /* free in y + */
		    check = strncmp(bc_text, "frzm", 4); if(!check) node[node_num].bc_type = 17;  /* free in z - */
		    check = strncmp(bc_text, "frzp", 4); if(!check) node[node_num].bc_type = 18;  /* free in z + */
		    check = strncmp(bc_text, "ifxm", 4); if(!check) node[node_num].bc_type = 19;  /* inflow in x - */
		    check = strncmp(bc_text, "ifxp", 4); if(!check) node[node_num].bc_type = 20;  /* inflow in x + */
		    check = strncmp(bc_text, "ifym", 4); if(!check) node[node_num].bc_type = 21;  /* inflow in y - */
		    check = strncmp(bc_text, "ifyp", 4); if(!check) node[node_num].bc_type = 22;  /* inflow in y + */
		    check = strncmp(bc_text, "ifzm", 4); if(!check) node[node_num].bc_type = 23;  /* inflow in z - */
		    check = strncmp(bc_text, "ifzp", 4); if(!check) node[node_num].bc_type = 24;  /* inflow in z + */
		    check = strncmp(bc_text, "fsxm", 4); if(!check) node[node_num].bc_type = 25;  /* freestream in x - */
		    check = strncmp(bc_text, "fsxp", 4); if(!check) node[node_num].bc_type = 26;  /* freestream in x + */
		    check = strncmp(bc_text, "fsym", 4); if(!check) node[node_num].bc_type = 27;  /* freestream in y - */
		    check = strncmp(bc_text, "fsyp", 4); if(!check) node[node_num].bc_type = 28;  /* freestream in y + */
		    check = strncmp(bc_text, "fszm", 4); if(!check) node[node_num].bc_type = 29;  /* freestream in z - */
		    check = strncmp(bc_text, "fszp", 4); if(!check) node[node_num].bc_type = 30;  /* freestream in z + */
		    check = strncmp(bc_text, "fr2xm", 5); if(!check) node[node_num].bc_type = 31; /* free 2 in x - */
		    check = strncmp(bc_text, "fr2xp", 5); if(!check) node[node_num].bc_type = 32; /* free 2 in x + */
		    check = strncmp(bc_text, "fr2ym", 5); if(!check) node[node_num].bc_type = 33; /* free 2 in y - */
		    check = strncmp(bc_text, "fr2yp", 5); if(!check) node[node_num].bc_type = 34; /* free 2 in y + */
		    check = strncmp(bc_text, "fr2zm", 5); if(!check) node[node_num].bc_type = 35; /* free 2 in z - */
		    check = strncmp(bc_text, "fr2zp", 5); if(!check) node[node_num].bc_type = 36; /* free 2 in z + */

		    check = strncmp(bc_text, "nsxm", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 37;  /* noslip in x - */
		    }

		    check = strncmp(bc_text, "nsxp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 38;  /* noslip in x + */
		    }

		    check = strncmp(bc_text, "nsym", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 39;  /* noslip in y - */
		    }

		    check = strncmp(bc_text, "nsyp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 40;  /* noslip in y + */
		    }

		    check = strncmp(bc_text, "nszm", 4); if(!check) node[node_num].bc_type = 41;  /* noslip in z - */
		    check = strncmp(bc_text, "nszp", 4); if(!check) node[node_num].bc_type = 42;  /* noslip in z + */
/* Corner nodes */
		    check = strncmp(bc_text, "nsxmym", 6); if(!check) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
		    check = strncmp(bc_text, "nsxmyp", 6); if(!check) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
		    check = strncmp(bc_text, "nsxpym", 6); if(!check) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
		    check = strncmp(bc_text, "nsxpyp", 6); if(!check) node[node_num].bc_type = 46;  /* corner noslip in x + y + */

		    if(node[node_num].bc_type > 36 && node[node_num].bc_type < 63)
		    {

/* I test whether a body number is given for the noslip node.  If there is no body id given, then
   it is assumed that all the noslip nodes are associated with 1 body.
*/ 
			file_loc = ftell(o1);

			fscanf( o1,"%lf %lf %lf %lf %lf",
				&fdum, &fdum2, &fdum3,
				&fdum4, &fdum5);

			nsid_read_flag = 0;
			while(( one_char = (unsigned char) fgetc(o1)) != '\n')
			{
			    if(one_char != ' ' )
			    {
				ungetc( one_char, o1);
				fscanf( o1,"%lf", &fdum6);
				nsid_read_flag = 1;
				break;
			    }
			}

			fseek(o1, file_loc, 0);

			if( nsid_read_flag == 1 )
			{
				fscanf( o1,"%3d", &node[node_num].nsid);
				if(print_flag) printf( "%3d", node[node_num].nsid);
				if(num_bodies < node[node_num].nsid) num_bodies = node[node_num].nsid;
			}
			else
			{
				node[node_num].nsid = 1;
				if(print_flag) printf( "%3d", node[node_num].nsid);
				num_bodies = 1;
			}
		    }
		    else if(print_flag) printf( "   ");

		    check = strncmp(bc_text, "prs0", 4); if(!check) node[node_num].bc_type = 63;  /* pressure set to 0.0 */
		    check = strncmp(bc_text, "dis", 3);  if(!check) node[node_num].bc_type = 100; /* discarded */

		    fscanf( o1,"%lf %lf %lf %lf %lf\n%d",
			&node[node_num].prs, &node[node_num].vx, &node[node_num].vy, &node[node_num].vz,
			&node[node_num].vmag, &i);
		    if(print_flag) printf( "%12.4e %12.4e %12.4e %12.4e %12.4e\n",
			node[node_num].prs, node[node_num].vx, node[node_num].vy, node[node_num].vz, node[node_num].vmag);
		}
		if(print_flag) printf( "\n\n");
	    }
	}

/* For the 2D mesh */

	if(flag_2D)
	{
	    if(print_flag) printf( "node              bc        prs          vx           vy ");

	    file_loc = ftell(o1);

	    fscanf( o1,"%d", &i);
	    if( i > -10 )
	    {

		fscanf( o1,"%d", &j);

		fscanf( o1,"%6s %lf %lf %lf", bc_text, &prs, &vx, &vy);

		vmag_read_flag = 0;
		while(( one_char = (unsigned char) fgetc(o1)) != '\n')
		{
		    if(one_char != ' ' )
		    {
			ungetc( one_char, o1);
			fscanf( o1,"%lf", &vmag);
			vmag_read_flag = 1;
			break;
		    }
		}

		fseek(o1, file_loc, 0);

		fscanf( o1,"%d",&i);
	    }
	    if( !vmag_read_flag )
	    {
		if(print_flag) printf( "\n");
		while( i > -1 )
		{
		    fscanf( o1,"%d", &j);
		    node_num = i*ny + j;
		    node[node_num].i = i; node[node_num].j = j; node[node_num].k = 0;
		    if(print_flag) printf( "node (%6d)   ", node_num);
		    node[node_num].vz = 0.0;
		    fscanf( o1,"%6s ", bc_text);
		    if(print_flag) printf( "%6s ", bc_text);

/* Record the boundary condition type. */

		    check = strncmp(bc_text, "none", 4); if(!check) node[node_num].bc_type = 0;   /* None */
		    check = strncmp(bc_text, "smxm", 4); if(!check) node[node_num].bc_type = 1;   /* symmetric in x - */
		    check = strncmp(bc_text, "smxp", 4); if(!check) node[node_num].bc_type = 2;   /* symmetric in x + */
		    check = strncmp(bc_text, "smym", 4); if(!check) node[node_num].bc_type = 3;   /* symmetric in y - */
		    check = strncmp(bc_text, "smyp", 4); if(!check) node[node_num].bc_type = 4;   /* symmetric in y + */
		    check = strncmp(bc_text, "ffxm", 4); if(!check) node[node_num].bc_type = 7;   /* far field in x - */
		    check = strncmp(bc_text, "ffxp", 4); if(!check) node[node_num].bc_type = 8;   /* far field in x + */
		    check = strncmp(bc_text, "ffym", 4); if(!check) node[node_num].bc_type = 9;   /* far field in y - */
		    check = strncmp(bc_text, "ffyp", 4); if(!check) node[node_num].bc_type = 10;  /* far field in y + */
		    check = strncmp(bc_text, "frxm", 4); if(!check) node[node_num].bc_type = 13;  /* free in x - */
		    check = strncmp(bc_text, "frxp", 4); if(!check) node[node_num].bc_type = 14;  /* free in x + */
		    check = strncmp(bc_text, "frym", 4); if(!check) node[node_num].bc_type = 15;  /* free in y - */
		    check = strncmp(bc_text, "fryp", 4); if(!check) node[node_num].bc_type = 16;  /* free in y + */
		    check = strncmp(bc_text, "ifxm", 4); if(!check) node[node_num].bc_type = 19;  /* inflow in x - */
		    check = strncmp(bc_text, "ifxp", 4); if(!check) node[node_num].bc_type = 20;  /* inflow in x + */
		    check = strncmp(bc_text, "ifym", 4); if(!check) node[node_num].bc_type = 21;  /* inflow in y - */
		    check = strncmp(bc_text, "ifyp", 4); if(!check) node[node_num].bc_type = 22;  /* inflow in y + */
		    check = strncmp(bc_text, "fsxm", 4); if(!check) node[node_num].bc_type = 25;  /* freestream in x - */
		    check = strncmp(bc_text, "fsxp", 4); if(!check) node[node_num].bc_type = 26;  /* freestream in x + */
		    check = strncmp(bc_text, "fsym", 4); if(!check) node[node_num].bc_type = 27;  /* freestream in y - */
		    check = strncmp(bc_text, "fsyp", 4); if(!check) node[node_num].bc_type = 28;  /* freestream in y + */
		    check = strncmp(bc_text, "fr2xm", 5); if(!check) node[node_num].bc_type = 31; /* free 2 in x - */
		    check = strncmp(bc_text, "fr2xp", 5); if(!check) node[node_num].bc_type = 32; /* free 2 in x + */
		    check = strncmp(bc_text, "fr2ym", 5); if(!check) node[node_num].bc_type = 33; /* free 2 in y - */
		    check = strncmp(bc_text, "fr2yp", 5); if(!check) node[node_num].bc_type = 34; /* free 2 in y + */

		    check = strncmp(bc_text, "nsxm", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 37;  /* noslip in x - */
		    }

		    check = strncmp(bc_text, "nsxp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 38;  /* noslip in x + */
		    }

		    check = strncmp(bc_text, "nsym", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 39;  /* noslip in y - */
		    }

		    check = strncmp(bc_text, "nsyp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 40;  /* noslip in y + */
		    }

/* Corner nodes */
		    check = strncmp(bc_text, "nsxmym", 6); if(!check) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
		    check = strncmp(bc_text, "nsxmyp", 6); if(!check) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
		    check = strncmp(bc_text, "nsxpym", 6); if(!check) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
		    check = strncmp(bc_text, "nsxpyp", 6); if(!check) node[node_num].bc_type = 46;  /* corner noslip in x + y + */

		    if(node[node_num].bc_type > 36 && node[node_num].bc_type < 63)
		    {

/* I test whether a body number is given for the noslip node.  If there is no body id given, then
   it is assumed that all the noslip nodes are associated with 1 body.
*/ 
			file_loc = ftell(o1);

			fscanf( o1,"%lf %lf %lf %lf %lf %lf %lf",
				&fdum, &fdum2, &fdum3);

			nsid_read_flag = 0;
			while(( one_char = (unsigned char) fgetc(o1)) != '\n')
			{
			    if(one_char != ' ' )
			    {
				ungetc( one_char, o1);
				fscanf( o1,"%lf", &fdum4);
				nsid_read_flag = 1;
				break;
			    }
			}

			fseek(o1, file_loc, 0);

			if( nsid_read_flag == 1 )
			{
				fscanf( o1,"%3d", &node[node_num].nsid);
				if(print_flag) printf( "%3d", node[node_num].nsid);
				if(num_bodies < node[node_num].nsid) num_bodies = node[node_num].nsid;
			}
			else
			{
				node[node_num].nsid = 1;
				if(print_flag) printf( "%3d", node[node_num].nsid);
				num_bodies = 1;
			}
		    }
		    else if(print_flag) printf( "   ");

		    fscanf( o1,"%lf %lf %lf\n%d",
			&node[node_num].prs, &node[node_num].vx, &node[node_num].vy, &i);
			node[node_num].vz = 0.0;
		    if(print_flag) printf( "%12.4e %12.4e %12.4e\n",
			node[node_num].prs, node[node_num].vx, node[node_num].vy);

		    check = strncmp(bc_text, "prs0", 4); if(!check) node[node_num].bc_type = 63;  /* pressure set to 0.0 */
		    check = strncmp(bc_text, "dis", 3);  if(!check) node[node_num].bc_type = 100; /* discarded */
		}
		if(print_flag) printf( "\n\n");
	    }
	    else
	    {
		if(print_flag) printf( "          vmag\n");
		while( i > -1 )
		{
		    fscanf( o1,"%d", &j);
		    node_num = i*ny + j;
		    node[node_num].i = i; node[node_num].j = j; node[node_num].k = 0;
		    if(print_flag) printf( "node (%6d)   ", node_num);
		    node[node_num].vz = 0.0;
		    fscanf( o1,"%6s ", bc_text);
		    if(print_flag) printf( "%6s ", bc_text);

/* Record the boundary condition type. */

		    check = strncmp(bc_text, "none", 4); if(!check) node[node_num].bc_type = 0;   /* None */
		    check = strncmp(bc_text, "smxm", 4); if(!check) node[node_num].bc_type = 1;   /* symmetric in x - */
		    check = strncmp(bc_text, "smxp", 4); if(!check) node[node_num].bc_type = 2;   /* symmetric in x + */
		    check = strncmp(bc_text, "smym", 4); if(!check) node[node_num].bc_type = 3;   /* symmetric in y - */
		    check = strncmp(bc_text, "smyp", 4); if(!check) node[node_num].bc_type = 4;   /* symmetric in y + */
		    check = strncmp(bc_text, "ffxm", 4); if(!check) node[node_num].bc_type = 7;   /* far field in x - */
		    check = strncmp(bc_text, "ffxp", 4); if(!check) node[node_num].bc_type = 8;   /* far field in x + */
		    check = strncmp(bc_text, "ffym", 4); if(!check) node[node_num].bc_type = 9;   /* far field in y - */
		    check = strncmp(bc_text, "ffyp", 4); if(!check) node[node_num].bc_type = 10;  /* far field in y + */
		    check = strncmp(bc_text, "frxm", 4); if(!check) node[node_num].bc_type = 13;  /* free in x - */
		    check = strncmp(bc_text, "frxp", 4); if(!check) node[node_num].bc_type = 14;  /* free in x + */
		    check = strncmp(bc_text, "frym", 4); if(!check) node[node_num].bc_type = 15;  /* free in y - */
		    check = strncmp(bc_text, "fryp", 4); if(!check) node[node_num].bc_type = 16;  /* free in y + */
		    check = strncmp(bc_text, "ifxm", 4); if(!check) node[node_num].bc_type = 19;  /* inflow in x - */
		    check = strncmp(bc_text, "ifxp", 4); if(!check) node[node_num].bc_type = 20;  /* inflow in x + */
		    check = strncmp(bc_text, "ifym", 4); if(!check) node[node_num].bc_type = 21;  /* inflow in y - */
		    check = strncmp(bc_text, "ifyp", 4); if(!check) node[node_num].bc_type = 22;  /* inflow in y + */
		    check = strncmp(bc_text, "fsxm", 4); if(!check) node[node_num].bc_type = 25;  /* freestream in x - */
		    check = strncmp(bc_text, "fsxp", 4); if(!check) node[node_num].bc_type = 26;  /* freestream in x + */
		    check = strncmp(bc_text, "fsym", 4); if(!check) node[node_num].bc_type = 27;  /* freestream in y - */
		    check = strncmp(bc_text, "fsyp", 4); if(!check) node[node_num].bc_type = 28;  /* freestream in y + */
		    check = strncmp(bc_text, "fr2xm", 5); if(!check) node[node_num].bc_type = 31; /* free 2 in x - */
		    check = strncmp(bc_text, "fr2xp", 5); if(!check) node[node_num].bc_type = 32; /* free 2 in x + */
		    check = strncmp(bc_text, "fr2ym", 5); if(!check) node[node_num].bc_type = 33; /* free 2 in y - */
		    check = strncmp(bc_text, "fr2yp", 5); if(!check) node[node_num].bc_type = 34; /* free 2 in y + */

		    check = strncmp(bc_text, "nsxm", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 37;  /* noslip in x - */
		    }

		    check = strncmp(bc_text, "nsxp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 39) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 40) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 38;  /* noslip in x + */
		    }

		    check = strncmp(bc_text, "nsym", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 39;  /* noslip in y - */
		    }

		    check = strncmp(bc_text, "nsyp", 4);
		    if(!check) {
			if(node[node_num].bc_type == 37) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
			if(node[node_num].bc_type == 38) node[node_num].bc_type = 46;  /* corner noslip in x + y + */
			if(node[node_num].bc_type == 0) node[node_num].bc_type = 40;  /* noslip in y + */
		    }

/* Corner nodes */
		    check = strncmp(bc_text, "nsxmym", 6); if(!check) node[node_num].bc_type = 43;  /* corner noslip in x - y - */
		    check = strncmp(bc_text, "nsxmyp", 6); if(!check) node[node_num].bc_type = 44;  /* corner noslip in x - y + */
		    check = strncmp(bc_text, "nsxpym", 6); if(!check) node[node_num].bc_type = 45;  /* corner noslip in x + y - */
		    check = strncmp(bc_text, "nsxpyp", 6); if(!check) node[node_num].bc_type = 46;  /* corner noslip in x + y + */

		    if(node[node_num].bc_type > 36 && node[node_num].bc_type < 63)
		    {

/* I test whether a body number is given for the noslip node.  If there is no body id given, then
   it is assumed that all the noslip nodes are associated with 1 body.
*/ 
			file_loc = ftell(o1);

			fscanf( o1,"%lf %lf %lf %lf", &fdum, &fdum2, &fdum3, &fdum4);

			nsid_read_flag = 0;
			while(( one_char = (unsigned char) fgetc(o1)) != '\n')
			{
			    if(one_char != ' ' )
			    {
				ungetc( one_char, o1);
				fscanf( o1,"%lf", &fdum5);
				nsid_read_flag = 1;
				break;
			    }
			}

			fseek(o1, file_loc, 0);

			if( nsid_read_flag == 1 )
			{
				fscanf( o1,"%3d", &node[node_num].nsid);
				if(print_flag) printf( "%3d", node[node_num].nsid);
				if(num_bodies < node[node_num].nsid) num_bodies = node[node_num].nsid;
			}
			else
			{
				node[node_num].nsid = 1;
				if(print_flag) printf( "%3d", node[node_num].nsid);
				num_bodies = 1;
			}
		    }
		    else if(print_flag) printf( "   ");

		    check = strncmp(bc_text, "prs0", 4); if(!check) node[node_num].bc_type = 63;  /* pressure set to 0.0 */
		    check = strncmp(bc_text, "dis", 3);  if(!check) node[node_num].bc_type = 100; /* discarded */

		    fscanf( o1,"%lf %lf %lf %lf\n%d",
			&node[node_num].prs, &node[node_num].vx, &node[node_num].vy, &node[node_num].vmag, &i);
			node[node_num].vz = 0.0;
		    if(print_flag) printf( "%12.4e %12.4e %12.4e %12.4e\n",
			node[node_num].prs, node[node_num].vx, node[node_num].vy, node[node_num].vmag);

		}
		if(print_flag) printf( "\n\n");
	    }
	}


/* Read in the pathline data */

	num_pt0 = path.num_pt0;

	if(path.read_flag)
	{
	    if(path.new_flag)
	    {
		fgets( buf, BUFSIZ, path.file_in );
		for( i = 0; i < num_pt0; ++i )
		{
		    fscanf( path.file_in, "%d ", &dum);
		    fscanf( path.file_in, "%lf %lf\n", &path.pt0[dum].x, &path.pt0[dum].y);
		    /*printf( "%4d %16.8e %16.8e\n", dum, path.pt0[dum].x, path.pt0[dum].y);*/
		}
	    }
	    else
	    {
		for( i = 0; i < path.nstep_in; ++i )
		{
		    for( j = 0; j < num_pt0; ++j )
		    {
			dum = num_pt0*i + j;
			fscanf( path.file_in, "%lf %lf", &path.pt[dum].x, &path.pt[dum].y);
			/*printf( "%9.4f %9.4f", path.pt[dum].x, path.pt[dum].y);*/
		    }
		    fscanf( path.file_in, "\n");
		    /*printf( "\n");*/
		}

/* I still need to set the starting point of the pathlines given by pathline0 */

		for( j = 0; j < num_pt0; ++j )
		{
		    path.pt0[j].x = path.pt[num_pt0*(path.nstep_in-1) + j].x;
		    path.pt0[j].y = path.pt[num_pt0*(path.nstep_in-1) + j].y;
		}

	    }
	}



/* Read in the streakline data */

	num_pt0 = streak.num_pt0;

	if(streak.read_flag)
	{
	    if(streak.new_flag)
	    {
		fgets( buf, BUFSIZ, streak.file_in );
		for( i = 0; i < num_pt0; ++i )
		{
		    fscanf( streak.file_in, "%d ", &dum);
		    fscanf( streak.file_in, "%lf %lf\n", &streak.pt0[dum].x, &streak.pt0[dum].y);
		    /*printf( "%4d %16.8e %16.8e\n", dum, streak.pt0[dum].x, streak.pt0[dum].y);*/
		}
	    }
	    else
	    {
		for( i = 0; i < streak.nstep_in; ++i )
		{
		    for( j = 0; j < num_pt0; ++j )
		    {
			dum = num_pt0*i + j;
			fscanf( streak.file_in, "%lf %lf", &streak.pt[dum].x, &streak.pt[dum].y);
			/*printf( "%9.4f %9.4f", streak.pt[dum].x, streak.pt[dum].y);*/
		    }
		    fscanf( streak.file_in, "\n");
		    /*printf( "\n");*/
		}

/* I still need to set the starting point of the streaklines given by streakline0 */

		for( j = 0; j < num_pt0; ++j )
		{
		    streak.pt0[j].x = streak.pt[num_pt0*(streak.nstep_in-1) + j].x;
		    streak.pt0[j].y = streak.pt[num_pt0*(streak.nstep_in-1) + j].y;
		}

	    }
	}


/* Read in the streamline data */

	num_pt0 = stream.num_pt0;

	if(stream.read_flag)
	{
	    if(stream.new_flag)
	    {
		fgets( buf, BUFSIZ, stream.file_in );
		for( i = 0; i < num_pt0; ++i )
		{
		    fscanf( stream.file_in, "%d ", &dum);
		    fscanf( stream.file_in, "%lf %lf\n", &stream.pt0[dum].x, &stream.pt0[dum].y);
		    /*printf( "%4d %16.8e %16.8e\n", dum, stream.pt0[dum].x, stream.pt0[dum].y);*/
		}
	    }
	    else
	    {
		for( i = 0; i < stream.nstep_in; ++i )
		{
		    for( j = 0; j < num_pt0; ++j )
		    {
			dum = num_pt0*i + j;
			fscanf( stream.file_in, "%lf %lf", &stream.pt[dum].x, &stream.pt[dum].y);
			/*printf( "%9.4f %9.4f", stream.pt[dum].x, stream.pt[dum].y);*/
		    }
		    fscanf( stream.file_in, "\n");
		    /*printf( "\n");*/
		}

/* I still need to set the starting point of the streamlines given by streamline0 */

		for( j = 0; j < num_pt0; ++j )
		{
		    stream.pt0[j].x = stream.pt[num_pt0*(stream.nstep_in-1) + j].x;
		    stream.pt0[j].y = stream.pt[num_pt0*(stream.nstep_in-1) + j].y;
		}

	    }
	}

	return 1;
}

