/* This library function writes the resulting data for the program which
   uses the finite difference method to calculate an incompressible CFD flow
   field. 

                     Updated 4/7/08

    SLFCFD source file
    Version:  1.2
    Copyright (C) 2005  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"

#define SMALL   1.0e-20

extern int sof, nx, ny, nz;
extern int flag_2D, flag_3D;
extern double dx, dy, dz;
extern double dt, time_current, err;
extern int nstep, cstep;

int meshBCset( int *, char *);

int writer_2D (BOUND bc, MATL *matl, char *name, INC_NODE *node)
{
	int i, j, k, dum, node_num, numnp, bc_type, check, name_length;
	char *ccheck;
	int dum2,dum3,dum4;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, prs, ene, enthal, vmag;
	double gam, Re;
	char text[10], out[30];
	FILE *o3;

/* Every output file is named after the input file with
   a ".ofd" extension */

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	memset(out,0,30*sizeof(char));

	ccheck = strncpy(out, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(out+name_length, ".ofd", 4);
	if(!ccheck) printf( " Problems with strncpy \n");

	o3 = fopen( out,"w" );
	if(o3 == NULL ) {
		printf("Can't find file %30s\n", out); 
		exit(1);
	} 

	gam = matl[0].gam;
	Re = matl[0].Re;

	numnp = nx*ny;

	fprintf( o3, "total nodes, nodes in x, nodes in y for the shock bubble\n");
	fprintf( o3, "%4d    %4d %4d\n", numnp, nx, ny);
	fprintf( o3, "Time step, number of steps, current step, current time, SOR tolerence\n");
	fprintf( o3, "%16.8e %7d %7d %16.8e %16.8e\n", dt, nstep, cstep, time_current, err);
	fprintf( o3, "spatial discretization dx, dy\n");
	fprintf( o3, "%16.8e %16.8e\n", dx, dy);
	fprintf( o3, "Reynolds Number\n");
	fprintf( o3, "%16.8e\n", Re);

	fprintf( o3, "   i    j    bc           pressure     velocity x");
	fprintf( o3, "   velocity y   velocity magnitude\n");

	dum= 0;
	for ( node_num = 0; node_num < numnp; node_num++)
	{
		dns    = 0.0;
		temp   = 0.0;
		vx     = node[node_num].vx;
		vy     = node[node_num].vy;
		prs    = node[node_num].prs;
		ene    = 0.0;
		enthal = 0.0;
		vmag   = node[node_num].vmag;
		bc_type = node[node_num].bc_type;

		i = node[node_num].i;
		j = node[node_num].j;

		memset(text, 0, 10*sizeof(char));

		check = meshBCset( &bc_type, text);
		if(!check) printf( " Problems with meshBCset\n");

		fprintf( o3, "%4d %4d  %10s", i, j, text); 

		if(bc_type > 36 && bc_type < 63)
		    fprintf( o3, "%3d", node[node_num].nsid); 
		else
		    fprintf( o3, "   "); 

		fprintf( o3, "%12.4e %12.4e %12.4e %12.4e\n",
			prs, vx, vy, vmag);


		//fprintf(o3, "%4d %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e\n",
		//  dum, temp, vx, vy, prs, ene, dns); 
		//fprintf(o3, "%4d %4d %4d %14.6e %14.6e %14.6e\n",
		//    i, j, dum, fdum1, temp, dns); 
		/*fprintf(o3, "%4d %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n",
			dum, fdum1, temp, dns, prs, ene, enthal);*/
		/*fprintf(o3, "%4d %4d %4d %4d %4d %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n",
			dum, i, j, node_num, bc_type, fdum1, temp, dns, prs, ene, enthal); */
	}

	fprintf( o3, "%4d ",-10);

	fclose(o3);

	return 1;
}


int writer_SLFFEA_2D (BOUND bc, MATL *matl, INC_NODE *node)
{
	int i, j, k, dum, node_num;
	int dof, nmat, numel, numed, numnp, plane_stress_flag;
	int dum2,dum3,dum4;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, vz, prs, ene, enthal;
	double gam, Re;
	double side_length, side_width, el_length, el_width;
	int length_el_num, width_el_num, el_type;
	FILE *o1;

	o1 = fopen( "rect","w" );

	gam = matl[0].gam;
	Re = matl[0].Re;

	nmat = 1;
	plane_stress_flag = 1;
	numel = (nx - 1)*(ny - 1);
	numnp = nx*ny;
	el_length = dx;
	el_width = dy;

    // ofstream fout("ouput");

    // San: Print initial conditions for internal nodes(of nodeList) from 1/3(ymax) < y < ymax


	fprintf( o1, "   numel numnp nmat nmode plane_stress_flag");
	fprintf( o1, "  This is the rectangular waveguide problem \n ");
	fprintf( o1, "    %4d %4d %4d %4d %4d \n ",
	    numel,numnp,nmat,0,plane_stress_flag);

	fprintf( o1, "matl no., E modulus, Poisson Ratio, density \n");
	for( i = 0; i < nmat; ++i )
	{
	    fprintf( o1, "%3d ",0);
	    fprintf( o1, " %9.4f %9.4f     %9.4f \n ",10000000.0, 0.3, 2.77e3);
	}

	fprintf( o1, "el no., connectivity, matl no \n");

	dum = 0;
	for( i = 0; i < ny - 1; ++i )
	{
		for( j = 0; j < nx - 1; ++j )
		{
			fprintf( o1, "%4d ",dum);
			fprintf( o1, " %4d %4d %4d %4d",i*nx + j,
				i*nx+j+1, (i+1)*nx+j+1,
				(i+1)*nx+j);
			fprintf( o1, "%4d ",0);
			fprintf( o1, "\n");
			++dum;
		}
	}

	dum = 0;
	fprintf( o1, "node no., coordinates \n");
	for( i = 0; i < ny; ++i )
	{
		for( j = 0; j < nx; ++j )
		{
			fdum1 = el_length*((double)j);
			fdum2 = el_width*((double)i);
			fprintf( o1, "%d ",dum);
			fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
			fprintf( o1, "\n");
			++dum;
		}
	}

	dum= 0;
	fprintf( o1, "prescribed displacement x: node  disp value\n");
	for( i = 0; i < ny; ++i )
	{
	    dum = (nx+1)*i;
	    fprintf( o1, "%4d %14.6e\n",dum,0.0);
	}
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "prescribed displacement y: node  disp value\n");
	for( i = 0; i < nx; ++i )
	{
	    fprintf( o1, "%4d %14.6e\n",i,0.0);
	}
	fprintf( o1, "%4d\n ",-10);

	dum = 0;
	fprintf( o1, "node with point load and load vector in x,y\n");
	for( k = 0; k < 1; ++k )
	{
	    for( j = 0; j < ny; ++j )
	    {
		for( i = 0; i < nx; ++i )
		{
		    node_num = i*ny + j;

		    dns    = 0.0;
		    temp   = 0.0;
		    vx     = node[node_num].vx;
		    vy     = node[node_num].vy;
		    prs    = node[node_num].prs;
		    ene    = 0.0;
		    enthal = 0.0;

		    fdum1 = (vx*vx + vy*vy);
		    fdum1 = sqrt(fdum1) + SMALL;
		    vx /= fdum1;
		    vy /= fdum1;
		    //fprintf(o1, "%4d %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e\n",
		    //  dum, temp, vx, vy, vz, prs, ene, dns); 
		    //fprintf(o1, "%4d %4d %4d %14.6e %14.6e %14.6e\n",
		    //    i, j, dum, fdum1, temp, dns); 
		    // printf( "%4d %14.6e %14.6e\n", dum, vx, vy); 
		    if(fdum1 > 0.000001) fprintf(o1, "%4d %14.6e %14.6e\n", dum, vx, vy); 
		    ++dum;
		}
	    }
	}
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "node no. with vmag, temp, dns\n");

	dum= 0;
	for ( k=0; k < 1; k++)
	{
	  for ( j=0; j < ny; j++)
	  {
	    for ( i=0; i < nx; i++)
	    {
		node_num = i*ny + j;

		dns    = 0.0;
		temp   = 0.0;
		vx     = node[node_num].vx;
		vy     = node[node_num].vy;
		prs    = node[node_num].prs;
		ene    = 0.0;
		enthal = 0.0;

		fdum1 = (vx*vx + vy*vy)/(Re*gam*temp);
		fdum1 = sqrt(fdum1);
		/* fprintf(o1, "%4d %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e\n",
		  dum, temp, vx, vy, prs, ene, dns); 
		fprintf(o1, "%4d %4d %4d %14.6e %14.6e %14.6e\n",
		    i, j, dum, fdum1, temp, dns); */
		/*fprintf(o1, "%4d %4d %4d %14.6e %14.6e %14.6e\n", i, j, dum, fdum1, temp, dns);*/
		fprintf(o1, "%4d %14.6e %14.6e %14.6e\n", dum, fdum1, temp, dns);
		++dum;
	    }
	  }
	}
	fprintf( o1, "%4d ",-10);

	fclose(o1);

	return 1;
}


